@@ -70,8 +70,8 @@ ConduitDataCollection::~ConduitDataCollection()
 void ConduitDataCollection::Save()
 {
    std::string dir_name = MeshDirectoryName();
-   int err = create_directory(dir_name, mesh, myid);
-   if (err)
+   int err_ = create_directory(dir_name, mesh, myid);
+   if (err_)
    {
       MFEM_ABORT("Error creating directory: " << dir_name);
    }
@@ -88,6 +88,7 @@ void ConduitDataCollection::Save()
                  << verify_info.to_json());
    }
 
+   // wrap all grid functions
    FieldMapConstIterator itr;
    for ( itr = field_map.begin(); itr != field_map.end(); itr++)
    {
@@ -103,6 +104,16 @@ void ConduitDataCollection::Save()
       }
    }
 
+   // wrap all quadrature functions
+   QFieldMapConstIterator qf_itr;
+   for ( qf_itr = q_field_map.begin(); qf_itr != q_field_map.end(); qf_itr++)
+   {
+      std::string name = qf_itr->first;
+      QuadratureFunction *qf = qf_itr->second;
+      QuadratureFunctionToBlueprintField(qf,
+                                         n_mesh["fields"][name]);
+   }
+
    // save mesh data
    SaveMeshAndFields(myid,
                      n_mesh,
@@ -157,6 +168,16 @@ ConduitDataCollection::SetProtocol(const std::string &protocol)
    relay_protocol = protocol;
 }
 
+// Conduit data type id for the MFEM precision
+constexpr conduit::index_t mfem_precision_conduit_id =
+#if defined(MFEM_USE_DOUBLE)
+   CONDUIT_NATIVE_DOUBLE_ID;
+#elif defined(MFEM_USE_SINGLE)
+   CONDUIT_NATIVE_FLOAT_ID;
+#else
+#error Unknown MFEM precision
+#endif
+
 //------------------------------
 // begin static public methods
 //------------------------------
@@ -206,42 +227,41 @@ ConduitDataCollection::BlueprintMeshToMesh(const Node &n_mesh,
    // get the number of points
    int num_verts = n_coordset_vals[0].dtype().number_of_elements();
    // get vals for points
-   const double *verts_ptr = NULL;
+   const real_t *verts_ptr = NULL;
 
    // the mfem mesh constructor needs coords with interleaved (aos) type
-   // ordering, even for 1d + 2d we always need 3 doubles b/c it uses
-   // Array<Vertex> and Vertex is a pod of 3 doubles. we check for this
+   // ordering, even for 1d + 2d we always need 3 real_t (double/float) b/c it
+   // uses Array<Vertex> and Vertex is a pod of 3 real_t. we check for this
    // case, if we don't have it we convert the data
 
    if (ndims == 3 &&
-       n_coordset_vals[0].dtype().is_double() &&
+       n_coordset_vals[0].dtype().id() == mfem_precision_conduit_id &&
        blueprint::mcarray::is_interleaved(n_coordset_vals) )
    {
-      // already interleaved mcarray of 3 doubles,
+      // already interleaved mcarray of 3 real_t (double/float),
       // return ptr to beginning
       verts_ptr = n_coordset_vals[0].value();
    }
    else
    {
       Node n_tmp;
-      // check all vals, if we don't have doubles convert
-      // to doubles
+      // check all vals, if we don't have real_t (double/float) convert
+      // to real_t
       NodeConstIterator itr = n_coordset_vals.children();
       while (itr.has_next())
       {
          const Node &c_vals = itr.next();
          std::string c_name = itr.name();
 
-         if ( c_vals.dtype().is_double() )
+         if ( c_vals.dtype().id() == mfem_precision_conduit_id )
          {
             // zero copy current coords
             n_tmp[c_name].set_external(c_vals);
-
          }
          else
          {
             // convert
-            c_vals.to_double_array(n_tmp[c_name]);
+            c_vals.to_data_type(mfem_precision_conduit_id, n_tmp[c_name]);
          }
       }
 
@@ -250,13 +270,13 @@ ConduitDataCollection::BlueprintMeshToMesh(const Node &n_mesh,
       if (ndims < 3)
       {
          // add dummy z
-         n_tmp["z"].set(DataType::c_double(num_verts));
+         n_tmp["z"].set(DataType(mfem_precision_conduit_id, num_verts));
       }
 
       if (ndims < 2)
       {
          // add dummy y
-         n_tmp["y"].set(DataType::c_double(num_verts));
+         n_tmp["y"].set(DataType(mfem_precision_conduit_id, num_verts));
       }
 
       Node &n_conv_coords_vals = n_conv["coordsets"][coords_name]["values"];
@@ -452,7 +472,7 @@ ConduitDataCollection::BlueprintMeshToMesh(const Node &n_mesh,
    //       if nodes gf is attached later, it resets the space dim based
    //       on the gf's fes.
    Mesh *mesh = new Mesh(// from coordset
-      const_cast<double*>(verts_ptr),
+      const_cast<real_t*>(verts_ptr),
       num_verts,
       // from topology
       const_cast<int*>(elem_indices),
@@ -519,7 +539,7 @@ ConduitDataCollection::BlueprintFieldToGridFunction(Mesh *mesh,
    // can't return a gf that zero copies the conduit data
    Node n_conv;
 
-   const double *vals_ptr = NULL;
+   const real_t *vals_ptr = NULL;
 
    int vdim = 1;
 
@@ -529,10 +549,10 @@ ConduitDataCollection::BlueprintFieldToGridFunction(Mesh *mesh,
    {
       vdim = n_field["values"].number_of_children();
 
-      // need to check that we have doubles and
+      // need to check that we have real_t (double/float) and
       // cover supported layouts
 
-      if ( n_field["values"][0].dtype().is_double() )
+      if ( n_field["values"][0].dtype().id() == mfem_precision_conduit_id )
       {
          // check for contig
          if (n_field["values"].is_contiguous())
@@ -556,27 +576,26 @@ ConduitDataCollection::BlueprintFieldToGridFunction(Mesh *mesh,
             vals_ptr = n_conv["values"].child(0).value();
          }
       }
-      else // convert to doubles and use contig
+      else // convert to real_t (double/float) and use contig
       {
          Node n_tmp;
-         // check all vals, if we don't have doubles convert
-         // to doubles
+         // check all vals, if we don't have real_t (double/float) convert
+         // to real_t
          NodeConstIterator itr = n_field["values"].children();
          while (itr.has_next())
          {
             const Node &c_vals = itr.next();
             std::string c_name = itr.name();
 
-            if ( c_vals.dtype().is_double() )
+            if ( c_vals.dtype().id() == mfem_precision_conduit_id )
             {
                // zero copy current coords
                n_tmp[c_name].set_external(c_vals);
-
             }
             else
             {
                // convert
-               c_vals.to_double_array(n_tmp[c_name]);
+               c_vals.to_data_type(mfem_precision_conduit_id, n_tmp[c_name]);
             }
          }
 
@@ -589,14 +608,15 @@ ConduitDataCollection::BlueprintFieldToGridFunction(Mesh *mesh,
    }
    else
    {
-      if (n_field["values"].dtype().is_double() &&
+      if (n_field["values"].dtype().id() == mfem_precision_conduit_id &&
           n_field["values"].is_compact())
       {
          vals_ptr = n_field["values"].value();
       }
       else
       {
-         n_field["values"].to_double_array(n_conv["values"]);
+         n_field["values"].to_data_type(mfem_precision_conduit_id,
+                                        n_conv["values"]);
          vals_ptr = n_conv["values"].value();
       }
    }
@@ -620,14 +640,14 @@ ConduitDataCollection::BlueprintFieldToGridFunction(Mesh *mesh,
 
    if (zero_copy)
    {
-      res = new GridFunction(fes,const_cast<double*>(vals_ptr));
+      res = new GridFunction(fes,const_cast<real_t*>(vals_ptr));
    }
    else
    {
       // copy case, this constructor will alloc the space for the GF data
       res = new GridFunction(fes);
       // create an mfem vector that wraps the conduit data
-      Vector vals_vec(const_cast<double*>(vals_ptr),fes->GetVSize());
+      Vector vals_vec(const_cast<real_t*>(vals_ptr),fes->GetVSize());
       // copy values into the result
       (*res) = vals_vec;
    }
@@ -639,6 +659,155 @@ ConduitDataCollection::BlueprintFieldToGridFunction(Mesh *mesh,
    return res;
 }
 
+//---------------------------------------------------------------------------//
+mfem::QuadratureFunction *
+ConduitDataCollection::BlueprintFieldToQuadratureFunction(Mesh *mesh,
+                                                          const Node &n_field,
+                                                          bool zero_copy)
+{
+   // n_conv holds converted data (when necessary for mfem api)
+   // if n_conv is used ( !n_conv.dtype().empty() ) we
+   // know that some data allocation was necessary, so we
+   // can't return a qf that zero copies the conduit data
+   Node n_conv;
+
+   const real_t *vals_ptr = NULL;
+   int vdim = 1;
+
+   if (n_field["values"].dtype().is_object())
+   {
+      vdim = n_field["values"].number_of_children();
+
+      // need to check that we have real_t (double/float) and
+      // cover supported layouts
+      if ( n_field["values"][0].dtype().id() == mfem_precision_conduit_id )
+      {
+         // quad funcs use what mfem calls byVDIM
+         // and what conduit calls interleaved
+         // check for interleaved
+         if (blueprint::mcarray::is_interleaved(n_field["values"]))
+         {
+            // conduit mcarray interleaved == mfem byVDIM
+            vals_ptr = n_field["values"].child(0).value();
+         }
+         else
+         {
+            // for mcarray generic case --  default to byVDIM
+            // aka interleaved
+            blueprint::mcarray::to_interleaved(n_field["values"],
+                                               n_conv["values"]);
+            vals_ptr = n_conv["values"].child(0).value();
+         }
+      }
+      else // convert to real_t (double/float) and use interleaved
+      {
+         Node n_tmp;
+         // check all vals, if we don't have real_t (double/float) convert
+         // to real_t
+         NodeConstIterator itr = n_field["values"].children();
+         while (itr.has_next())
+         {
+            const Node &c_vals = itr.next();
+            std::string c_name = itr.name();
+
+            if ( c_vals.dtype().id() == mfem_precision_conduit_id )
+            {
+               // zero copy current coords
+               n_tmp[c_name].set_external(c_vals);
+            }
+            else
+            {
+               // convert
+               c_vals.to_data_type(mfem_precision_conduit_id, n_tmp[c_name]);
+            }
+         }
+
+         // for mcarray generic case --  default to byVDIM
+         // aka interleaved
+         blueprint::mcarray::to_interleaved(n_tmp,
+                                            n_conv["values"]);
+         vals_ptr = n_conv["values"].child(0).value();
+      }
+   }
+   else // scalar case
+   {
+      if (n_field["values"].dtype().id() == mfem_precision_conduit_id &&
+          n_field["values"].is_compact())
+      {
+         vals_ptr = n_field["values"].value();
+      }
+      else
+      {
+         n_field["values"].to_data_type(mfem_precision_conduit_id,
+                                        n_conv["values"]);
+         vals_ptr = n_conv["values"].value();
+      }
+   }
+
+   if (zero_copy && !n_conv.dtype().is_empty())
+   {
+      //Info: "Cannot zero-copy since data conversions were necessary"
+      zero_copy = false;
+   }
+
+   // we need basis name to create the proper mfem quad space and quad func
+   // the pattern used to encode the quad space params is:
+   // QF_{ORDER}_{VDIM}
+   // ORDER is the degree of the polynomials for the quad rule
+   // VDIM  is the number of components at each quad point (scalar, vector, etc)
+
+   int qf_order = 0;
+   int qf_vdim  = 0;
+   std::string qf_name = n_field["basis"].as_string();
+   const char *qf_name_cstr = qf_name.c_str();
+   if (!strncmp(qf_name_cstr, "QF_", 3))
+   {
+      // parse {ORDER}
+      qf_order = atoi(qf_name_cstr + 3);
+      // find second `_`
+      const char *qf_vdim_cstr = strstr(qf_name_cstr+3,"_");
+      if (qf_vdim_cstr == NULL)
+      {
+         MFEM_ABORT("Error parsing quadrature function description string: "
+                    << qf_name << std::endl
+                    << "Expected: QF_{ORDER}_{VDIM}");
+      }
+      // parse {VDIM}
+      qf_vdim  = atoi(qf_vdim_cstr+1);
+   }
+   else
+   {
+      MFEM_ABORT("Error parsing quadrature function description string: "
+                 << qf_name << std::endl
+                 << "Expected: QF_{ORDER}_{VDIM}");
+   }
+   MFEM_VERIFY(qf_vdim == vdim, "vector dimension mismatch: vdim = " << vdim
+               << ", qf_vdim = " << qf_vdim);
+
+   mfem::QuadratureSpace *quad_space = new mfem::QuadratureSpace(mesh, qf_order);
+   mfem::QuadratureFunction *res = new mfem::QuadratureFunction();
+
+   if (zero_copy)
+   {
+      res->SetSpace(quad_space, const_cast<real_t*>(vals_ptr), vdim);
+      res->SetOwnsSpace(true);
+   }
+   else
+   {
+      res->SetSpace(quad_space, vdim);
+      res->SetOwnsSpace(true);
+      // copy case, this constructor will alloc the space for the quad data
+      // create an mfem vector that wraps the conduit data
+      Vector vals_vec(const_cast<real_t*>(vals_ptr),res->Size());
+      // copy values into the result
+      (*res) = vals_vec;
+   }
+
+   return res;
+}
+
+
+
 //---------------------------------------------------------------------------//
 void
 ConduitDataCollection::MeshToBlueprintMesh(Mesh *mesh,
@@ -656,20 +825,20 @@ ConduitDataCollection::MeshToBlueprintMesh(Mesh *mesh,
    // Setup main coordset
    ////////////////////////////////////////////
 
-   // Assumes  mfem::Vertex has the layout of a double array.
+   // Assumes  mfem::Vertex has the layout of a real_t (double/float) array.
 
-   // this logic assumes an mfem vertex is always 3 doubles wide
+   // this logic assumes an mfem vertex is always 3 real_t (double/float) wide
    int stride = sizeof(mfem::Vertex);
    int num_vertices = mesh->GetNV();
 
-   MFEM_ASSERT( ( stride == 3 * sizeof(double) ),
+   MFEM_ASSERT( ( stride == 3 * sizeof(real_t) ),
                 "Unexpected stride for Vertex");
 
    Node &n_mesh_coords = n_mesh["coordsets"][coordset_name];
    n_mesh_coords["type"] =  "explicit";
 
 
-   double *coords_ptr = mesh->GetVertex(0);
+   real_t *coords_ptr = mesh->GetVertex(0);
 
    n_mesh_coords["values/x"].set_external(coords_ptr,
                                           num_vertices,
@@ -680,14 +849,14 @@ ConduitDataCollection::MeshToBlueprintMesh(Mesh *mesh,
    {
       n_mesh_coords["values/y"].set_external(coords_ptr,
                                              num_vertices,
-                                             sizeof(double),
+                                             sizeof(real_t),
                                              stride);
    }
    if (dim >= 3)
    {
       n_mesh_coords["values/z"].set_external(coords_ptr,
                                              num_vertices,
-                                             sizeof(double) * 2,
+                                             sizeof(real_t) * 2,
                                              stride);
    }
 
@@ -912,7 +1081,7 @@ ConduitDataCollection::GridFunctionToBlueprintField(mfem::GridFunction *gf,
 
    if (vdim == 1) // scalar case
    {
-      n_field["values"].set_external(gf->GetData(),
+      n_field["values"].set_external(const_cast<real_t *>(gf->HostRead()),
                                      ndofs);
    }
    else // vector case
@@ -925,23 +1094,76 @@ ConduitDataCollection::GridFunctionToBlueprintField(mfem::GridFunction *gf,
       int vdim_stride  = (ordering == Ordering::byNODES ? ndofs : 1);
 
       index_t offset = 0;
-      index_t stride = sizeof(double) * entry_stride;
+      index_t stride = sizeof(real_t) * entry_stride;
 
       for (int d = 0;  d < vdim; d++)
       {
          std::ostringstream oss;
          oss << "v" << d;
          std::string comp_name = oss.str();
-         n_field["values"][comp_name].set_external(gf->GetData(),
+         n_field["values"][comp_name].set_external(const_cast<real_t *>(gf->HostRead()),
                                                    ndofs,
                                                    offset,
                                                    stride);
-         offset +=  sizeof(double) * vdim_stride;
+         offset +=  sizeof(real_t) * vdim_stride;
       }
    }
 
 }
 
+//---------------------------------------------------------------------------//
+void
+ConduitDataCollection::QuadratureFunctionToBlueprintField(
+   mfem::QuadratureFunction *qf,
+   Node &n_field,
+   const std::string &main_topology_name)
+{
+   // For quadrature functions, use basis pattern:
+   //   QF_{ORDER}_{VDIM}
+
+   int qf_vdim  = qf->GetVDim();
+   int qf_order = qf->GetSpace()->GetOrder();
+   int qf_size  = qf->GetSpace()->GetSize();
+
+   {
+      std::ostringstream oss;
+      oss << "QF_" << qf_order << "_" << qf_vdim;
+
+      n_field["basis"] = oss.str();
+      n_field["topology"] = main_topology_name;
+   }
+
+   if (qf_vdim == 1) // scalar case
+   {
+      n_field["values"].set_external(const_cast<real_t *>(qf->HostRead()),
+                                     qf_size);
+   }
+   else // vector case
+   {
+      // deal with striding of all components
+      // quadrature functions are always byVDIM
+      // or what conduit calls interleaved
+
+      index_t offset = 0;
+      index_t stride = sizeof(real_t) * qf_vdim;
+
+      for (int d = 0;  d < qf_vdim; d++)
+      {
+         std::ostringstream oss;
+         oss << "v" << d;
+         std::string comp_name = oss.str();
+         n_field["values"][comp_name].set_external(const_cast<real_t *>(qf->HostRead()),
+                                                   qf_size,
+                                                   offset,
+                                                   stride);
+         offset += sizeof(real_t);
+      }
+   }
+
+}
+
+
+
 //------------------------------
 // end static public methods
 //------------------------------
@@ -967,7 +1189,7 @@ ConduitDataCollection::RootFileName()
 //---------------------------------------------------------------------------//
 std::string
 ConduitDataCollection::MeshFileName(int domain_id,
-                                    const std::string &relay_protocol)
+                                    const std::string &relay_protocol_)
 {
    std::string res = prefix_path +
                      name  +
@@ -976,7 +1198,7 @@ ConduitDataCollection::MeshFileName(int domain_id,
                      "/domain_" +
                      to_padded_string(domain_id, pad_digits_rank) +
                      "." +
-                     relay_protocol;
+                     relay_protocol_;
 
    return res;
 }
@@ -994,7 +1216,7 @@ ConduitDataCollection::MeshDirectoryName()
 
 //---------------------------------------------------------------------------//
 std::string
-ConduitDataCollection::MeshFilePattern(const std::string &relay_protocol)
+ConduitDataCollection::MeshFilePattern(const std::string &relay_protocol_)
 {
    std::ostringstream oss;
    oss << name
@@ -1003,7 +1225,7 @@ ConduitDataCollection::MeshFilePattern(const std::string &relay_protocol)
        << "/domain_%0"
        << pad_digits_rank
        << "d."
-       << relay_protocol;
+       << relay_protocol_;
 
    return oss.str();
 }
@@ -1013,14 +1235,14 @@ ConduitDataCollection::MeshFilePattern(const std::string &relay_protocol)
 void
 ConduitDataCollection::SaveRootFile(int num_domains,
                                     const Node &n_mesh,
-                                    const std::string &relay_protocol)
+                                    const std::string &relay_protocol_)
 {
    // default to json root file, except for hdf5 case
    std::string root_proto = "json";
 
-   if (relay_protocol == "hdf5")
+   if (relay_protocol_ == "hdf5")
    {
-      root_proto = relay_protocol;
+      root_proto = relay_protocol_;
    }
 
    Node n_root;
@@ -1051,14 +1273,14 @@ ConduitDataCollection::SaveRootFile(int num_domains,
       }
    }
    // add extra header info
-   n_root["protocol/name"]    =  relay_protocol;
+   n_root["protocol/name"]    =  relay_protocol_;
    n_root["protocol/version"] = "0.3.1";
 
 
    // we will save one file per domain, so trees == files
    n_root["number_of_files"]  = num_domains;
    n_root["number_of_trees"]  = num_domains;
-   n_root["file_pattern"]     = MeshFilePattern(relay_protocol);
+   n_root["file_pattern"]     = MeshFilePattern(relay_protocol_);
    n_root["tree_pattern"]     = "";
 
    // Add the time, time step, and cycle
@@ -1073,9 +1295,9 @@ ConduitDataCollection::SaveRootFile(int num_domains,
 void
 ConduitDataCollection::SaveMeshAndFields(int domain_id,
                                          const Node &n_mesh,
-                                         const std::string &relay_protocol)
+                                         const std::string &relay_protocol_)
 {
-   relay::io::save(n_mesh, MeshFileName(domain_id, relay_protocol));
+   relay::io::save(n_mesh, MeshFileName(domain_id, relay_protocol_));
 }
 
 //---------------------------------------------------------------------------//
@@ -1172,13 +1394,13 @@ ConduitDataCollection::LoadRootFile(Node &root_out)
 //---------------------------------------------------------------------------//
 void
 ConduitDataCollection::LoadMeshAndFields(int domain_id,
-                                         const std::string &relay_protocol)
+                                         const std::string &relay_protocol_)
 {
    // Note: This path doesn't use any info from the root file
    // it uses the implicit mfem ConduitDataCollection layout
 
    Node n_mesh;
-   relay::io::load( MeshFileName(domain_id, relay_protocol), n_mesh);
+   relay::io::load( MeshFileName(domain_id, relay_protocol_), n_mesh);
 
 
    Node verify_info;
