@@ -133,6 +133,7 @@ private:
 
    /// A collection of named QuadratureFunctions
    typedef NamedFieldsMap<QuadratureFunction> QFieldMap;
+
 public:
    typedef GFieldMap::MapType FieldMapType;
    typedef GFieldMap::iterator FieldMapIterator;
@@ -249,10 +250,9 @@ public:
    { field_map.Deregister(field_name, own_data); }
 
    /// Add a QuadratureFunction to the collection.
-   virtual void RegisterQField(const std::string& q_field_name,
+   virtual void RegisterQField(const std::string& field_name,
                                QuadratureFunction *qf)
-   { q_field_map.Register(q_field_name, qf, own_data); }
-
+   { q_field_map.Register(field_name, qf, own_data); }
 
    /// Remove a QuadratureFunction from the collection
    virtual void DeregisterQField(const std::string& field_name)
@@ -280,13 +280,13 @@ public:
 #endif
 
    /// Check if a QuadratureFunction with the given name is in the collection.
-   bool HasQField(const std::string& q_field_name) const
-   { return q_field_map.Has(q_field_name); }
+   bool HasQField(const std::string& field_name) const
+   { return q_field_map.Has(field_name); }
 
    /// Get a pointer to a QuadratureFunction in the collection.
    /** Returns NULL if @a field_name is not in the collection. */
-   QuadratureFunction *GetQField(const std::string& q_field_name)
-   { return q_field_map.Get(q_field_name); }
+   QuadratureFunction *GetQField(const std::string& field_name)
+   { return q_field_map.Get(field_name); }
 
    /// Get a const reference to the internal field map.
    /** The keys in the map are the field names and the values are pointers to
@@ -302,11 +302,13 @@ public:
 
    /// Get a pointer to the mesh in the collection
    Mesh *GetMesh() { return mesh; }
+
    /// Set/change the mesh associated with the collection
    /** When passed a Mesh, assumes the serial case: MPI rank id is set to 0 and
        MPI num_procs is set to 1.  When passed a ParMesh, MPI info from the
        ParMesh is used to set the DataCollection's MPI rank and num_procs. */
    virtual void SetMesh(Mesh *new_mesh);
+
 #ifdef MFEM_USE_MPI
    /// Set/change the mesh associated with the collection.
    /** For this case, @a comm is used to set the DataCollection's MPI rank id
@@ -369,8 +371,7 @@ public:
    /// Save one field, assuming the collection directory already exists.
    virtual void SaveField(const std::string &field_name);
    /// Save one q-field, assuming the collection directory already exists.
-   virtual void SaveQField(const std::string &q_field_name);
-
+   virtual void SaveQField(const std::string &field_name);
    /// Load the collection. Not implemented in the base class DataCollection.
    virtual void Load(int cycle_ = 0);
 
@@ -407,12 +408,18 @@ public:
 class VisItFieldInfo
 {
 public:
-   std::string association;
-   int num_components;
-   int lod;
-   VisItFieldInfo() { association = ""; num_components = 0; lod = 1;}
-   VisItFieldInfo(std::string association_, int num_components_, int lod_ = 1)
-   { association = association_; num_components = num_components_; lod =lod_;}
+   std::string association = "";
+   int num_components = 0;
+   int lod = 1;
+   std::string basis = "";
+   int order = -1;
+   VisItFieldInfo() = default;
+   VisItFieldInfo(std::string association_, int num_components_, int lod_ = 1,
+                  std::string basis_ = "", int order_ = -1)
+   {
+      association = association_; num_components = num_components_; lod =lod_;
+      basis = basis_; order = order_;
+   }
 };
 
 /// Data collection with VisIt I/O routines
@@ -502,59 +509,28 @@ public:
 };
 
 
-/// Helper class for ParaView visualization data
-class ParaViewDataCollection : public DataCollection
+/// Abstract base class for ParaViewDataCollection and ParaViewHDFDataCollection
+class ParaViewDataCollectionBase : public DataCollection
 {
-private:
-   int levels_of_detail;
-   int compression_level;
-   std::fstream pvd_stream;
-   VTKFormat pv_data_format;
-   bool high_order_output;
-   bool restart_mode;
-
 protected:
-   void WritePVTUHeader(std::ostream &out);
-   void WritePVTUFooter(std::ostream &out, const std::string &vtu_prefix);
-   void SaveDataVTU(std::ostream &out, int ref);
-   void SaveGFieldVTU(std::ostream& out, int ref_, const FieldMapIterator& it);
-   const char *GetDataFormatString() const;
-   const char *GetDataTypeString() const;
-   /// @brief If compression is enabled, return the compression level, otherwise
-   /// return 0.
-   int GetCompressionLevel() const;
-
-   std::string GenerateCollectionPath();
-   std::string GenerateVTUFileName(const std::string &prefix, int rank);
-   std::string GenerateVTUPath();
-   std::string GeneratePVDFileName();
-   std::string GeneratePVTUFileName(const std::string &prefix);
-   std::string GeneratePVTUPath();
-
+   int levels_of_detail = 1;
+   int compression_level = -1;
+   bool high_order_output = false;
+   bool restart_mode = false;
+   bool bdr_output = false;
+   VTKFormat pv_data_format = VTKFormat::BINARY;
 
 public:
-   /// Constructor. The collection name is used when saving the data.
-   /** If @a mesh_ is NULL, then the mesh can be set later by calling SetMesh().
-       Before saving the data collection, some parameters in the collection can
-       be adjusted, e.g. SetPadDigits(), SetPrefixPath(), etc. */
-   ParaViewDataCollection(const std::string& collection_name,
-                          mfem::Mesh *mesh_ = NULL);
+   ParaViewDataCollectionBase(const std::string &name, Mesh *mesh);
 
-   /// Set refinement levels - every element is uniformly split based on
-   /// levels_of_detail_. The initial value is 1.
-   void SetLevelsOfDetail(int levels_of_detail_);
-
-   /// Save the collection - the directory name is constructed based on the
-   /// cycle value
-   void Save() override;
-
-   /// Set the data format for the ParaView output files. Possible options are
-   /// VTKFormat::ASCII, VTKFormat::BINARY, and VTKFormat::BINARY32.
-   /// The ASCII and BINARY options output double precision data, whereas the
-   /// BINARY32 option outputs single precision data.
+   /// @brief Set the refinement level.
    ///
-   /// The initial format is VTKFormat::BINARY.
-   void SetDataFormat(VTKFormat fmt);
+   /// In "low-order mode", every element is uniformly split based on the levels
+   /// of detail. In "high-order mode", this sets the polynomial degree for the
+   /// element transformations.
+   ///
+   /// The initial value is 1.
+   void SetLevelsOfDetail(int levels_of_detail_);
 
    /// @brief Set the zlib compression level.
    ///
@@ -570,28 +546,142 @@ public:
    /// Any nonzero compression level will enable compression.
    void SetCompressionLevel(int compression_level_);
 
-   /// Enable or disable zlib compression. If the input is true, use the default
-   /// zlib compression level (unless the compression level has previously been
-   /// set by calling SetCompressionLevel()).
-   void SetCompression(bool compression_) override;
+   /// @brief Sets whether or not to output the data as high-order elements
+   /// (false by default).
+   ///
+   /// Reading high-order data requires ParaView 5.5 or later.
+   void SetHighOrderOutput(bool high_order_output_);
+
+   /// @brief Configures collection to save only fields evaluated on boundaries of
+   /// the mesh.
+   void SetBoundaryOutput(bool bdr_output_);
+
+   /// If compression is enabled, return the compression level, else return 0.
+   int GetCompressionLevel() const;
+
+   /// @brief Set the data format for the ParaView output files.
+   ///
+   /// Possible options are VTKFormat::ASCII, VTKFormat::BINARY, and
+   /// VTKFormat::BINARY32. The ASCII and BINARY options output double precision
+   /// data, whereas the BINARY32 option outputs single precision data.
+   ///
+   /// The initial format is VTKFormat::BINARY.
+   ///
+   /// VTKFormat::ASCII is not supported by ParaViewHDFDataCollection.
+   void SetDataFormat(VTKFormat fmt);
 
    /// Returns true if the output format is BINARY or BINARY32, false if ASCII.
    bool IsBinaryFormat() const;
 
-   /// Sets whether or not to output the data as high-order elements (false
-   /// by default). Reading high-order data requires ParaView 5.5 or later.
-   void SetHighOrderOutput(bool high_order_output_);
-
-   /// Enable or disable restart mode. If restart is enabled, new writes will
-   /// preserve timestep metadata for any solutions prior to the currently
-   /// defined time.
+   /// @brief Enable or disable restart mode.
    ///
-   /// Initially, restart mode is disabled.
+   /// If restart is enabled, new writes will preserve timestep metadata for any
+   /// solutions prior to the currently defined time.
    void UseRestartMode(bool restart_mode_);
-
-   /// Load the collection - not implemented in the ParaView writer
-   void Load(int cycle_ = 0) override;
 };
 
+/// Writer for ParaView visualization (PVD and VTU format)
+class ParaViewDataCollection : public ParaViewDataCollectionBase
+{
+private:
+   std::fstream pvd_stream;
+
+   /// A collection of named Coefficients and VectorCoefficients
+   using CoeffFieldMap = NamedFieldsMap<Coefficient>;
+   using VCoeffFieldMap = NamedFieldsMap<VectorCoefficient>;
+
+   /** A FieldMap mapping registered names to Coefficient and VectorCoefficient
+       pointers. */
+   CoeffFieldMap coeff_field_map;
+   VCoeffFieldMap vcoeff_field_map;
+protected:
+   void WritePVTUHeader(std::ostream &out);
+   void WritePVTUFooter(std::ostream &out, const std::string &vtu_prefix);
+   void SaveDataVTU(std::ostream &out, int ref);
+   void SaveGFieldVTU(std::ostream& out, int ref_, const FieldMapIterator& it);
+   void SaveCoeffFieldVTU(std::ostream& out, int ref_, const std::string &name,
+                          Coefficient &coeff);
+   void SaveVCoeffFieldVTU(std::ostream& out, int ref_, const std::string &name,
+                           VectorCoefficient& coeff);
+   const char *GetDataFormatString() const;
+   const char *GetDataTypeString() const;
+
+   std::string GenerateCollectionPath();
+   std::string GenerateVTUFileName(const std::string &prefix, int rank);
+   std::string GenerateVTUPath();
+   std::string GeneratePVDFileName();
+   std::string GeneratePVTUFileName(const std::string &prefix);
+   std::string GeneratePVTUPath();
+
+public:
+   /// Constructor. The collection name is used when saving the data.
+   /** If @a mesh_ is NULL, then the mesh can be set later by calling SetMesh().
+       Before saving the data collection, some parameters in the collection can
+       be adjusted, e.g. SetPadDigits(), SetPrefixPath(), etc. */
+   ParaViewDataCollection(const std::string& collection_name,
+                          Mesh *mesh_ = nullptr);
+
+   /// Get a const reference to the internal coefficient-field map.
+   const typename CoeffFieldMap::MapType &GetCoeffFieldMap() const
+   { return coeff_field_map.GetMap(); }
+   const typename VCoeffFieldMap::MapType &GetVCoeffFieldMap() const
+   { return vcoeff_field_map.GetMap(); }
+
+   /// Add a Coefficient or VectorCoefficient to the collection.
+   void RegisterCoeffField(const std::string& field_name, Coefficient *coeff)
+   { coeff_field_map.Register(field_name, coeff, own_data); }
+   void RegisterVCoeffField(const std::string& field_name,
+                            VectorCoefficient *vcoeff)
+   { vcoeff_field_map.Register(field_name, vcoeff, own_data); }
+
+   /// Remove a Coefficient or VectorCoefficient from the collection
+   void DeregisterCoeffField(const std::string& field_name)
+   { coeff_field_map.Deregister(field_name, own_data); }
+   void DeregisterVCoeffField(const std::string& field_name)
+   { vcoeff_field_map.Deregister(field_name, own_data); }
+
+   /// Save the collection - the directory name is constructed based on the
+   /// cycle value
+   void Save() override;
+};
+
+#ifdef MFEM_USE_HDF5
+
+/// Writer for ParaView visualization (%VTKHDF format)
+class ParaViewHDFDataCollection : public ParaViewDataCollectionBase
+{
+   /// The low-level VTKHDF object for I/O (pointer to implementation idiom).
+   std::unique_ptr<class VTKHDF> vtkhdf;
+
+   /// Create the VTKHDF object if it doesn't exist already.
+   void EnsureVTKHDF();
+
+   /// Save the collection (templated on floating point type).
+   template <typename FP_T> void TSave();
+
+public:
+   /// @brief Constructor. The collection name is used when saving the data.
+   ///
+   /// If @a mesh_ is NULL, then the mesh can be set later by calling SetMesh().
+   /// Before saving the data collection, some parameters in the collection can
+   /// be adjusted, e.g. SetPadDigits(), SetPrefixPath(), etc.
+   ParaViewHDFDataCollection(const std::string& collection_name,
+                             Mesh *mesh_ = nullptr);
+
+   /// @brief Enable or disable compression.
+   ///
+   /// The compression level can be set with SetCompressionLevel()). VTKHDF
+   /// compression does not require MFEM to be compiled with zlib support.
+   void SetCompression(bool compression_) override;
+
+   /// Save the collection.
+   void Save() override;
+
+   /// Destructor.
+   ~ParaViewHDFDataCollection();
+};
+
+#endif
+
 }
 #endif
