@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -13,6 +13,7 @@
 #define MFEM_FE_POS
 
 #include "fe_base.hpp"
+#include "fe_pyramid.hpp"
 
 namespace mfem
 {
@@ -34,31 +35,31 @@ public:
       ScalarFiniteElement(D, G, Do, O, F)
    { }
 
-   virtual void GetLocalInterpolation(ElementTransformation &Trans,
-                                      DenseMatrix &I) const
+   void GetLocalInterpolation(ElementTransformation &Trans,
+                              DenseMatrix &I) const override
    { ScalarLocalInterpolation(Trans, I, *this); }
 
-   virtual void GetLocalRestriction(ElementTransformation &Trans,
-                                    DenseMatrix &R) const
+   void GetLocalRestriction(ElementTransformation &Trans,
+                            DenseMatrix &R) const override
    { ScalarLocalL2Restriction(Trans, R, *this); }
 
-   virtual void GetTransferMatrix(const FiniteElement &fe,
-                                  ElementTransformation &Trans,
-                                  DenseMatrix &I) const
+   void GetTransferMatrix(const FiniteElement &fe,
+                          ElementTransformation &Trans,
+                          DenseMatrix &I) const override
    { CheckScalarFE(fe).ScalarLocalInterpolation(Trans, I, *this); }
 
    using FiniteElement::Project;
 
    // Low-order monotone "projection" (actually it is not a projection): the
    // dofs are set to be the Coefficient values at the nodes.
-   virtual void Project(Coefficient &coeff,
-                        ElementTransformation &Trans, Vector &dofs) const;
+   void Project(Coefficient &coeff,
+                ElementTransformation &Trans, Vector &dofs) const override;
 
-   virtual void Project (VectorCoefficient &vc,
-                         ElementTransformation &Trans, Vector &dofs) const;
+   void Project (VectorCoefficient &vc,
+                 ElementTransformation &Trans, Vector &dofs) const override;
 
-   virtual void Project(const FiniteElement &fe, ElementTransformation &Trans,
-                        DenseMatrix &I) const;
+   void Project(const FiniteElement &fe, ElementTransformation &Trans,
+                DenseMatrix &I) const override;
 };
 
 
@@ -88,17 +89,17 @@ class BiQuadPos2DFiniteElement : public PositiveFiniteElement
 public:
    /// Construct the BiQuadPos2DFiniteElement
    BiQuadPos2DFiniteElement();
-   virtual void CalcShape(const IntegrationPoint &ip, Vector &shape) const;
-   virtual void CalcDShape(const IntegrationPoint &ip,
-                           DenseMatrix &dshape) const;
-   virtual void GetLocalInterpolation(ElementTransformation &Trans,
-                                      DenseMatrix &I) const;
+   void CalcShape(const IntegrationPoint &ip, Vector &shape) const override;
+   void CalcDShape(const IntegrationPoint &ip,
+                   DenseMatrix &dshape) const override;
+   void GetLocalInterpolation(ElementTransformation &Trans,
+                              DenseMatrix &I) const override;
    using FiniteElement::Project;
-   virtual void Project(Coefficient &coeff, ElementTransformation &Trans,
-                        Vector &dofs) const;
-   virtual void Project(VectorCoefficient &vc, ElementTransformation &Trans,
-                        Vector &dofs) const;
-   virtual void ProjectDelta(int vertex, Vector &dofs) const
+   void Project(Coefficient &coeff, ElementTransformation &Trans,
+                Vector &dofs) const override;
+   void Project(VectorCoefficient &vc, ElementTransformation &Trans,
+                Vector &dofs) const override;
+   void ProjectDelta(int vertex, Vector &dofs) const override
    { dofs = 0.; dofs(vertex) = 1.; }
 };
 
@@ -109,9 +110,9 @@ class QuadPos1DFiniteElement : public PositiveFiniteElement
 public:
    /// Construct the QuadPos1DFiniteElement
    QuadPos1DFiniteElement();
-   virtual void CalcShape(const IntegrationPoint &ip, Vector &shape) const;
-   virtual void CalcDShape(const IntegrationPoint &ip,
-                           DenseMatrix &dshape) const;
+   void CalcShape(const IntegrationPoint &ip, Vector &shape) const override;
+   void CalcDShape(const IntegrationPoint &ip,
+                   DenseMatrix &dshape) const override;
 };
 
 
@@ -131,10 +132,10 @@ private:
 public:
    /// Construct the H1Pos_SegmentElement of order @a p
    H1Pos_SegmentElement(const int p);
-   virtual void CalcShape(const IntegrationPoint &ip, Vector &shape) const;
-   virtual void CalcDShape(const IntegrationPoint &ip,
-                           DenseMatrix &dshape) const;
-   virtual void ProjectDelta(int vertex, Vector &dofs) const;
+   void CalcShape(const IntegrationPoint &ip, Vector &shape) const override;
+   void CalcDShape(const IntegrationPoint &ip,
+                   DenseMatrix &dshape) const override;
+   void ProjectDelta(int vertex, Vector &dofs) const override;
 };
 
 
@@ -150,10 +151,10 @@ private:
 public:
    /// Construct the H1Pos_QuadrilateralElement of order @a p
    H1Pos_QuadrilateralElement(const int p);
-   virtual void CalcShape(const IntegrationPoint &ip, Vector &shape) const;
-   virtual void CalcDShape(const IntegrationPoint &ip,
-                           DenseMatrix &dshape) const;
-   virtual void ProjectDelta(int vertex, Vector &dofs) const;
+   void CalcShape(const IntegrationPoint &ip, Vector &shape) const override;
+   void CalcDShape(const IntegrationPoint &ip,
+                   DenseMatrix &dshape) const override;
+   void ProjectDelta(int vertex, Vector &dofs) const override;
 };
 
 
@@ -169,10 +170,10 @@ private:
 public:
    /// Construct the H1Pos_HexahedronElement of order @a p
    H1Pos_HexahedronElement(const int p);
-   virtual void CalcShape(const IntegrationPoint &ip, Vector &shape) const;
-   virtual void CalcDShape(const IntegrationPoint &ip,
-                           DenseMatrix &dshape) const;
-   virtual void ProjectDelta(int vertex, Vector &dofs) const;
+   void CalcShape(const IntegrationPoint &ip, Vector &shape) const override;
+   void CalcDShape(const IntegrationPoint &ip,
+                   DenseMatrix &dshape) const override;
+   void ProjectDelta(int vertex, Vector &dofs) const override;
 };
 
 
@@ -198,9 +199,9 @@ public:
    static void CalcDShape(const int p, const real_t x, const real_t y,
                           real_t *dshape_1d, real_t *dshape);
 
-   virtual void CalcShape(const IntegrationPoint &ip, Vector &shape) const;
-   virtual void CalcDShape(const IntegrationPoint &ip,
-                           DenseMatrix &dshape) const;
+   void CalcShape(const IntegrationPoint &ip, Vector &shape) const override;
+   void CalcDShape(const IntegrationPoint &ip,
+                   DenseMatrix &dshape) const override;
 };
 
 
@@ -227,9 +228,9 @@ public:
    static void CalcDShape(const int p, const real_t x, const real_t y,
                           const real_t z, real_t *dshape_1d, real_t *dshape);
 
-   virtual void CalcShape(const IntegrationPoint &ip, Vector &shape) const;
-   virtual void CalcDShape(const IntegrationPoint &ip,
-                           DenseMatrix &dshape) const;
+   void CalcShape(const IntegrationPoint &ip, Vector &shape) const override;
+   void CalcDShape(const IntegrationPoint &ip,
+                   DenseMatrix &dshape) const override;
 };
 
 
@@ -250,9 +251,72 @@ public:
    /// Construct the H1Pos_WedgeElement of order @a p
    H1Pos_WedgeElement(const int p);
 
+   void CalcShape(const IntegrationPoint &ip, Vector &shape) const override;
+   void CalcDShape(const IntegrationPoint &ip,
+                   DenseMatrix &dshape) const override;
+};
+
+
+/// Arbitrary order H1 elements in 3D utilizing the Bernstein basis on a pyramid
+///
+/// The pyramid affine-related coordinates $\lambda_i$ for $i=1,\ldots,5$ can
+/// be used to define a positive H1 basis by noting that $\lambda_i \ge 0$
+/// inside the pyramid for all $i$ and that $\sum_{i=1}^5\lambda_i=1$. This
+/// leads to $1 = (\sum_{i=1}^5\lambda_i)^p$. The terms of this product,
+/// expanded as a polynomial in the $\lambda_i$, can be used as a Bernstein
+/// basis of order $p$ on a pyramid.
+class H1Pos_PyramidElement : public PositiveFiniteElement, FuentesPyramid
+{
+protected:
+   const int nterms;
+#ifndef MFEM_THREAD_SAFE
+   mutable Vector m_shape_1d;
+   mutable Vector m_shape;
+   mutable DenseMatrix m_dshape;
+#endif
+   std::map<int,int> dof_map;
+
+   struct Index
+   {
+      Index() = default;
+      int operator()(int i1, int i2, int i3, int i4, int i5)
+      {
+         const int p = i1 + i2 + i3 + i4 + i5;
+         const int min24 = std::min(i2,i4);
+         i1 += min24;
+         i2 -= min24;
+         i3 += min24;
+         i4 -= min24;
+         return i2 + i3 * (p - i4 - i5) - i4 * i5 * (p + 2)
+                - ((i3 - 3) * i3) / 2 + i4 * ((p + 1) * (p + 2)) / 2
+                + (i4 * i5 * (i4 + i5)) / 2 + ((i4 - 1) * i4 * (i4 +1)) / 6
+                - (p + 2) * ((i4 - 1) * i4) / 2
+                + (i5 * (5 + 2 * p - i5) * (i5 * i5 - i5 * (5 + 2 * p)
+                                            + 2 * (5 + 5 * p + p * p))) / 24;
+      }
+   };
+
+public:
+   /// Construct the H1Pos_PyramidElement of order @a p
+   H1Pos_PyramidElement(const int p);
+
+   // The size of shape is (p+1)(p+2)(p+3)(p+4)/24.
+   // The size of shape_1d should be at least p+1.
+   static void CalcShape(const int p, const real_t x, const real_t y,
+                         const real_t z, real_t *shape_1d, real_t *shape);
+
+   // The size of dshape is (p+1)(p+2)(p+3)(p+4)/24 by 3.
+   // The size of dshape_1d should be at least p+1.
+   static void CalcDShape(const int p, const real_t x, const real_t y,
+                          const real_t z, real_t *dshape_1d, real_t *dshape);
+
    virtual void CalcShape(const IntegrationPoint &ip, Vector &shape) const;
    virtual void CalcDShape(const IntegrationPoint &ip,
                            DenseMatrix &dshape) const;
+
+   // Returns (p+1)(p+2)(p+3)(p+4)/24 which is the size of the temporary arrays
+   // needed above
+   int GetNumTerms() const { return nterms; }
 };
 
 
@@ -267,10 +331,10 @@ private:
 public:
    /// Construct the L2Pos_SegmentElement of order @a p
    L2Pos_SegmentElement(const int p);
-   virtual void CalcShape(const IntegrationPoint &ip, Vector &shape) const;
-   virtual void CalcDShape(const IntegrationPoint &ip,
-                           DenseMatrix &dshape) const;
-   virtual void ProjectDelta(int vertex, Vector &dofs) const;
+   void CalcShape(const IntegrationPoint &ip, Vector &shape) const override;
+   void CalcDShape(const IntegrationPoint &ip,
+                   DenseMatrix &dshape) const override;
+   void ProjectDelta(int vertex, Vector &dofs) const override;
 };
 
 
@@ -285,10 +349,10 @@ private:
 public:
    /// Construct the L2Pos_QuadrilateralElement of order @a p
    L2Pos_QuadrilateralElement(const int p);
-   virtual void CalcShape(const IntegrationPoint &ip, Vector &shape) const;
-   virtual void CalcDShape(const IntegrationPoint &ip,
-                           DenseMatrix &dshape) const;
-   virtual void ProjectDelta(int vertex, Vector &dofs) const;
+   void CalcShape(const IntegrationPoint &ip, Vector &shape) const override;
+   void CalcDShape(const IntegrationPoint &ip,
+                   DenseMatrix &dshape) const override;
+   void ProjectDelta(int vertex, Vector &dofs) const override;
 };
 
 
@@ -303,10 +367,10 @@ private:
 public:
    /// Construct the L2Pos_HexahedronElement of order @a p
    L2Pos_HexahedronElement(const int p);
-   virtual void CalcShape(const IntegrationPoint &ip, Vector &shape) const;
-   virtual void CalcDShape(const IntegrationPoint &ip,
-                           DenseMatrix &dshape) const;
-   virtual void ProjectDelta(int vertex, Vector &dofs) const;
+   void CalcShape(const IntegrationPoint &ip, Vector &shape) const override;
+   void CalcDShape(const IntegrationPoint &ip,
+                   DenseMatrix &dshape) const override;
+   void ProjectDelta(int vertex, Vector &dofs) const override;
 };
 
 
@@ -321,10 +385,10 @@ private:
 public:
    /// Construct the L2Pos_TriangleElement of order @a p
    L2Pos_TriangleElement(const int p);
-   virtual void CalcShape(const IntegrationPoint &ip, Vector &shape) const;
-   virtual void CalcDShape(const IntegrationPoint &ip,
-                           DenseMatrix &dshape) const;
-   virtual void ProjectDelta(int vertex, Vector &dofs) const;
+   void CalcShape(const IntegrationPoint &ip, Vector &shape) const override;
+   void CalcDShape(const IntegrationPoint &ip,
+                   DenseMatrix &dshape) const override;
+   void ProjectDelta(int vertex, Vector &dofs) const override;
 };
 
 
@@ -340,10 +404,10 @@ private:
 public:
    /// Construct the L2Pos_TetrahedronElement of order @a p
    L2Pos_TetrahedronElement(const int p);
-   virtual void CalcShape(const IntegrationPoint &ip, Vector &shape) const;
-   virtual void CalcDShape(const IntegrationPoint &ip,
-                           DenseMatrix &dshape) const;
-   virtual void ProjectDelta(int vertex, Vector &dofs) const;
+   void CalcShape(const IntegrationPoint &ip, Vector &shape) const override;
+   void CalcDShape(const IntegrationPoint &ip,
+                   DenseMatrix &dshape) const override;
+   void ProjectDelta(int vertex, Vector &dofs) const override;
 };
 
 
@@ -364,6 +428,61 @@ public:
    /// Construct the L2Pos_WedgeElement of order @a p
    L2Pos_WedgeElement(const int p);
 
+   void CalcShape(const IntegrationPoint &ip, Vector &shape) const override;
+   void CalcDShape(const IntegrationPoint &ip,
+                   DenseMatrix &dshape) const override;
+};
+
+/// Arbitrary order L2 elements in 3D utilizing the Bernstein basis on a pyramid
+class L2Pos_PyramidElement : public PositiveFiniteElement, FuentesPyramid
+{
+protected:
+   const int nterms;
+#ifndef MFEM_THREAD_SAFE
+   mutable Vector m_shape_1d;
+   mutable Vector m_shape;
+   mutable DenseMatrix m_dshape;
+#endif
+   std::map<int,int> dof_map;
+
+   struct Index
+   {
+      Index() = default;
+      int operator()(int i1, int i2, int i3, int i4, int i5)
+      {
+         const int p = i1 + i2 + i3 + i4 + i5;
+         const int min24 = std::min(i2,i4);
+         i1 += min24;
+         i2 -= min24;
+         i3 += min24;
+         i4 -= min24;
+         return i2 + i3 * (p - i4 - i5) - i4 * i5 * (p + 2)
+                - ((i3 - 3) * i3) / 2 + i4 * ((p + 1) * (p + 2)) / 2
+                + (i4 * i5 * (i4 + i5)) / 2 + ((i4 - 1) * i4 * (i4 +1)) / 6
+                - (p + 2) * ((i4 - 1) * i4) / 2
+                + (i5 * (5 + 2 * p - i5) * (i5 * i5 - i5 * (5 + 2 * p)
+                                            + 2 * (5 + 5 * p + p * p))) / 24;
+      }
+   };
+
+   // Returns (p+1)(p+2)(p+3)(p+4)/24 which is the size of the temporary arrays
+   // needed below
+   int GetNumTerms() const { return nterms; }
+
+   // The size of shape is (p+1)(p+2)(p+3)(p+4)/24.
+   // The size of shape_1d should be at least p+1.
+   static void CalcShape(const int p, const real_t x, const real_t y,
+                         const real_t z, real_t *shape_1d, real_t *shape);
+
+   // The size of dshape is (p+1)(p+2)(p+3)(p+4)/24 by 3.
+   // The size of dshape_1d should be at least p+1.
+   static void CalcDShape(const int p, const real_t x, const real_t y,
+                          const real_t z, real_t *dshape_1d, real_t *dshape);
+
+public:
+   /// Construct the L2Pos_PyramidElement of order @a p
+   L2Pos_PyramidElement(const int p);
+
    virtual void CalcShape(const IntegrationPoint &ip, Vector &shape) const;
    virtual void CalcDShape(const IntegrationPoint &ip,
                            DenseMatrix &dshape) const;
