@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -13,6 +13,7 @@
 #define MFEM_FESPACE
 
 #include "../config/config.hpp"
+#include "../linalg/ordering.hpp"
 #include "../linalg/sparsemat.hpp"
 #include "../mesh/mesh.hpp"
 #include "fe_coll.hpp"
@@ -24,52 +25,23 @@
 namespace mfem
 {
 
-/** @brief The ordering method used when the number of unknowns per mesh node
-    (vector dimension) is bigger than 1. */
-class Ordering
-{
-public:
-   /// %Ordering methods:
-   enum Type
-   {
-      byNODES, /**< loop first over the nodes (inner loop) then over the vector
-                    dimension (outer loop); symbolically it can be represented
-                    as: XXX...,YYY...,ZZZ... */
-      byVDIM   /**< loop first over the vector dimension (inner loop) then over
-                    the nodes (outer loop); symbolically it can be represented
-                    as: XYZ,XYZ,XYZ,... */
-   };
-
-   template <Type Ord>
-   static inline int Map(int ndofs, int vdim, int dof, int vd);
-
-   template <Type Ord>
-   static void DofsToVDofs(int ndofs, int vdim, Array<int> &dofs);
-};
-
 /// @brief Type describing possible layouts for Q-vectors.
 /// @sa QuadratureInterpolator and FaceQuadratureInterpolator.
 enum class QVectorLayout
 {
-   byNODES,  ///< NQPT x VDIM x NE (values) / NQPT x VDIM x DIM x NE (grads)
-   byVDIM    ///< VDIM x NQPT x NE (values) / VDIM x DIM x NQPT x NE (grads)
+   /** Layout depending on the input space and the computed quantity:
+       - scalar H1/L2 spaces, values: NQPT x VDIM x NE,
+       - scalar H1/L2 spaces, gradients: NQPT x VDIM x DIM x NE,
+       - vector RT/ND spaces, values: NQPT x SDIM x NE (vdim = 1). */
+   byNODES,
+
+   /** Layout depending on the input space and the computed quantity:
+       - scalar H1/L2 spaces, values: VDIM x NQPT x NE,
+       - scalar H1/L2 spaces, gradients: VDIM x DIM x NQPT x NE,
+       - vector RT/ND spaces, values: SDIM x NQPT x NE (vdim = 1). */
+   byVDIM
 };
 
-template <> inline int
-Ordering::Map<Ordering::byNODES>(int ndofs, int vdim, int dof, int vd)
-{
-   MFEM_ASSERT(dof < ndofs && -1-dof < ndofs && 0 <= vd && vd < vdim, "");
-   return (dof >= 0) ? dof+ndofs*vd : dof-ndofs*vd;
-}
-
-template <> inline int
-Ordering::Map<Ordering::byVDIM>(int ndofs, int vdim, int dof, int vd)
-{
-   MFEM_ASSERT(dof < ndofs && -1-dof < ndofs && 0 <= vd && vd < vdim, "");
-   return (dof >= 0) ? vd+vdim*dof : -1-(vd+vdim*(-1-dof));
-}
-
-
 /// Constants describing the possible orderings of the DOFs in one element.
 enum class ElementDofOrdering
 {
@@ -78,18 +50,34 @@ enum class ElementDofOrdering
        interpolation from the DOFs to quadrature points does not use the
        tensor-product structure. */
    NATIVE,
-   /// Lexicographic ordering for tensor-product FiniteElements.
-   /** This ordering can be used only with tensor-product elements. */
+   /// Lexicographic: DOFs are listed in order of increasing x-coordinate,
+   /// followed by increasing y-coordinate, and z-coordinate.
+   /** This ordering is usually used with tensor-product elements, but it is
+       also supported by some non-tensor elements. */
    LEXICOGRAPHIC
 };
 
+/** Represents the index of an element to p-refine, plus a change to the order
+    of that element. */
+struct pRefinement
+{
+   int index; ///< Mesh element number
+   int delta; ///< Change to element order
+
+   pRefinement() = default;
+
+   pRefinement(int element, int change)
+      : index(element), delta(change) {}
+};
+
 // Forward declarations
 class NURBSExtension;
 class BilinearFormIntegrator;
 class QuadratureSpace;
 class QuadratureInterpolator;
 class FaceQuadratureInterpolator;
-
+class PRefinementTransferOperator;
+struct DerefineMatrixOp;
 
 /** @brief Class FiniteElementSpace - responsible for providing FEM view of the
     mesh, mainly managing the set of degrees of freedom.
@@ -203,7 +191,7 @@ class FaceQuadratureInterpolator;
     @par
     %Vector dofs do not represent a specific index space the way the three
     previous types of dofs do. Rather they are related to modifications of
-    these other index spaces to accomodate multiple copies of the underlying
+    these other index spaces to accommodate multiple copies of the underlying
     function spaces.
     @par
     When using @b vdofs, i.e. when @b vdim != 1, the FiniteElementSpace only
@@ -222,6 +210,7 @@ class FiniteElementSpace
    friend class PRefinementTransferOperator;
    friend void Mesh::Swap(Mesh &, bool);
    friend class LORBase;
+   friend struct DerefineMatrixOp;
 
 protected:
    /// The mesh that FE space lives on (not owned).
@@ -241,22 +230,44 @@ protected:
    /// Number of degrees of freedom. Number of unknowns is #ndofs * #vdim.
    int ndofs;
 
+   bool variableOrder = false;
+
    /** Polynomial order for each element. If empty, all elements are assumed
        to be of the default order (fec->GetOrder()). */
    Array<char> elem_order;
 
-   int nvdofs, nedofs, nfdofs, nbdofs;
+   int nvdofs, nedofs, nfdofs, nbdofs, lnedofs, lnfdofs;
    int uni_fdof; ///< # of single face DOFs if all faces uniform; -1 otherwise
    int *bdofs; ///< internal DOFs of elements if mixed/var-order; NULL otherwise
 
-   /** Variable order spaces only: DOF assignments for edges and faces, see
+   /** Variable-order spaces only: DOF assignments for edges and faces, see
        docs in MakeDofTable. For constant order spaces the tables are empty. */
    Table var_edge_dofs;
    Table var_face_dofs; ///< NOTE: also used for spaces with mixed faces
 
+   // Temporary data for condensing all DOFs to local DOFs.
+   Table loc_var_edge_dofs, loc_var_face_dofs;
+
+   /** Map from all DOFs of all orders on each entity to local DOFs of orders
+       occurring on a local element containing the entity. */
+   Array<int> all2local;
+
+   /// Bit-mask representing a set of orders needed by an edge/face.
+   typedef std::uint64_t VarOrderBits;
+   static constexpr int MaxVarOrder = 8*sizeof(VarOrderBits) - 1;
+
    /** Additional data for the var_*_dofs tables: individual variant orders
        (these are basically alternate J arrays for var_edge/face_dofs). */
    Array<char> var_edge_orders, var_face_orders;
+   Array<char> loc_var_edge_orders, loc_var_face_orders;
+   Array<char> ghost_edge_orders, ghost_face_orders;
+
+   /// Minimum order among neighboring elements.
+   mutable Array<int> edge_min_nghb_order, face_min_nghb_order;
+
+   /// Marker arrays for ghost master entities to be skipped in conforming
+   /// interpolation constraints.
+   Array<bool> skip_edge, skip_face;
 
    // precalculated DOFs for each element, boundary element, and face
    mutable Table *elem_dof; // owned (except in NURBS FE space)
@@ -265,9 +276,16 @@ protected:
    mutable Table *bdr_elem_fos; // bdr face orientations by bdr element index
    mutable Table *face_dof; // owned; in var-order space contains variant 0 DOFs
 
-   Array<int> dof_elem_array, dof_ldof_array;
+   mutable Array<int> dof_elem_array;
+   mutable Array<int> dof_ldof_array;
+   mutable Array<int> dof_bdr_elem_array;
+   mutable Array<int> dof_bdr_ldof_array;
 
    NURBSExtension *NURBSext;
+   /** array of NURBS extension for H(div) and H(curl) vector elements.
+       For each direction an extension is created from the base NURBSext,
+       with an increase in order in the appropriate direction. */
+   Array<NURBSExtension*> VNURBSext;
    int own_ext;
    mutable Array<int> face_to_be; // NURBS FE space only
 
@@ -286,9 +304,18 @@ protected:
    /// Operator computing the action of the transpose of the restriction.
    mutable std::unique_ptr<Operator> R_transpose;
 
+   /** Stores the previous FiniteElementSpace, before p-refinement, in the case
+       that @a PTh is constructed by PRefineAndUpdate(). */
+   std::unique_ptr<FiniteElementSpace> fesPrev;
+
    /// Transformation to apply to GridFunctions after space Update().
    OperatorHandle Th;
 
+   std::shared_ptr<PRefinementTransferOperator> PTh;
+
+   /// Flag to indicate whether the last update was for p-refinement.
+   bool lastUpdatePRef = false;
+
    /// The element restriction operators, see GetElementRestriction().
    mutable OperatorHandle L2E_nat, L2E_lex;
    /// The face restriction operators, see GetFaceRestriction().
@@ -325,7 +352,10 @@ protected:
 
    void UpdateNURBS();
 
+   /** Helper function for constructing the data in this class, for initial
+       construction or updates (e.g. h- or p-refinement). */
    void Construct();
+
    void Destroy();
 
    void ConstructDoFTransArray();
@@ -335,15 +365,28 @@ protected:
    void BuildBdrElementToDofTable() const;
    void BuildFaceToDofTable() const;
 
+   /** Get all @a edges and @a faces (in 3D) on boundary elements with attribute
+       marked as essential in @a bdr_attr_is_ess. */
+   void GetEssentialBdrEdgesFaces(const Array<int> &bdr_attr_is_ess,
+                                  std::set<int> & edges,
+                                  std::set<int> & faces) const;
+
+   /** @brief Initialize internal data that enables the use of the methods
+    GetElementForDof() and GetLocalDofForDof(). */
+   void BuildDofToArrays_() const;
+
+   /** @brief Initialize internal data that enables the use of the methods
+      GetBdrElementForDof() and GetBdrLocalDofForDof(). */
+   void BuildDofToBdrArrays() const;
+
    /** @brief  Generates partial face_dof table for a NURBS space.
 
        The table is only defined for exterior faces that coincide with a
        boundary. */
    void BuildNURBSFaceToDofTable() const;
 
-   /// Bit-mask representing a set of orders needed by an edge/face.
-   typedef std::uint64_t VarOrderBits;
-   static constexpr int MaxVarOrder = 8*sizeof(VarOrderBits) - 1;
+   /// Sets @a all2local. See documentation of @a all2local for details.
+   void SetVarOrderLocalDofs();
 
    /// Return the minimum order (least significant bit set) in the bit mask.
    static int MinOrder(VarOrderBits bits);
@@ -351,20 +394,52 @@ protected:
    /// Return element order: internal version of GetElementOrder without checks.
    int GetElementOrderImpl(int i) const;
 
-   /** In a variable order space, calculate a bitmask of polynomial orders that
-       need to be represented on each edge and face. */
-   void CalcEdgeFaceVarOrders(Array<VarOrderBits> &edge_orders,
-                              Array<VarOrderBits> &face_orders) const;
+   /// Returns true if the space is H1 and has variable-order elements.
+   bool IsVariableOrderH1() const
+   {
+      return variableOrder &&
+             dynamic_cast<const H1_FECollection*>(fec);
+   }
 
-   /** Build the table var_edge_dofs (or var_face_dofs) in a variable order
-       space; return total edge/face DOFs. */
-   int MakeDofTable(int ent_dim, const Array<int> &entity_orders,
+   /** In a variable-order space, calculate a bitmask of polynomial orders that
+       need to be represented on each edge and face. */
+   void CalcEdgeFaceVarOrders(
+      Array<VarOrderBits> &edge_orders, Array<VarOrderBits> &face_orders,
+      Array<VarOrderBits> &edge_elem_orders,
+      Array<VarOrderBits> &face_elem_orders,
+      Array<bool> &skip_edges, Array<bool> &skip_faces) const;
+
+   /// Helper function for ParFiniteElementSpace.
+   virtual void ApplyGhostElementOrdersToEdgesAndFaces(
+      Array<VarOrderBits> &edge_orders, Array<VarOrderBits> &face_orders) const;
+
+   /// Helper function for ParFiniteElementSpace.
+   virtual void GhostFaceOrderToEdges(
+      const Array<VarOrderBits> &face_orders,
+      Array<VarOrderBits> &edge_orders) const { }
+
+   /// Returns true if order propagation is done, for variable-order spaces.
+   virtual bool OrderPropagation(const std::set<int> &edges,
+                                 const std::set<int> &faces,
+                                 Array<VarOrderBits> &edge_orders,
+                                 Array<VarOrderBits> &face_orders) const
+   { return edges.size() == 0 && faces.size() == 0; };
+
+   /// Returns the number of ghost edges (nonzero in ParFiniteElementSpace).
+   virtual int NumGhostEdges() const { return 0; }
+
+   /// Returns the number of ghost faces (nonzero in ParFiniteElementSpace).
+   virtual int NumGhostFaces() const { return 0; }
+
+   /** Build the table var_edge_dofs (or var_face_dofs) in a variable-order
+        space; return total edge/face DOFs. */
+   int MakeDofTable(int ent_dim, const Array<VarOrderBits> &entity_orders,
                     Table &entity_dofs, Array<char> *var_ent_order);
 
    /// Search row of a DOF table for a DOF set of size 'ndof', return first DOF.
    int FindDofs(const Table &var_dof_table, int row, int ndof) const;
 
-   /** In a variable order space, return edge DOFs associated with a polynomial
+   /** In a variable-order space, return edge DOFs associated with a polynomial
        order that has 'ndof' degrees of freedom. */
    int FindEdgeDof(int edge, int ndof) const
    { return FindDofs(var_edge_dofs, edge, ndof); }
@@ -397,6 +472,11 @@ protected:
    /// Calculate the cP and cR matrices for a nonconforming mesh.
    void BuildConformingInterpolation() const;
 
+   /** In variable-order spaces, enforce the minimum order rule on edges and
+       faces, by adding constraints to @a deps for high-order DOFs to
+       interpolate the lowest-order DOFs per mesh entity. */
+   void VariableOrderMinimumRule(SparseMatrix & deps) const;
+
    static void AddDependencies(SparseMatrix& deps, Array<int>& master_dofs,
                                Array<int>& slave_dofs, DenseMatrix& I,
                                int skipfirst = 0);
@@ -454,7 +534,7 @@ protected:
       DerefinementOperator(const FiniteElementSpace *f_fes,
                            const FiniteElementSpace *c_fes,
                            BilinearFormIntegrator *mass_integ);
-      virtual void Mult(const Vector &x, Vector &y) const;
+      void Mult(const Vector &x, Vector &y) const override;
       virtual ~DerefinementOperator();
    };
 
@@ -469,6 +549,11 @@ protected:
                                        const Table *coarse_elem_fos,
                                        const DenseTensor localP[]) const;
 
+   /* This method returns the Refinement matrix (i.e., the embedding)
+      from a coarse variable-order fes to a fine fes (after a geometric refinement) */
+   SparseMatrix *VariableOrderRefinementMatrix(const int coarse_ndofs,
+                                               const Table &coarse_elem_dof) const;
+
    void GetLocalRefinementMatrices(Geometry::Type geom,
                                    DenseTensor &localP) const;
    void GetLocalDerefinementMatrices(Geometry::Type geom,
@@ -517,6 +602,8 @@ protected:
                                                const Array<int> *perm);
 
 public:
+
+
    /** @brief Default constructor: the object is invalid until initialized using
        the method Load(). */
    FiniteElementSpace();
@@ -540,8 +627,7 @@ public:
 
    FiniteElementSpace(Mesh *mesh,
                       const FiniteElementCollection *fec,
-                      int vdim = 1, int ordering = Ordering::byNODES)
-   { Constructor(mesh, NULL, fec, vdim, ordering); }
+                      int vdim = 1, int ordering = Ordering::byNODES);
 
    /// Construct a NURBS FE space based on the given NURBSExtension, @a ext.
    /** @note If the pointer @a ext is NULL, this constructor is equivalent to
@@ -549,8 +635,7 @@ public:
        NURBSExtension, @a ext. */
    FiniteElementSpace(Mesh *mesh, NURBSExtension *ext,
                       const FiniteElementCollection *fec,
-                      int vdim = 1, int ordering = Ordering::byNODES)
-   { Constructor(mesh, ext, fec, vdim, ordering); }
+                      int vdim = 1, int ordering = Ordering::byNODES);
 
    /// Copy assignment not supported
    FiniteElementSpace& operator=(const FiniteElementSpace&) = delete;
@@ -562,25 +647,38 @@ public:
    NURBSExtension *GetNURBSext() { return NURBSext; }
    NURBSExtension *StealNURBSext();
 
-   bool Conforming() const { return mesh->Conforming() && cP == NULL; }
-   bool Nonconforming() const { return mesh->Nonconforming() || cP != NULL; }
+   bool Conforming() const
+   {
+      return NURBSext != NULL ||
+             (mesh->Conforming() && cP == NULL);
+   }
+   bool Nonconforming() const { return !Conforming(); }
+
+   /** Set the prolongation operator of the space to an arbitrary sparse matrix,
+       creating a copy of the argument. */
+   void SetProlongation(const SparseMatrix& p);
+
+   /** Set the restriction operator of the space to an arbitrary sparse matrix,
+       creating a copy of the argument. */
+   void SetRestriction(const SparseMatrix& r);
 
    /// Sets the order of the i'th finite element.
    /** By default, all elements are assumed to be of fec->GetOrder(). Once
-       SetElementOrder is called, the space becomes a variable order space. */
+       SetElementOrder is called, the space becomes a variable-order space. */
    void SetElementOrder(int i, int p);
 
    /// Returns the order of the i'th finite element.
    int GetElementOrder(int i) const;
 
-   /// Return the maximum polynomial order.
-   int GetMaxElementOrder() const
+   /// Return the maximum polynomial order over all elements.
+   virtual int GetMaxElementOrder() const
    { return IsVariableOrder() ? elem_order.Max() : fec->GetOrder(); }
 
    /// Returns true if the space contains elements of varying polynomial orders.
-   bool IsVariableOrder() const { return elem_order.Size(); }
+   bool IsVariableOrder() const { return variableOrder; }
 
-   /// The returned SparseMatrix is owned by the FiniteElementSpace.
+   /// The returned SparseMatrix is owned by the FiniteElementSpace. The method
+   /// returns nullptr if the matrix is identity.
    const SparseMatrix *GetConformingProlongation() const;
 
    /// The returned SparseMatrix is owned by the FiniteElementSpace.
@@ -593,7 +691,8 @@ public:
    /// The returned SparseMatrix is owned by the FiniteElementSpace.
    const SparseMatrix *GetHpConformingRestriction() const;
 
-   /// The returned Operator is owned by the FiniteElementSpace.
+   /// The returned Operator is owned by the FiniteElementSpace. The method
+   /// returns nullptr if the prolongation matrix is identity.
    virtual const Operator *GetProlongationMatrix() const
    { return GetConformingProlongation(); }
 
@@ -644,7 +743,10 @@ public:
    const ElementRestrictionOperator *GetElementRestriction(
       ElementDofOrdering e_ordering) const;
 
-   /// Return an Operator that converts L-vectors to E-vectors on each face.
+   /** @brief Return an Operator that converts L-vectors to E-vectors on each
+       face. */
+   /** @warning only meshes with tensor-product elements are currently
+       supported. */
    virtual const FaceRestriction *GetFaceRestriction(
       ElementDofOrdering f_ordering, FaceType,
       L2FaceValues mul = L2FaceValues::DoubleValued) const;
@@ -661,7 +763,10 @@ public:
        @note The returned pointer is shared. A good practice, before using it,
        is to set all its properties to their expected values, as other parts of
        the code may also change them. That is, it's good to call
-       SetOutputLayout() and DisableTensorProducts() before interpolating. */
+       SetOutputLayout() and DisableTensorProducts() before interpolating.
+
+       @note If the space is not supported by QuadratureInterpolator, nullptr is
+       returned. */
    const QuadratureInterpolator *GetQuadratureInterpolator(
       const IntegrationRule &ir) const;
 
@@ -677,7 +782,10 @@ public:
        @note The returned pointer is shared. A good practice, before using it,
        is to set all its properties to their expected values, as other parts of
        the code may also change them. That is, it's good to call
-       SetOutputLayout() and DisableTensorProducts() before interpolating. */
+       SetOutputLayout() and DisableTensorProducts() before interpolating.
+
+       @note If the space is not supported by QuadratureInterpolator, nullptr is
+       returned. */
    const QuadratureInterpolator *GetQuadratureInterpolator(
       const QuadratureSpace &qs) const;
 
@@ -687,7 +795,10 @@ public:
        @note The returned pointer is shared. A good practice, before using it,
        is to set all its properties to their expected values, as other parts of
        the code may also change them. That is, it's good to call
-       SetOutputLayout() and DisableTensorProducts() before interpolating. */
+       SetOutputLayout() and DisableTensorProducts() before interpolating.
+
+       @note If the space is not supported by FaceQuadratureInterpolator,
+       nullptr is returned. */
    const FaceQuadratureInterpolator *GetFaceQuadratureInterpolator(
       const IntegrationRule &ir, FaceType type) const;
 
@@ -695,14 +806,16 @@ public:
    /** NOTE: it is recommended to use GetElementOrder in new code. */
    int GetOrder(int i) const { return GetElementOrder(i); }
 
-   /** Return the order of an edge. In a variable order space, return the order
+   /** Return the order of an edge. In a variable-order space, return the order
        of a specific variant, or -1 if there are no more variants. */
    int GetEdgeOrder(int edge, int variant = 0) const;
 
    /// Returns the polynomial degree of the i'th face finite element
    int GetFaceOrder(int face, int variant = 0) const;
 
-   /// Returns vector dimension.
+   /// Returns the vector dimension of the finite element space.
+   /** Since the finite elements could be vector-valued, this may not be the
+       dimension of an actual vector in the space; see GetVectorDim(). */
    inline int GetVDim() const { return vdim; }
 
    /// @brief Returns number of degrees of freedom.
@@ -721,6 +834,22 @@ public:
 
    int GetConformingVSize() const { return vdim * GetNConformingDofs(); }
 
+   /// Return the total dimension of a vector in the space
+   /** This accounts for the vectorization of elements and cases where the
+       elements themselves are vector-valued; see FiniteElement:GetRangeDim().
+       If the finite elements are FiniteElement::SCALAR, this equals GetVDim().
+
+       Note: For vector-valued elements, the results pads up the range dimension
+       to the spatial dimension. E.g., consider a stack of 5 vector-valued
+       elements each representing 2D vectors, living in a 3 dimensional space.
+       Then this fucntion would give 15, not 10.
+       */
+   int GetVectorDim() const;
+
+   /// Return the dimension of the curl of a GridFunction defined on this space.
+   /** Note: This assumes a space dimension of 2 or 3 only. */
+   int GetCurlDim() const;
+
    /// Return the ordering method.
    inline Ordering::Type GetOrdering() const { return ordering; }
 
@@ -770,6 +899,9 @@ public:
    { return mesh->GetBdrElementType(i); }
 
    /// Returns ElementTransformation for the @a i-th element.
+   /// @note The returned pointer references an object owned by the associated
+   /// @a Mesh that will be modified by other calls to `GetElementTransformation`.
+   /// As such, this pointer should @b not be deleted by the caller.
    ElementTransformation *GetElementTransformation(int i) const
    { return mesh->GetElementTransformation(i); }
 
@@ -795,8 +927,8 @@ public:
    /// could be used to produce the appropriate offsets from these local dofs.
    ///@{
 
-   /// @brief Returns indices of degrees of freedom of element 'elem'.
-   /// The returned indices are offsets into an @ref ldof vector. See also
+   /// @brief Returns indices of degrees of freedom of element 'elem'. The
+   /// returned indices are offsets into an @ref ldof vector. See also
    /// GetElementVDofs().
    ///
    /// @note In many cases the returned DofTransformation object will be NULL.
@@ -806,15 +938,18 @@ public:
    /// needed for Nedelec basis functions of order 2 and above on 3D elements
    /// with triangular faces.
    ///
-   /// @note The returned object should NOT be deleted by the caller.
+   /// @deprecated Use of the returned object is deprecated. The returned object
+   /// should @b not be deleted by the caller. If the DofTransformation is
+   /// needed, use GetElementDofs(int, Array<int> &, DofTransformation &)
+   /// instead.
    DofTransformation *GetElementDofs(int elem, Array<int> &dofs) const;
 
-   /// @brief The same as GetElementDofs(), but with a user-allocated
-   /// DofTransformation object. @a doftrans must be allocated in advance and
-   /// will be owned by the caller. The user can use the
-   /// DofTransformation::GetDofTransformation method on the returned
-   /// @a doftrans object to detect if the DofTransformation should actually be
-   /// used.
+   /// @brief The same as GetElementDofs(), but with a user-provided
+   /// DofTransformation object.
+   ///
+   /// The user can use DofTransformation::IsIdentity on the returned @a
+   /// doftrans object to determine if the DofTransformation needs to actually
+   /// be used.
    virtual void GetElementDofs(int elem, Array<int> &dofs,
                                DofTransformation &doftrans) const;
 
@@ -829,22 +964,25 @@ public:
    /// needed for Nedelec basis functions of order 2 and above on 3D elements
    /// with triangular faces.
    ///
-   /// @note The returned object should NOT be deleted by the caller.
+   /// @deprecated Use of the returned object is deprecated. The returned object
+   /// should @b not be deleted by the caller. If the DofTransformation is
+   /// needed, use GetBdrElementDofs(int, Array<int> &, DofTransformation &)
+   /// instead.
    DofTransformation *GetBdrElementDofs(int bel, Array<int> &dofs) const;
 
-   /// @brief The same as GetBdrElementDofs(), but with a user-allocated
-   /// DofTransformation object. @a doftrans must be allocated in advance and
-   /// will be owned by the caller. The user can use the
-   /// DofTransformation::GetDofTransformation method on the returned
-   /// @a doftrans object to detect if the DofTransformation should actually be
-   /// used.
+   /// @brief The same as GetBdrElementDofs(), but with a user-provided
+   /// DofTransformation object.
+   ///
+   /// The user can use DofTransformation::IsIdentity on the returned @a
+   /// doftrans object to determine if the DofTransformation needs to actually
+   /// be used.
    virtual void GetBdrElementDofs(int bel, Array<int> &dofs,
                                   DofTransformation &doftrans) const;
 
    /// @brief Returns the indices of the degrees of freedom for the specified
    /// face, including the DOFs for the edges and the vertices of the face.
    ///
-   /// In variable order spaces, multiple variants of DOFs can be returned.
+   /// In variable-order spaces, multiple variants of DOFs can be returned.
    /// See GetEdgeDofs() for more details.
    /// @return Order of the selected variant, or -1 if there are no more
    /// variants.
@@ -856,7 +994,7 @@ public:
    /// @brief Returns the indices of the degrees of freedom for the specified
    /// edge, including the DOFs for the vertices of the edge.
    ///
-   /// In variable order spaces, multiple sets of DOFs may exist on an edge,
+   /// In variable-order spaces, multiple sets of DOFs may exist on an edge,
    /// corresponding to the different polynomial orders of incident elements.
    /// The 'variant' parameter is the zero-based index of the desired DOF set.
    /// The variants are ordered from lowest polynomial degree to the highest.
@@ -1041,15 +1179,18 @@ public:
    /// needed for Nedelec basis functions of order 2 and above on 3D elements
    /// with triangular faces.
    ///
-   /// @note The returned object should NOT be deleted by the caller.
+   /// @deprecated Use of the returned object is deprecated. The returned object
+   /// should @b not be deleted by the caller. If the DofTransformation is
+   /// needed, use GetElementVDofs(int, Array<int> &, DofTransformation &)
+   /// instead.
    DofTransformation *GetElementVDofs(int i, Array<int> &vdofs) const;
 
-   /// @brief The same as GetElementVDofs(), but with a user-allocated
-   /// DofTransformation object. @a doftrans must be allocated in advance and
-   /// will be owned by the caller. The user can use the
-   /// DofTransformation::GetDofTransformation method on the returned
-   /// @a doftrans object to detect if the DofTransformation should actually be
-   /// used.
+   /// @brief The same as GetElementVDofs(), but with a user-provided
+   /// DofTransformation object.
+   ///
+   /// The user can use DofTransformation::IsIdentity on the returned @a
+   /// doftrans object to determine if the DofTransformation needs to actually
+   /// be used.
    void GetElementVDofs(int i, Array<int> &vdofs,
                         DofTransformation &doftrans) const;
 
@@ -1065,15 +1206,18 @@ public:
    /// needed for Nedelec basis functions of order 2 and above on 3D elements
    /// with triangular faces.
    ///
-   /// @note The returned object should NOT be deleted by the caller.
+   /// @deprecated Use of the returned object is deprecated. The returned object
+   /// should @b not be deleted by the caller. If the DofTransformation is
+   /// needed, use GetBdrElementVDofs(int, Array<int> &, DofTransformation &)
+   /// instead.
    DofTransformation *GetBdrElementVDofs(int i, Array<int> &vdofs) const;
 
-   /// @brief The same as GetBdrElementVDofs(), but with a user-allocated
-   /// DofTransformation object. @a doftrans must be allocated in advance and
-   /// will be owned by the caller. The user can use the
-   /// DofTransformation::GetDofTransformation method on the returned
-   /// @a doftrans object to detect if the DofTransformation should actually be
-   /// used.
+   /// @brief The same as GetBdrElementVDofs(), but with a user-provided
+   /// DofTransformation object.
+   ///
+   /// The user can use DofTransformation::IsIdentity on the returned @a
+   /// doftrans object to determine if the DofTransformation needs to actually
+   /// be used.
    void GetBdrElementVDofs(int i, Array<int> &vdofs,
                            DofTransformation &doftrans) const;
 
@@ -1149,18 +1293,21 @@ public:
    const Table &GetFaceToDofTable() const
    { if (!face_dof) { BuildFaceToDofTable(); } return *face_dof; }
 
-   /** @brief Initialize internal data that enables the use of the methods
-       GetElementForDof() and GetLocalDofForDof(). */
-   void BuildDofToArrays();
+   /// Deprecated. This function is not required to be called by the user.
+   MFEM_DEPRECATED void BuildDofToArrays() const { BuildDofToArrays_(); }
+
+   /// Return the index of the first element that contains ldof index @a i.
+   int GetElementForDof(int i) const { BuildDofToArrays_(); return dof_elem_array[i]; }
+
+   /// Return the dof index within the element from GetElementForDof() for ldof index @a i.
+   int GetLocalDofForDof(int i) const { BuildDofToArrays_(); return dof_ldof_array[i]; }
+
+   /// Return the index of the first boundary element that contains ldof index @a i.
+   int GetBdrElementForDof(int i) const { BuildDofToBdrArrays(); return dof_bdr_elem_array[i]; }
+
+   /// Return the dof index within the boundary element from GetBdrElementForDof() for ldof index @a i.
+   int GetBdrLocalDofForDof(int i) const { BuildDofToBdrArrays(); return dof_bdr_ldof_array[i]; }
 
-   /// Return the index of the first element that contains dof @a i.
-   /** This method can be called only after setup is performed using the method
-       BuildDofToArrays(). */
-   int GetElementForDof(int i) const { return dof_elem_array[i]; }
-   /// Return the local dof index in the first element that contains dof @a i.
-   /** This method can be called only after setup is performed using the method
-       BuildDofToArrays(). */
-   int GetLocalDofForDof(int i) const { return dof_ldof_array[i]; }
 
    /** @brief Returns pointer to the FiniteElement in the FiniteElementCollection
         associated with i'th element in the mesh object.
@@ -1168,6 +1315,13 @@ public:
         an empty partition. */
    virtual const FiniteElement *GetFE(int i) const;
 
+   /** @brief Return GetFE(0) if the local mesh is not empty; otherwise return a
+       typical FE based on the Geometry types in the global mesh.
+
+       This method can be used as a replacement for GetFE(0) that will be valid
+       even if the local mesh is empty. */
+   const FiniteElement *GetTypicalFE() const;
+
    /** @brief Returns pointer to the FiniteElement in the FiniteElementCollection
         associated with i'th boundary face in the mesh object. */
    const FiniteElement *GetBE(int i) const;
@@ -1185,6 +1339,12 @@ public:
    /// Return the trace element from element 'i' to the given 'geom_type'
    const FiniteElement *GetTraceElement(int i, Geometry::Type geom_type) const;
 
+   /// @brief Return a "typical" trace element.
+   ///
+   /// This can be used in situations where the local mesh partition may be
+   /// empty.
+   const FiniteElement *GetTypicalTraceElement() const;
+
    /** @brief Mark degrees of freedom associated with boundary elements with
        the specified boundary attributes (marked in 'bdr_attr_is_ess').
        For spaces with 'vdim' > 1, the 'component' parameter can be used
@@ -1208,6 +1368,18 @@ public:
        marked as essential. */
    void GetBoundaryTrueDofs(Array<int> &boundary_dofs, int component = -1);
 
+   /** @brief Mark degrees of freedom associated with exterior faces of the
+       mesh. For spaces with 'vdim' > 1, the 'component' parameter can be used
+       to restricts the marked vDOFs to the specified component. */
+   virtual void GetExteriorVDofs(Array<int> &exterior_vdofs,
+                                 int component = -1) const;
+
+   /** @brief Get a list of all true dofs on the exterior of the mesh,
+       @a exterior_dofs. For spaces with 'vdim' > 1, the 'component' parameter
+       can be used to restricts the marked tDOFs to the specified component. */
+   virtual void GetExteriorTrueDofs(Array<int> &exterior_dofs,
+                                    int component = -1) const;
+
    /// Convert a Boolean marker array to a list containing all marked indices.
    static void MarkerToList(const Array<int> &marker, Array<int> &list);
 
@@ -1276,17 +1448,32 @@ public:
    virtual void GetTrueTransferOperator(const FiniteElementSpace &coarse_fes,
                                         OperatorHandle &T) const;
 
-   /** @brief Reflect changes in the mesh: update number of DOFs, etc. Also, calculate
-       GridFunction transformation operator (unless want_transform is false).
-       Safe to call multiple times, does nothing if space already up to date. */
+   /** @brief Reflect changes in the mesh: update number of DOFs, etc. Also,
+       calculate GridFunction transformation operator (unless want_transform is
+       false). Safe to call multiple times, does nothing if space already up to
+       date. */
    virtual void Update(bool want_transform = true);
 
+   /** P-refine and update the space. If @a want_transfer, also maintain the old
+       space and a transfer operator accessible by GetPrefUpdateOperator(). */
+   virtual void PRefineAndUpdate(const Array<pRefinement> & refs,
+                                 bool want_transfer = true);
+
+   /** Return true iff p-refinement is supported in this space. Current support
+       is only for L2 or H1 spaces on purely quadrilateral or hexahedral
+       meshes. */
+   bool PRefinementSupported();
+
    /// Get the GridFunction update operator.
    const Operator* GetUpdateOperator() { Update(); return Th.Ptr(); }
 
    /// Return the update operator in the given OperatorHandle, @a T.
    void GetUpdateOperator(OperatorHandle &T) { T = Th; }
 
+   /** Returns @a PTh, the transfer operator from the previous space to the
+       current space, after p-refinement. */
+   std::shared_ptr<const PRefinementTransferOperator> GetPrefUpdateOperator();
+
    /** @brief Set the ownership of the update operator: if set to false, the
        Operator returned by GetUpdateOperator() must be deleted outside the
        FiniteElementSpace. */
@@ -1309,13 +1496,16 @@ public:
        GridFunction to check if it is up to date with the space. */
    long GetSequence() const { return sequence; }
 
+   /// Return a flag indicating whether the last update was for p-refinement.
+   bool LastUpdatePRef() const { return lastUpdatePRef; }
+
    /// Return whether or not the space is discontinuous (L2)
    bool IsDGSpace() const
    {
       return dynamic_cast<const L2_FECollection*>(fec) != NULL;
    }
 
-   /** In variable order spaces on nonconforming (NC) meshes, this function
+   /** In variable-order spaces on nonconforming (NC) meshes, this function
        controls whether strict conformity is enforced in cases where coarse
        edges/faces have higher polynomial order than their fine NC neighbors.
        In the default (strict) case, the coarse side polynomial order is
@@ -1331,6 +1521,18 @@ public:
       Update(false);
    }
 
+   /** @brief Compute the space's node positions w.r.t. given mesh positions.
+       The function uses FiniteElement::GetNodes() to obtain the reference DOF
+       positions of each finite element.
+
+       @param[in]  mesh_nodes   Mesh positions. Assumes that it has the same
+                                topology & ordering as the mesh of the FE space,
+                                i.e, same size as this->GetMesh()->GetNodes().
+       @param[out] fes_node_pos Positions of the FE space's nodes.
+       @param[in]  fes_nodes_ordering  Ordering of fes_node_pos.     */
+   void GetNodePositions(const Vector &mesh_nodes, Vector &fes_node_pos,
+                         int fes_nodes_ordering = Ordering::byNODES) const;
+
    /// Save finite element space to output stream @a out.
    void Save(std::ostream &out) const;
 
@@ -1341,18 +1543,19 @@ public:
    virtual ~FiniteElementSpace();
 };
 
-/// @brief Return true if the mesh contains only one topology and the elements are tensor elements.
+/// @brief Return true if the mesh contains only one topology and the elements
+/// are tensor elements.
 inline bool UsesTensorBasis(const FiniteElementSpace& fes)
 {
    Mesh & mesh = *fes.GetMesh();
    const bool mixed = mesh.GetNumGeometries(mesh.Dimension()) > 1;
-   // Potential issue: empty local mesh --> no element 0.
    return !mixed &&
-          dynamic_cast<const mfem::TensorBasisElement *>(fes.GetFE(0))!=nullptr;
+          dynamic_cast<const mfem::TensorBasisElement *>(
+             fes.GetTypicalFE()) != nullptr;
 }
 
-/// @brief Return LEXICOGRAPHIC if mesh contains only one topology and the elements are tensor
-/// elements, otherwise, return NATIVE.
+/// @brief Return LEXICOGRAPHIC if mesh contains only one topology and the
+/// elements are tensor elements, otherwise, return NATIVE.
 ElementDofOrdering GetEVectorOrdering(const FiniteElementSpace& fes);
 
 }
