@@ -34,6 +34,8 @@
 
 using namespace mfem::internal;
 
+static const bool dof_transformation = false;
+
 namespace mfem
 {
 
@@ -64,38 +66,17 @@ public:
       }
    }
 
-   bool is_vector_fe() const
+   BilinearFormIntegrator *newBFormIntegrator() const
    {
-      bool is_vector_fe = false;
-      for (auto i_ptr : integrators)
-      {
-         if (i_ptr->is_vector_fe())
-         {
-            is_vector_fe = true;
-            break;
-         }
-      }
-
-      return is_vector_fe;
-   }
-
-   BilinearFormIntegrator * new_mass_integrator() const
-   {
-      if (is_vector_fe())
-      {
-         return new VectorFEMassIntegrator();
-      }
-      else
-      {
-         return new MassIntegrator();
-      }
+      assert(!integrators.empty());
+      return integrators[0]->newBFormIntegrator();
    }
 };
 
 ParMortarAssembler::~ParMortarAssembler() = default;
 
-void ParMortarAssembler::SetSolver(const std::shared_ptr<IterativeSolver>
-                                   &solver)
+void ParMortarAssembler::SetSolver(
+   const std::shared_ptr<IterativeSolver> &solver)
 {
    impl_->solver = solver;
 }
@@ -105,7 +86,8 @@ void ParMortarAssembler::SetAssembleMassAndCouplingTogether(const bool value)
    impl_->assemble_mass_and_coupling_together = value;
 }
 
-void ParMortarAssembler::SetMaxSolverIterations(const int max_solver_iterations)
+void ParMortarAssembler::SetMaxSolverIterations(
+   const int max_solver_iterations)
 {
    impl_->ensure_solver();
 
@@ -389,11 +371,13 @@ private:
    inline static void copy_global_dofs(ParFiniteElementSpace &fe,
                                        std::vector<ElementDofMap> &dof_map)
    {
+
       dof_map.resize(fe.GetNE());
       Array<int> vdofs;
       for (int i = 0; i < fe.GetNE(); ++i)
       {
          fe.GetElementVDofs(i, vdofs);
+
          for (int k = 0; k < vdofs.Size(); ++k)
          {
             long g_dof = 0;
@@ -571,6 +555,7 @@ static FiniteElementCollection *FECollFromName(const std::string &comp_name)
 }
 
 static void read_space(moonolith::InputStream &is,
+                       const int vdim,
                        std::shared_ptr<FiniteElementSpace> &space,
                        std::vector<ElementDofMap> &dof_map)
 {
@@ -632,10 +617,11 @@ static void read_space(moonolith::InputStream &is,
    Finalize(*mesh_ptr, true);
    // }
 
-   space = make_shared<FiniteElementSpace>(mesh_ptr, fe_coll);
+   space = make_shared<FiniteElementSpace>(mesh_ptr, fe_coll, vdim);
 }
 
-static void read_spaces(moonolith::InputStream &is, Spaces &spaces)
+static void read_spaces(moonolith::InputStream &is, const int vdim,
+                        Spaces &spaces)
 {
    bool has_source, has_destination;
    is >> has_source >> has_destination;
@@ -644,7 +630,7 @@ static void read_spaces(moonolith::InputStream &is, Spaces &spaces)
 
    if (has_source)
    {
-      read_space(is, spaces.spaces()[0], spaces.dof_map(0));
+      read_space(is, vdim, spaces.spaces()[0], spaces.dof_map(0));
       spaces.set_must_destroy_attached(0, true);
    }
    else
@@ -654,7 +640,7 @@ static void read_spaces(moonolith::InputStream &is, Spaces &spaces)
 
    if (has_destination)
    {
-      read_space(is, spaces.spaces()[1], spaces.dof_map(1));
+      read_space(is, vdim, spaces.spaces()[1], spaces.dof_map(1));
       spaces.set_must_destroy_attached(1, true);
    }
    else
@@ -722,7 +708,7 @@ static bool Assemble(moonolith::Communicator &comm,
    std::map<long, std::shared_ptr<Spaces>> spaces;
    std::map<long, std::vector<std::shared_ptr<Spaces>>> migrated_spaces;
 
-   auto read = [&spaces, &migrated_spaces,
+   auto read = [&spaces, &migrated_spaces, &source,
                          comm](const long ownerrank, const long /*senderrank*/,
                       bool is_forwarding, DataContainer &data, InputStream &in)
    {
@@ -730,7 +716,7 @@ static bool Assemble(moonolith::Communicator &comm,
 
       std::shared_ptr<Spaces> proc_space = std::make_shared<Spaces>(comm);
 
-      read_spaces(in, *proc_space);
+      read_spaces(in, source->GetVDim(), *proc_space);
 
       if (!is_forwarding)
       {
@@ -769,7 +755,7 @@ static bool Assemble(moonolith::Communicator &comm,
                                 &comm](const long ownerrank, const long /*recvrank*/,
                         const std::vector<long>::const_iterator &begin,
                         const std::vector<long>::const_iterator &end,
-                        const DataContainer &/*data*/, OutputStream &out)
+                        const DataContainer & /*data*/, OutputStream &out)
    {
       CHECK_STREAM_WRITE_BEGIN("vol_proj", out);
 
@@ -834,8 +820,10 @@ static bool Assemble(moonolith::Communicator &comm,
 
 static void add_matrix(const Array<int> &destination_vdofs,
                        const Array<int> &source_vdofs,
-                       const DenseMatrix &elem_mat, moonolith::SparseMatrix<double> &mat_buffer)
+                       const DenseMatrix &elem_mat,
+                       moonolith::SparseMatrix<double> &mat_buffer)
 {
+
    for (int i = 0; i < destination_vdofs.Size(); ++i)
    {
       long dof_I = destination_vdofs[i];
@@ -872,7 +860,7 @@ std::shared_ptr<HypreParMatrix> convert_to_hypre_matrix(
    moonolith::SparseMatrix<double> &mat_buffer)
 {
 
-   auto && comm = mat_buffer.comm();
+   auto &&comm = mat_buffer.comm();
 
    moonolith::Redistribute<moonolith::SparseMatrix<double>> redist(comm.get());
    redist.apply(destination_ranges, mat_buffer, moonolith::AddAssign<double>());
@@ -883,7 +871,7 @@ std::shared_ptr<HypreParMatrix> convert_to_hypre_matrix(
    std::vector<HYPRE_Int> J;
    std::vector<double> data;
    J.reserve(mat_buffer.n_local_entries());
-   data.reserve(J.size());
+   data.reserve(J.capacity());
 
    for (auto it = mat_buffer.iter(); it; ++it)
    {
@@ -897,12 +885,12 @@ std::shared_ptr<HypreParMatrix> convert_to_hypre_matrix(
       I[i] += I[i - 1];
    }
 
-   return std::make_shared<HypreParMatrix>(
-             comm.get(), s_offsets[1] - s_offsets[0], mat_buffer.rows(),
-             mat_buffer.cols(), &I[0], &J[0], &data[0], s_offsets, m_offsets);
+   auto ret = std::make_shared<HypreParMatrix>(
+                 comm.get(), s_offsets[1] - s_offsets[0], mat_buffer.rows(),
+                 mat_buffer.cols(), &I[0], &J[0], &data[0], s_offsets, m_offsets);
+   return ret;
 }
 
-
 int order_multiplier(const Geometry::Type type, const int dim)
 {
    return
@@ -912,16 +900,14 @@ int order_multiplier(const Geometry::Type type, const int dim)
 
 template <int Dimensions>
 static bool
-Assemble(moonolith::Communicator &comm,
-         ParMortarAssembler::Impl &impl,
+Assemble(moonolith::Communicator &comm, ParMortarAssembler::Impl &impl,
          const moonolith::SearchSettings &settings, const bool verbose)
 {
    auto &source = impl.source;
    auto &destination = impl.destination;
 
    auto &integrators = impl.integrators;
-   auto &pmat  = impl.coupling_matrix;
-
+   auto &pmat = impl.coupling_matrix;
 
    bool lump_mass = false;
    int max_q_order = 0;
@@ -945,21 +931,22 @@ Assemble(moonolith::Communicator &comm,
    IntegrationRule src_ir;
    IntegrationRule dest_ir;
 
+   const HYPRE_BigInt m_global_n_dofs = source->GlobalTrueVSize();
+   HYPRE_BigInt *m_offsets = source->GetTrueDofOffsets();
+
+   const HYPRE_BigInt s_global_n_dofs = destination->GlobalTrueVSize();
+   HYPRE_BigInt *s_offsets = destination->GetTrueDofOffsets();
+
    double local_element_matrices_sum = 0.0;
 
-   const auto m_global_n_dofs = source->GlobalVSize();
-   auto *m_offsets = source->GetDofOffsets();
-
-   const auto s_global_n_dofs = destination->GlobalVSize();
-   auto *s_offsets = destination->GetDofOffsets();
-
    moonolith::SparseMatrix<double> mat_buffer(comm);
    mat_buffer.set_size(s_global_n_dofs, m_global_n_dofs);
 
    moonolith::SparseMatrix<double> mass_mat_buffer(comm);
    mass_mat_buffer.set_size(s_global_n_dofs, s_global_n_dofs);
 
-   std::unique_ptr<BilinearFormIntegrator> mass_integr(impl.new_mass_integrator());
+   std::unique_ptr<BilinearFormIntegrator> mass_integr(
+      impl.newBFormIntegrator());
 
    auto fun = [&](const ElementAdapter<Dimensions> &source,
                   const ElementAdapter<Dimensions> &destination) -> bool
@@ -990,7 +977,8 @@ Assemble(moonolith::Communicator &comm,
          contraction_order = std::max(contraction_order, 2 * dest_order);
       }
 
-      const int order = contraction_order + dest_order_mult * dest_Trans.OrderW() + max_q_order;
+      const int order =
+      contraction_order + dest_order_mult * dest_Trans.OrderW() + max_q_order;
 
       cut->SetIntegrationOrder(order);
 
@@ -1022,7 +1010,8 @@ Assemble(moonolith::Communicator &comm,
          }
 
          local_element_matrices_sum += Sum(cumulative_elemmat);
-         add_matrix(destination_vdofs, source_vdofs, cumulative_elemmat, mat_buffer);
+         add_matrix(destination_vdofs, source_vdofs, cumulative_elemmat,
+                    mat_buffer);
 
          if (impl.assemble_mass_and_coupling_together)
          {
@@ -1046,7 +1035,8 @@ Assemble(moonolith::Communicator &comm,
                }
             }
 
-            add_matrix(destination_vdofs, destination_vdofs, elemmat, mass_mat_buffer);
+            add_matrix(destination_vdofs, destination_vdofs, elemmat,
+                       mass_mat_buffer);
          }
 
          return true;
@@ -1082,11 +1072,8 @@ Assemble(moonolith::Communicator &comm,
    comm.all_reduce(&destination_ranges[0], destination_ranges.size(),
                    moonolith::MPIMax());
 
-   pmat = convert_to_hypre_matrix(
-             destination_ranges,
-             s_offsets,
-             m_offsets,
-             mat_buffer);
+   pmat = convert_to_hypre_matrix(destination_ranges, s_offsets, m_offsets,
+                                  mat_buffer);
 
    if (impl.assemble_mass_and_coupling_together)
    {
@@ -1121,14 +1108,12 @@ bool ParMortarAssembler::Assemble(std::shared_ptr<HypreParMatrix> &pmat)
    moonolith::Communicator comm(impl_->comm);
    if (impl_->source->GetMesh()->Dimension() == 2)
    {
-      ok = mfem::Assemble<2>(comm, *impl_, settings,
-                             impl_->verbose);
+      ok = mfem::Assemble<2>(comm, *impl_, settings, impl_->verbose);
    }
 
    if (impl_->source->GetMesh()->Dimension() == 3)
    {
-      ok = mfem::Assemble<3>(comm, *impl_, settings,
-                             impl_->verbose);
+      ok = mfem::Assemble<3>(comm, *impl_, settings, impl_->verbose);
    }
 
    pmat = impl_->coupling_matrix;
@@ -1157,9 +1142,7 @@ bool ParMortarAssembler::Apply(const ParGridFunction &src_fun,
 
    auto &B = *impl_->coupling_matrix;
    auto &D = *impl_->mass_matrix;
-
-   auto &P_source = *impl_->source->Dof_TrueDof_Matrix();
-   auto &P_destination = *impl_->destination->Dof_TrueDof_Matrix();
+   auto &P_destination = *impl_->destination->GetProlongationMatrix();
 
    impl_->ensure_solver();
 
@@ -1173,22 +1156,46 @@ bool ParMortarAssembler::Apply(const ParGridFunction &src_fun,
       impl_->solver->SetPrintLevel(3);
    }
 
-   Vector P_x_src_fun(B.Width());
-   P_source.MultTranspose(src_fun, P_x_src_fun);
+   if (dof_transformation)
+   {
+      // Maybe delete in the future!?
 
-   Vector B_x_src_fun(B.Height());
-   B_x_src_fun = 0.0;
+      Vector P_x_src_fun(B.Width());
+      auto &P_source = *impl_->source->GetProlongationMatrix();
+      P_source.MultTranspose(src_fun, P_x_src_fun);
 
-   B.Mult(P_x_src_fun, B_x_src_fun);
+      Vector B_x_src_fun(B.Height());
+      B_x_src_fun = 0.0;
 
-   Vector R_x_dest_fun(D.Height());
-   R_x_dest_fun = 0.0;
+      B.Mult(P_x_src_fun, B_x_src_fun);
 
-   impl_->solver->Mult(B_x_src_fun, R_x_dest_fun);
+      Vector R_x_dest_fun(D.Height());
+      R_x_dest_fun = 0.0;
 
-   P_destination.Mult(R_x_dest_fun, dest_fun);
+      impl_->solver->Mult(B_x_src_fun, R_x_dest_fun);
 
-   dest_fun.Update();
+      P_destination.Mult(R_x_dest_fun, dest_fun);
+
+      dest_fun.Update();
+   }
+   else
+   {
+
+      std::unique_ptr<HypreParVector> td_src_fun =
+         std::unique_ptr<HypreParVector>(src_fun.ParallelProject());
+
+      Vector B_x_src_fun(B.Height());
+      B_x_src_fun = 0.0;
+      B.Mult(*td_src_fun, B_x_src_fun);
+
+
+      Vector R_x_dest_fun(D.Height());
+      R_x_dest_fun = 0.0;
+      impl_->solver->Mult(B_x_src_fun, R_x_dest_fun);
+
+      P_destination.Mult(R_x_dest_fun, dest_fun);
+
+   }
 
    // Sanity check!
    int converged = impl_->solver->GetFinalNorm() < 1e-5;
@@ -1200,7 +1207,6 @@ bool ParMortarAssembler::Update()
 {
    using namespace std;
    const bool verbose = impl_->verbose;
-   static const bool dof_transformation = true;
 
    moonolith::Communicator comm(impl_->comm);
 
@@ -1238,13 +1244,12 @@ bool ParMortarAssembler::Update()
    if (!impl_->assemble_mass_and_coupling_together)
    {
       ParBilinearForm b_form(impl_->destination.get());
-      b_form.AddDomainIntegrator(impl_->new_mass_integrator());
+      b_form.AddDomainIntegrator(impl_->newBFormIntegrator());
       b_form.Assemble();
       b_form.Finalize();
       impl_->mass_matrix = shared_ptr<HypreParMatrix>(b_form.ParallelAssemble());
    }
 
-
    comm.barrier();
 
    if (verbose && comm.is_root())
