@@ -69,9 +69,7 @@ QuadratureInterpolator::QuadratureInterpolator(const FiniteElementSpace &fes,
 
    d_buffer.UseDevice(true);
    if (fespace->GetNE() == 0) { return; }
-   const FiniteElement *fe = fespace->GetTypicalFE();
-   MFEM_VERIFY(fe->GetMapType() == FiniteElement::MapType::VALUE ||
-               fe->GetMapType() == FiniteElement::MapType::H_DIV,
+   MFEM_VERIFY(SupportsFESpace(fes),
                "Only elements with MapType VALUE and H_DIV are supported!");
 }
 
@@ -86,12 +84,20 @@ QuadratureInterpolator::QuadratureInterpolator(const FiniteElementSpace &fes,
 {
    d_buffer.UseDevice(true);
    if (fespace->GetNE() == 0) { return; }
-   const FiniteElement *fe = fespace->GetTypicalFE();
-   MFEM_VERIFY(fe->GetMapType() == FiniteElement::MapType::VALUE ||
-               fe->GetMapType() == FiniteElement::MapType::H_DIV,
+   MFEM_VERIFY(SupportsFESpace(fes),
                "Only elements with MapType VALUE and H_DIV are supported!");
 }
 
+bool QuadratureInterpolator::SupportsFESpace(const FiniteElementSpace &fespace)
+{
+   const FiniteElement *fe = fespace.GetTypicalFE();
+   const Mesh &mesh = *fespace.GetMesh();
+   return (fe->GetMapType() == FiniteElement::MapType::VALUE ||
+           fe->GetMapType() == FiniteElement::MapType::H_DIV)
+          && (!fespace.IsVariableOrder())
+          && (!mesh.IsMixedMesh());
+}
+
 namespace internal
 {
 
@@ -138,7 +144,7 @@ static void Eval1D(const int NE,
    {
       for (int q = 0; q < nq; ++q)
       {
-         if (eval_flags & QI::VALUES)
+         if (eval_flags & (QI::VALUES | QI::PHYSICAL_VALUES))
          {
             for (int c = 0; c < vdim; c++)
             {
@@ -242,7 +248,7 @@ static void Eval2D(const int NE,
 
       MFEM_FOREACH_THREAD(q, x, NQ)
       {
-         if (eval_flags & QI::VALUES)
+         if (eval_flags & (QI::VALUES | QI::PHYSICAL_VALUES))
          {
             real_t ed[max_VDIM];
             for (int c = 0; c < VDIM; c++) { ed[c] = 0.0; }
@@ -326,7 +332,7 @@ static void Eval2D(const int NE,
                   const double E = j(0,0)*j(0,0) + j(1,0)*j(1,0) + j(2,0)*j(2,0);
                   const double F = j(0,0)*j(0,1) + j(1,0)*j(1,1) + j(2,0)*j(2,1);
                   const double G = j(0,1)*j(0,1) + j(1,1)*j(1,1) + j(2,1)*j(2,1);
-                  det(q,e) = sqrt(E*G - F*F);
+                  det(q,e) = std::sqrt(E*G - F*F);
                }
             }
          }
@@ -396,7 +402,7 @@ static void Eval3D(const int NE,
 
       MFEM_FOREACH_THREAD(q, x, NQ)
       {
-         if (eval_flags & QI::VALUES)
+         if (eval_flags & (QI::VALUES | QI::PHYSICAL_VALUES))
          {
             real_t ed[max_VDIM];
             for (int c = 0; c < VDIM; c++) { ed[c] = 0.0; }
@@ -543,7 +549,7 @@ void QuadratureInterpolator::Mult(const Vector &e_vec,
 
    if (use_tensor_eval)
    {
-      if (eval_flags & VALUES)
+      if (eval_flags & (VALUES | PHYSICAL_VALUES))
       {
          TensorEvalKernels::Run(dim, q_layout, vdim, nd, nq, ne, maps.B.Read(),
                                 e_vec.Read(), q_val.Write(), vdim, nd, nq);
@@ -566,8 +572,8 @@ void QuadratureInterpolator::Mult(const Vector &e_vec,
    }
    else // use_tensor_eval == false
    {
-      EvalKernels::Run(dim, vdim, maps.ndof, maps.nqpt, ne,vdim,q_layout,
-                       geom, maps,e_vec, q_val,q_der,q_det,eval_flags);
+      EvalKernels::Run(dim, vdim, maps.ndof, maps.nqpt, ne,vdim, q_layout,
+                       geom, maps, e_vec, q_val, q_der, q_det, eval_flags);
    }
 }
 
@@ -658,6 +664,13 @@ void QuadratureInterpolator::Values(const Vector &e_vec,
    Mult(e_vec, VALUES, q_val, empty, empty);
 }
 
+void QuadratureInterpolator::PhysValues(const Vector &e_vec,
+                                        Vector &q_val) const
+{
+   Vector empty;
+   Mult(e_vec, PHYSICAL_VALUES, q_val, empty, empty);
+}
+
 void QuadratureInterpolator::Derivatives(const Vector &e_vec,
                                          Vector &q_der) const
 {
