@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -40,6 +40,10 @@ protected:
    OperatorHandle fw_t_oper; ///< Forward true-dof operator
    OperatorHandle bw_t_oper; ///< Backward true-dof operator
 
+   bool use_ea;
+
+   MemoryType d_mt;
+
 #ifdef MFEM_USE_MPI
    bool parallel;
 #endif
@@ -59,14 +63,23 @@ protected:
 
 public:
    /** Construct a transfer algorithm between the domain, @a dom_fes_, and
-       range, @a ran_fes_, FE spaces. */
-   GridTransfer(FiniteElementSpace &dom_fes_, FiniteElementSpace &ran_fes_);
+       range, @a ran_fes_, FE spaces, d_mt_ will specify memory space for
+       large data structures */
+   GridTransfer(FiniteElementSpace &dom_fes_,
+                FiniteElementSpace &ran_fes_);
 
    /// Virtual destructor
    virtual ~GridTransfer() { }
 
+   /** Uses device friendly element assembly versions for L2Projection
+       transfers, L2, H1 FEM spaces currently supported */
+   void UseEA(bool use_ea_) { use_ea = use_ea_;}
+
+   /** Set memory type for large data structures */
+   void SetMemType(MemoryType d_mt_) {d_mt = d_mt_;}
+
    /** @brief Set the desired Operator::Type for the construction of all
-       operators defined by the underlying transfer algorithm. */
+        operators defined by the underlying transfer algorithm. */
    /** The default value is Operator::ANY_TYPE which typically corresponds to a
        matrix-free operator representation. Note that derived classes are not
        required to support this setting and can ignore it. */
@@ -143,9 +156,9 @@ public:
    void SetMassIntegrator(BilinearFormIntegrator *mass_integ_,
                           bool own_mass_integ_ = true);
 
-   virtual const Operator &ForwardOperator();
+   const Operator &ForwardOperator() override;
 
-   virtual const Operator &BackwardOperator();
+   const Operator &BackwardOperator() override;
 };
 
 
@@ -169,7 +182,8 @@ public:
     smaller than the number of coarse dofs. */
 class L2ProjectionGridTransfer : public GridTransfer
 {
-protected:
+   // Must be public due to host device lambdas
+public:
    /** Abstract class representing projection operator between a high-order
        finite element space on a coarse mesh, and a low-order finite element
        space on a refined mesh (LOR). We assume that the low-order space,
@@ -194,18 +208,66 @@ protected:
       const FiniteElementSpace& fes_ho;
       const FiniteElementSpace& fes_lor;
 
+      MemoryType d_mt;
+      Array<int> offsets;
       Table ho2lor;
 
       L2Projection(const FiniteElementSpace& fes_ho_,
-                   const FiniteElementSpace& fes_lor_);
+                   const FiniteElementSpace& fes_lor_,
+                   MemoryType d_mt_ = Device::GetHostMemoryType());
 
       void BuildHo2Lor(int nel_ho, int nel_lor,
                        const CoarseFineTransformations& cf_tr);
 
       void ElemMixedMass(Geometry::Type geom, const FiniteElement& fe_ho,
-                         const FiniteElement& fe_lor, ElementTransformation* el_tr,
+                         const FiniteElement& fe_lor, ElementTransformation* tr_ho,
+                         ElementTransformation* tr_lor,
                          IntegrationPointTransformation& ip_tr,
                          DenseMatrix& M_mixed_el) const;
+
+      void ElemMixedMass(Geometry::Type geom, const FiniteElement& fe_ho,
+                         const FiniteElement& fe_lor,
+                         ElementTransformation* el_tr,
+                         IntegrationPointTransformation& ip_tr,
+                         DenseMatrix& B_L, DenseMatrix& B_H) const;
+   public:
+      /* Returns the Mixed Mass M_LH via device element assembly by building the
+      basis functions and data at the quadrature points. */
+      void MixedMassEA(const FiniteElementSpace& fes_ho_,
+                       const FiniteElementSpace& fes_lor_,
+                       Vector &M_LH,
+                       MemoryType d_mt_ = Device::GetHostMemoryType());
+   };
+
+   // Class below must be public as we now have device code
+public:
+   class H1SpaceMixedMassOperator : public Operator
+   {
+   protected:
+      const FiniteElementSpace* fes_ho;
+      const FiniteElementSpace* fes_lor;
+      Table* ho2lor;
+      Vector* M_LH_ea;
+   public:
+      H1SpaceMixedMassOperator(const FiniteElementSpace* fes_ho_,
+                               const FiniteElementSpace* fes_lor_,
+                               Table* ho2lor_, Vector* M_LH_ea_);
+      void Mult(const Vector& x, Vector& y) const;
+      void MultTranspose(const Vector& x, Vector& y) const;
+   };
+
+   class H1SpaceLumpedMassOperator : public Operator
+   {
+   protected:
+      const FiniteElementSpace* fes_ho;
+      const FiniteElementSpace* fes_lor;
+      Vector* ML_inv; // inverse of lumped M_L
+   public:
+      H1SpaceLumpedMassOperator(const FiniteElementSpace* fes_ho_,
+                                const FiniteElementSpace* fes_lor_,
+                                Vector& ML_inv_);
+      void Mult(const Vector& x, Vector& y) const;
+      void MultTranspose(const Vector& x, Vector& y) const;
    };
 
    /** Class for projection operator between a L2 high-order finite element
@@ -213,24 +275,35 @@ protected:
        refined mesh (LOR). */
    class L2ProjectionL2Space : public L2Projection
    {
-      // The restriction and prolongation operators are represented as dense
-      // elementwise matrices (of potentially different sizes, because of mixed
-      // meshes or p-refinement). The matrix entries are stored in the R and P
-      // arrays. The entries of the i'th high-order element are stored at the
-      // index given by offsets[i].
+      /// The restriction and prolongation operators are represented as dense
+      /// elementwise matrices (of potentially different sizes, because of mixed
+      /// meshes or p-refinement). The matrix entries are stored in the R and P
+      /// arrays. The entries of the i'th high-order element are stored at the
+      /// index given by offsets[i].
       mutable Array<real_t> R, P;
-      Array<int> offsets;
+
+      const bool use_ea;
 
    public:
       L2ProjectionL2Space(const FiniteElementSpace& fes_ho_,
-                          const FiniteElementSpace& fes_lor_);
+                          const FiniteElementSpace& fes_lor_,
+                          const bool use_ea_,
+                          MemoryType d_mt_ = Device::GetHostMemoryType());
+
+      /*Same as above but assembles and stores R_ea, P_ea */
+      void EAL2ProjectionL2Space();
+
       /// Maps <tt>x</tt>, primal field coefficients defined on a coarse mesh
       /// with a higher order L2 finite element space, to <tt>y</tt>, primal
       /// field coefficients defined on a refined mesh with a low order L2
       /// finite element space. Refined mesh should be a uniform refinement of
       /// the coarse mesh. Coefficients are computed through minimization of L2
       /// error between the fields.
-      virtual void Mult(const Vector& x, Vector& y) const;
+      void Mult(const Vector& x, Vector& y) const override;
+
+      /// Perform mult on the device (same as above)
+      void EAMult(const Vector& x, Vector& y) const;
+
       /// Maps <tt>x</tt>, dual field coefficients defined on a refined mesh
       /// with a low order L2 finite element space, to <tt>y</tt>, dual field
       /// coefficients defined on a coarse mesh with a higher order L2 finite
@@ -238,7 +311,10 @@ protected:
       /// coarse mesh. Coefficients are computed through minimization of L2
       /// error between the primal fields. Note, if the <tt>x</tt>-coefficients
       /// come from ProlongateTranspose, then mass is conserved.
-      virtual void MultTranspose(const Vector& x, Vector& y) const;
+      void MultTranspose(const Vector& x, Vector& y) const override;
+
+      void EAMultTranspose(const Vector& x, Vector& y) const;
+
       /// Maps <tt>x</tt>, primal field coefficients defined on a refined mesh
       /// with a low order L2 finite element space, to <tt>y</tt>, primal field
       /// coefficients defined on a coarse mesh with a higher order L2 finite
@@ -246,7 +322,10 @@ protected:
       /// coarse mesh. Coefficients are computed from the mass conservative
       /// left-inverse prolongation operation. This functionality is also
       /// provided as an Operator by L2Prolongation.
-      virtual void Prolongate(const Vector& x, Vector& y) const;
+      void Prolongate(const Vector& x, Vector& y) const override;
+
+      void EAProlongate(const Vector& x, Vector& y) const;
+
       /// Maps <tt>x</tt>, dual field coefficients defined on a coarse mesh with
       /// a higher order L2 finite element space, to <tt>y</tt>, dual field
       /// coefficients defined on a refined mesh with a low order L2 finite
@@ -254,22 +333,47 @@ protected:
       /// coarse mesh. Coefficients are computed from the transpose of the mass
       /// conservative left-inverse prolongation operation. This functionality
       /// is also provided as an Operator by L2Prolongation.
-      virtual void ProlongateTranspose(const Vector& x, Vector& y) const;
-      virtual void SetRelTol(real_t p_rtol_) { } ///< No-op.
-      virtual void SetAbsTol(real_t p_atol_) { } ///< No-op.
+      void ProlongateTranspose(const Vector& x, Vector& y) const override;
+
+      void EAProlongateTranspose(const Vector& x, Vector& y) const;
+
+      void SetRelTol(real_t p_rtol_) override { } ///< No-op.
+      void SetAbsTol(real_t p_atol_) override { } ///< No-op.
    };
 
+protected:
+
+   /// Class below must be public as we now have device code
+public:
+
    /** Projection operator between a H1 high-order finite element space on a
        coarse mesh, and a H1 low-order finite element space on a refined mesh
        (LOR). */
    class L2ProjectionH1Space : public L2Projection
    {
+      const bool use_ea;
+
    public:
       L2ProjectionH1Space(const FiniteElementSpace &fes_ho_,
-                          const FiniteElementSpace &fes_lor_);
+                          const FiniteElementSpace &fes_lor_,
+                          const bool use_ea_,
+                          MemoryType d_mt_ = Device::GetHostMemoryType());
 #ifdef MFEM_USE_MPI
       L2ProjectionH1Space(const ParFiniteElementSpace &pfes_ho_,
-                          const ParFiniteElementSpace &pfes_lor_);
+                          const ParFiniteElementSpace &pfes_lor_,
+                          const bool use_ea_,
+                          MemoryType d_mt_ = Device::GetHostMemoryType());
+#endif
+      /// Same as above but assembles action of R through 4 parts:
+      ///   ( )  inv( lumped(M_L) ), which is a diagonal matrix (essentially a vector)
+      ///   ( )  ElementRestrictionOperator for LOR space
+      ///   ( )  mixed mass matrix M_{LH}
+      ///   ( )  ElementRestrictionOperator for HO space
+      void EAL2ProjectionH1Space();
+
+#ifdef MFEM_USE_MPI
+      void EAL2ProjectionH1Space(const ParFiniteElementSpace &pfes_ho_,
+                                 const ParFiniteElementSpace &pfes_lor_);
 #endif
       /// Maps <tt>x</tt>, primal field coefficients defined on a coarse mesh
       /// with a higher order H1 finite element space, to <tt>y</tt>, primal
@@ -277,7 +381,8 @@ protected:
       /// finite element space. Refined mesh should be a uniform refinement of
       /// the coarse mesh. Coefficients are computed through minimization of L2
       /// error between the fields.
-      virtual void Mult(const Vector& x, Vector& y) const;
+      void Mult(const Vector& x, Vector& y) const override;
+
       /// Maps <tt>x</tt>, dual field coefficients defined on a refined mesh
       /// with a low order H1 finite element space, to <tt>y</tt>, dual field
       /// coefficients defined on a coarse mesh with a higher order H1 finite
@@ -285,7 +390,8 @@ protected:
       /// coarse mesh. Coefficients are computed through minimization of L2
       /// error between the primal fields. Note, if the <tt>x</tt>-coefficients
       /// come from ProlongateTranspose, then mass is conserved.
-      virtual void MultTranspose(const Vector& x, Vector& y) const;
+      void MultTranspose(const Vector& x, Vector& y) const override;
+
       /// Maps <tt>x</tt>, primal field coefficients defined on a refined mesh
       /// with a low order H1 finite element space, to <tt>y</tt>, primal field
       /// coefficients defined on a coarse mesh with a higher order H1 finite
@@ -293,7 +399,8 @@ protected:
       /// coarse mesh. Coefficients are computed from the mass conservative
       /// left-inverse prolongation operation. This functionality is also
       /// provided as an Operator by L2Prolongation.
-      virtual void Prolongate(const Vector& x, Vector& y) const;
+      void Prolongate(const Vector& x, Vector& y) const override;
+
       /// Maps <tt>x</tt>, dual field coefficients defined on a coarse mesh with
       /// a higher order H1 finite element space, to <tt>y</tt>, dual field
       /// coefficients defined on a refined mesh with a low order H1 finite
@@ -301,15 +408,23 @@ protected:
       /// coarse mesh. Coefficients are computed from the transpose of the mass
       /// conservative left-inverse prolongation operation. This functionality
       /// is also provided as an Operator by L2Prolongation.
-      virtual void ProlongateTranspose(const Vector& x, Vector& y) const;
-      virtual void SetRelTol(real_t p_rtol_);
-      virtual void SetAbsTol(real_t p_atol_);
+      void ProlongateTranspose(const Vector& x, Vector& y) const override;
+
+      /// Returns the inverse of an on-rank lumped mass matrix
+      void LumpedMassInverse(Vector& ML_inv) const;
+
+      void SetRelTol(real_t p_rtol_) override;
+      void SetAbsTol(real_t p_atol_) override;
+
    protected:
       /// Sets up the PCG solver (sets parameters, operator, and preconditioner)
       void SetupPCG();
-      /// Computes on-rank R and M_LH matrices.
+
+      /// @brief Computes on-rank R and M_LH matrices. If true, computes mixed mass and/or
+      /// inverse lumped mass matrix error when compared to device implementation.
       std::pair<std::unique_ptr<SparseMatrix>,
           std::unique_ptr<SparseMatrix>> ComputeSparseRAndM_LH();
+
       /// @brief Recovers vector of tdofs given a vector of dofs and a finite
       /// element space
       void GetTDofs(const FiniteElementSpace& fes, const Vector& x, Vector& X) const;
@@ -332,10 +447,8 @@ protected:
       void TDofsListByVDim(const FiniteElementSpace& fes,
                            int vdim,
                            Array<int>& vdofs_list) const;
-      /// Returns the inverse of an on-rank lumped mass matrix
-      void LumpedMassInverse(Vector& ML_inv) const;
+
       /// @brief Computes sparsity pattern and initializes R matrix.
-      ///
       /// Based on BilinearForm::AllocMat(), except maps between coarse HO
       /// elements and refined LOR elements.
       std::unique_ptr<SparseMatrix> AllocR();
@@ -345,10 +458,34 @@ protected:
       // The restriction operator is represented as an Operator R. The
       // prolongation operator is a dense matrix computed as the inverse of (R^T
       // M_L R), and hence, is not stored.
+      // If element assembly is enabled
       std::unique_ptr<Operator> R;
       // Used to compute P = (RT*M_LH)^(-1) M_LH^T
       std::unique_ptr<Operator> M_LH;
+      // Inverted operator in P = (RT*M_LH)^(-1) M_LH^T. Used to compute P via PCG.
       std::unique_ptr<Operator> RTxM_LH;
+      // Lumped M_L inverse operator built via EA. Wrapped with restriction maps
+      // to multiply with scalar TDof LOR vectors.
+      std::unique_ptr<Operator> ML_inv_vea;
+      // LDof Mixed mass operator built via EA. Wrapped with restriction maps to send
+      // scalar LDof HO vectors to LDof LOR vectors.
+      Operator *M_LH_local_op;
+
+      // Scalar finite element spaces for stored Tdof-to-and-from-LDof maps.
+      std::unique_ptr<FiniteElementSpace> fes_ho_scalar;
+      std::unique_ptr<FiniteElementSpace> fes_lor_scalar;
+      // Element Assembled mixed mass
+      Vector M_LH_ea;
+      // Element Assembled lumped M_L inverse built via EA. Stores diagonal as a Ldof vector.
+      Vector ML_inv_ea;
+
+#ifdef MFEM_USE_MPI
+      std::unique_ptr<ParFiniteElementSpace> pfes_ho_scalar;
+      std::unique_ptr<ParFiniteElementSpace> pfes_lor_scalar;
+      Vector RML_inv;
+#endif
+
+      friend class L2ProjectionL2Space;
    };
 
    /** Mass-conservative prolongation operator going in the opposite direction
@@ -378,17 +515,18 @@ protected:
 public:
    L2ProjectionGridTransfer(FiniteElementSpace &coarse_fes_,
                             FiniteElementSpace &fine_fes_,
-                            bool force_l2_space_ = false)
+                            bool force_l2_space_ = false,
+                            MemoryType d_mt_ = Device::GetHostMemoryType()) // move to method
       : GridTransfer(coarse_fes_, fine_fes_),
         F(NULL), B(NULL), force_l2_space(force_l2_space_)
    { }
    virtual ~L2ProjectionGridTransfer();
 
-   virtual const Operator &ForwardOperator();
+   const Operator &ForwardOperator() override;
 
-   virtual const Operator &BackwardOperator();
+   const Operator &BackwardOperator() override;
 
-   virtual bool SupportsBackwardsOperator() const;
+   bool SupportsBackwardsOperator() const override;
 private:
    void BuildF();
 };
@@ -403,11 +541,11 @@ public:
    /// Constructs a transfer operator from \p lFESpace to \p hFESpace.
    /** No matrices are assembled, only the action to a vector is being computed.
        If both spaces' FE collection pointers are pointing to the same
-       collection we assume that the grid was refined while keeping the order
+       collection, we assume that the grid was refined while keeping the order
        constant. If the FE collections are different, it is assumed that both
-       spaces have are using the same mesh. If the first element of the
-       high-order space is a `TensorBasisElement`, the optimized tensor-product
-       transfers are used. If not, the general transfers used. */
+       spaces are using the same mesh. If the first element of the high-order
+       space is a `TensorBasisElement`, the optimized tensor-product transfers
+       are used. If not, the general transfers used. */
    TransferOperator(const FiniteElementSpace& lFESpace,
                     const FiniteElementSpace& hFESpace);
 
@@ -416,12 +554,12 @@ public:
 
    /// @brief Interpolation or prolongation of a vector \p x corresponding to
    /// the coarse space to the vector \p y corresponding to the fine space.
-   virtual void Mult(const Vector& x, Vector& y) const override;
+   void Mult(const Vector& x, Vector& y) const override;
 
    /// Restriction by applying the transpose of the Mult method.
    /** The vector \p x corresponding to the fine space is restricted to the
        vector \p y corresponding to the coarse space. */
-   virtual void MultTranspose(const Vector& x, Vector& y) const override;
+   void MultTranspose(const Vector& x, Vector& y) const override;
 };
 
 /// Matrix-free transfer operator between finite element spaces on the same mesh
@@ -442,16 +580,16 @@ public:
                                const FiniteElementSpace& hFESpace_);
 
    /// Destructor
-   virtual ~PRefinementTransferOperator();
+   virtual ~PRefinementTransferOperator() { }
 
    /// @brief Interpolation or prolongation of a vector \p x corresponding to
    /// the coarse space to the vector \p y corresponding to the fine space.
-   virtual void Mult(const Vector& x, Vector& y) const override;
+   void Mult(const Vector& x, Vector& y) const override;
 
    /// Restriction by applying the transpose of the Mult method.
    /** The vector \p x corresponding to the fine space is restricted to the
    vector \p y corresponding to the coarse space. */
-   virtual void MultTranspose(const Vector& x, Vector& y) const override;
+   void MultTranspose(const Vector& x, Vector& y) const override;
 };
 
 /// @brief Matrix-free transfer operator between finite element spaces on the
@@ -484,16 +622,16 @@ public:
       const FiniteElementSpace& hFESpace_);
 
    /// Destructor
-   virtual ~TensorProductPRefinementTransferOperator();
+   virtual ~TensorProductPRefinementTransferOperator() { }
 
    /// @brief Interpolation or prolongation of a vector \p x corresponding to
    /// the coarse space to the vector \p y corresponding to the fine space.
-   virtual void Mult(const Vector& x, Vector& y) const override;
+   void Mult(const Vector& x, Vector& y) const override;
 
    /// Restriction by applying the transpose of the Mult method.
    /** The vector \p x corresponding to the fine space is restricted to the
    vector \p y corresponding to the coarse space. */
-   virtual void MultTranspose(const Vector& x, Vector& y) const override;
+   void MultTranspose(const Vector& x, Vector& y) const override;
 };
 
 /// @brief Matrix-free transfer operator between finite element spaces working
@@ -522,12 +660,12 @@ public:
    /// dof vector \p y.
    /** The true dof vector \p x corresponding to the coarse space is restricted
        to the true dof vector \p y corresponding to the fine space. */
-   virtual void Mult(const Vector& x, Vector& y) const override;
+   void Mult(const Vector& x, Vector& y) const override;
 
    /// Restriction by applying the transpose of the Mult method.
    /** The true dof vector \p x corresponding to the fine space is restricted to
        the true dof vector \p y corresponding to the coarse space. */
-   virtual void MultTranspose(const Vector& x, Vector& y) const override;
+   void MultTranspose(const Vector& x, Vector& y) const override;
 };
 
 } // namespace mfem
