@@ -16,6 +16,8 @@
 #include "globals.hpp"
 #include "mem_manager.hpp"
 
+#include <string>
+
 namespace mfem
 {
 
@@ -95,7 +97,6 @@ struct Backend
       CEED_MASK = CEED_CPU | CEED_CUDA | CEED_HIP,
       /// Biwise-OR of all device backends
       DEVICE_MASK = CUDA_MASK | HIP_MASK | DEBUG_DEVICE,
-
       /// Biwise-OR of all RAJA backends
       RAJA_MASK = RAJA_CPU | RAJA_OMP | RAJA_CUDA | RAJA_HIP,
       /// Biwise-OR of all OCCA backends
@@ -123,16 +124,16 @@ class Device
 {
 private:
    friend class MemoryManager;
-   enum MODES {SEQUENTIAL, ACCELERATED};
 
    static bool device_env, mem_host_env, mem_device_env, mem_types_set;
    MFEM_ENZYME_INACTIVE static MFEM_EXPORT Device device_singleton;
 
-   MODES mode = Device::SEQUENTIAL;
    int dev = 0;   ///< Device ID of the configured device.
    int ngpu = -1; ///< Number of detected devices; -1: not initialized.
+
    /// Bitwise-OR of all configured backends.
    unsigned long backends = Backend::CPU;
+
    /// Set to true during configuration, except in 'device_singleton'.
    bool destroy_mm = false;
    bool mpi_gpu_aware = false;
@@ -145,29 +146,22 @@ private:
    /// Current Device MemoryClass
    MemoryClass device_mem_class = MemoryClass::HOST;
 
-   char *device_option = NULL;
-   Device(Device const&);
-   void operator=(Device const&);
+   // Delete copy constructor and copy assignment.
+   Device(const Device &) = delete;
+   Device &operator=(const Device &) = delete;
+
+   // Access the Device singleton.
    static Device& Get() { return device_singleton; }
 
-   /// Setup switcher based on configuration settings
-   void Setup(const int device_id = 0);
+   /// Setup switcher based on configuration settings.
+   void Setup(const std::string &device_option, const int device_id);
 
+   /// Configure host/device MemoryType/MemoryClass.
+   void UpdateMemoryTypeAndClass(const std::string &device_option);
+
+   /// Configure the backends to include @a b.
    void MarkBackend(Backend::Id b) { backends |= b; }
 
-   void UpdateMemoryTypeAndClass();
-
-   /// Enable the use of the configured device in the code that follows.
-   /** After this call MFEM classes will use the backend kernels whenever
-       possible, transferring data automatically to the device, if necessary.
-
-       If the only configured backend is the default host CPU one, the device
-       will remain disabled.
-
-       If the device is actually enabled, this method will also update the
-       current host/device MemoryType and MemoryClass. */
-   static void Enable();
-
 public:
    /** @brief Default constructor. Unless Configure() is called later, the
        default Backend::CPU will be used. */
@@ -183,45 +177,51 @@ public:
        a program.
        @note This object should be destroyed after all other MFEM objects that
        use the Device are destroyed. */
-   Device(const std::string &device, const int dev = 0)
-   { Configure(device, dev); }
+   Device(const std::string &device, const int device_id = 0)
+   { Configure(device, device_id); }
 
    /// Destructor.
    ~Device();
 
    /// Configure the Device backends.
    /** The string parameter @a device must be a comma-separated list of backend
-       string names (see below). The @a dev argument specifies the ID of the
-       actual devices (e.g. GPU) to use.
-       * The available backends are described by the Backend class.
-       * The string name of a backend is the lowercase version of the
+       string names (see below). The @a device_id argument specifies the ID of
+       the actual devices (e.g. GPU) to use.
+       - The available backends are described by the Backend class.
+       - The string name of a backend is the lowercase version of the
          Backend::Id enumeration constant with '_' replaced by '-', e.g. the
          string name of 'RAJA_CPU' is 'raja-cpu'. The string name of the debug
          backend (Backend::Id 'DEBUG_DEVICE') is exceptionally set to 'debug'.
-       * The 'cpu' backend is always enabled with lowest priority.
-       * The current backend priority from highest to lowest is:
+       - The 'cpu' backend is always enabled with lowest priority.
+       - The current backend priority from highest to lowest is:
          'ceed-cuda', 'occa-cuda', 'raja-cuda', 'cuda',
          'ceed-hip', 'hip', 'debug',
          'occa-omp', 'raja-omp', 'omp',
          'ceed-cpu', 'occa-cpu', 'raja-cpu', 'cpu'.
-       * Multiple backends can be configured at the same time.
-       * Only one 'occa-*' backend can be configured at a time.
-       * The backend 'occa-cuda' enables the 'cuda' backend unless 'raja-cuda'
+       - The following backend aliases are also available: 'ceed-gpu',
+         'occa-gpu', 'raja-gpu', and 'gpu' where they alias their respective
+         '*-cuda' or '*-hip' backends depending on the MFEM build-time
+         configuration.
+       - Multiple backends can be configured at the same time.
+       - Only one 'occa-*' backend can be configured at a time.
+       - The backend 'occa-cuda' enables the 'cuda' backend unless 'raja-cuda'
          is already enabled.
-       * The backend 'occa-omp' enables the 'omp' backend (if MFEM was built
+       - The backend 'occa-omp' enables the 'omp' backend (if MFEM was built
          with MFEM_USE_OPENMP=YES) unless 'raja-omp' is already enabled.
-       * Only one 'ceed-*' backend can be configured at a time.
-       * The backend 'ceed-cpu' delegates to a libCEED CPU backend the setup and
+       - Only one 'ceed-*' backend can be configured at a time.
+       - The backend 'ceed-cpu' delegates to a libCEED CPU backend the setup and
          evaluation of the operator.
-       * The backend 'ceed-cuda' delegates to a libCEED CUDA backend the setup
+       - The backend 'ceed-cuda' delegates to a libCEED CUDA backend the setup
          and evaluation of operators and enables the 'cuda' backend to avoid
          transfers between host and device.
-       * The backend 'ceed-hip' delegates to a libCEED HIP backend the setup
+       - The backend 'ceed-hip' delegates to a libCEED HIP backend the setup
          and evaluation of operators and enables the 'hip' backend to avoid
          transfers between host and device.
-       * The 'debug' backend should not be combined with other device backends.
-   */
-   void Configure(const std::string &device, const int dev = 0);
+       - The 'debug' backend should not be combined with other device backends.
+
+       @note If the device is actually enabled, this method will also update the
+       current host/device MemoryType and MemoryClass. */
+   void Configure(const std::string &device, const int device_id = 0);
 
    /// Set the default host and device MemoryTypes, @a h_mt and @a d_mt.
    /** The host and device MemoryTypes are also set to be dual to each other.
@@ -235,7 +235,7 @@ public:
    static void SetMemoryTypes(MemoryType h_mt, MemoryType d_mt);
 
    /// Print the configuration of the MFEM virtual device object.
-   void Print(std::ostream &out = mfem::out);
+   void Print(std::ostream &os = mfem::out);
 
    /// Return true if Configure() has been called previously.
    static inline bool IsConfigured() { return Get().ngpu >= 0; }
@@ -244,14 +244,17 @@ public:
    static inline bool IsAvailable() { return Get().ngpu > 0; }
 
    /// Return true if any backend other than Backend::CPU is enabled.
-   static inline bool IsEnabled() { return Get().mode == ACCELERATED; }
+   static inline bool IsEnabled() { return Get().backends & ~(Backend::CPU); }
 
    /// The opposite of IsEnabled().
    static inline bool IsDisabled() { return !IsEnabled(); }
 
-   /// Get the device id of the configured device.
+   /// Get the device ID of the configured device.
    static inline int GetId() { return Get().dev; }
 
+   /// Get the number of available devices (may be called before configuration).
+   static int GetDeviceCount();
+
    /** @brief Return true if any of the backends in the backend mask, @a b_mask,
        are allowed. */
    /** This method can be used with any of the Backend::Id constants, the
@@ -285,25 +288,29 @@ public:
    /** @deprecated Use GetDeviceMemoryClass() instead. */
    static inline MemoryClass GetMemoryClass() { return Get().device_mem_class; }
 
+   /** @brief Manually set the status of GPU-aware MPI flag for use in MPI
+       communication routines which have optimized implementations for device
+       buffers. */
    static void SetGPUAwareMPI(const bool force = true)
    { Get().mpi_gpu_aware = force; }
 
+   /// Get the status of GPU-aware MPI flag.
    static bool GetGPUAwareMPI() { return Get().mpi_gpu_aware; }
 
    /** Query the device driver for what memory type a given @a ptr is allocated
     * with. */
-   static MemoryType QueryMemoryType(void* ptr);
+   static MemoryType QueryMemoryType(const void* ptr);
 
    /** @brief The number of hardware compute units/streaming multiprocessors
-    * available on a given compute device @a dev. */
-   static int NumMultiprocessors(int dev);
+       available on a given compute device @a device_id. */
+   static int NumMultiprocessors(int device_id);
 
    /// Same as NumMultiprocessors(int), for the currently active device.
    static int NumMultiprocessors();
 
-   /** @brief The number of threads in a warp on a given compute device @a dev.
-    */
-   static int WarpSize(int dev);
+   /** @brief The number of threads in a warp on a given compute device
+       @a device_id. */
+   static int WarpSize(int device_id);
 
    /// Same as WarpSize(int), for the currently active device.
    static int WarpSize();
@@ -320,7 +327,7 @@ public:
     and ReadWrite(), while setting the device use flag in @a mem, if @a on_dev
     is true. */
 template <typename T>
-MemoryClass GetMemoryClass(const Memory<T> &mem, bool on_dev)
+inline MemoryClass GetMemoryClass(const Memory<T> &mem, bool on_dev)
 {
    if (!on_dev)
    {
@@ -384,6 +391,6 @@ inline T *HostReadWrite(Memory<T> &mem, int size)
    return mfem::ReadWrite(mem, size, false);
 }
 
-} // mfem
+} // namespace mfem
 
 #endif // MFEM_DEVICE_HPP
