@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -107,6 +107,12 @@ void SetGlobalMPI_Comm(MPI_Comm comm);
 
 #endif
 
+/// @brief Wrapper for std::getenv.
+///
+/// @note Directly calling getenv causes a warning with MSVC. Use this wrapper
+/// to suppress the warning.
+const char* GetEnv(const char* name);
+
 } // namespace mfem
 
 #endif
