@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2023, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -15,7 +15,11 @@
 #include "../config/config.hpp"
 #include "array.hpp"
 #include "globals.hpp"
+
+#include <array>
+#include <cstdint>
 #include <type_traits>
+#include <utility>
 
 namespace mfem
 {
@@ -53,9 +57,9 @@ struct Hashed4
  *  each time this class is invoked.
  *
  *  There are two main methods this class provides. The Get(...) methods always
- *  return an item given the two or four indices. If the item didn't previously
+ *  return an item given the two or four indices. If the item did not previously
  *  exist, the methods creates a new one. The Find(...) methods, on the other
- *  hand, just return NULL or -1 if the item doesn't exist.
+ *  hand, just return NULL or -1 if the item does not exist.
  *
  *  Each new item is automatically assigned a unique ID - the index of the item
  *  inside the BlockArray. The IDs may (but need not) be used as p1, p2, ... of
@@ -91,14 +95,14 @@ public:
        @param[in] init_hash_size The initial size of the hash table. Must be
                                  a power of 2. */
    HashTable(int block_size = 16*1024, int init_hash_size = 32*1024);
-   /// @brief Deep copy
+   /// Deep copy
    HashTable(const HashTable& other);
-   /// @brief Copy assignment not supported
+   /// Copy assignment not supported
    HashTable& operator=(const HashTable&) = delete;
    ~HashTable();
 
-   /** @brief Item accessor with key (or parents) the pair 'p1', 'p2'. Default
-       construct an item of type T if no value correspond to the requested key.
+   /** @brief Item accessor with key (or parents) the pair p1, p2. Default
+       construct an item of type T if no value corresponds to the requested key.
 
        @param[in] p1 First part of the key.
        @param[in] p2 Second part of the key.
@@ -107,9 +111,9 @@ public:
        @warning This method should only be called if T inherits from Hashed2. */
    T* Get(int p1, int p2);
 
-   /** @brief Item accessor with key (or parents) the quadruplet 'p1', 'p2',
-       'p3', 'p4'. The key 'p4' is optional. Default construct an item of type T
-       if no value corresponds to the requested key.
+   /** @brief Item accessor with key (or parents) the quadruplet p1, p2, p3, p4.
+       The key p4 is optional. Default construct an item of type T if no value
+       corresponds to the requested key.
 
        @param[in] p1 First part of the key.
        @param[in] p2 Second part of the key.
@@ -120,10 +124,10 @@ public:
       @warning This method should only be called if T inherits from Hashed4. */
    T* Get(int p1, int p2, int p3, int p4 = -1 /* p4 optional */);
 
-   /// Get id of item whose parents are p1, p2... Create it if it doesn't exist.
-   /** @brief Get the "id" of an item, this "id" corresponding to the index of the
-       item in the underlying BlockArray<T> object. Default construct an item
-       and id if no value corresponds to the requested key.
+   /** @brief Get the "id" of the item whose parents are p1, p2, this "id"
+       corresponding to the index of the item in the underlying BlockArray<T>
+       object. Default construct an item and "id" if no value corresponds to the
+       requested key.
 
        @param[in] p1 First part of the key.
        @param[in] p2 Second part of the key.
@@ -132,9 +136,9 @@ public:
        @warning This method should only be called if T inherits from Hashed2. */
    int GetId(int p1, int p2);
 
-   /** @brief Get the "id" of an item, this "id" corresponding to the index of the
-       item in the underlying BlockArray<T> object. Default construct an item
-       and id if no value correspond to the requested key.
+   /** @brief Get the "id" of an item, this "id" corresponding to the index of
+       the item in the underlying BlockArray<T> object. Default construct an item
+       and "id" if no value corresponds to the requested key.
 
        @param[in] p1 First part of the key.
        @param[in] p2 Second part of the key.
@@ -145,9 +149,8 @@ public:
        @warning This method should only be called if T inherits from Hashed4. */
    int GetId(int p1, int p2, int p3, int p4 = -1);
 
-   /// Find item whose parents are p1, p2... Return NULL if it doesn't exist.
-   /** @brief Item accessor with key (or parents) the pair 'p1', 'p2'. Return
-       nullptr if no value correspond to the requested key.
+   /** @brief Item accessor with key (or parents) the pair p1, p2. Return
+       NULL if no value corresponds to the requested key.
 
        @param[in] p1 First part of the key.
        @param[in] p2 Second part of the key.
@@ -156,9 +159,9 @@ public:
        @warning This method should only be called if T inherits from Hashed2. */
    T* Find(int p1, int p2);
 
-   /** @brief Item accessor with key (or parents) the quadruplet 'p1', 'p2',
-       'p3', 'p4'. The key 'p4' is optional. Return nullptr if no value
-       correspond to the requested key.
+   /** @brief Item accessor with key (or parents) the quadruplet p1, p2, p3, p4.
+       The key p4 is optional. Return NULL if no value corresponds to the
+       requested key.
 
        @param[in] p1 First part of the key.
        @param[in] p2 Second part of the key.
@@ -169,8 +172,8 @@ public:
        @warning This method should only be called if T inherits from Hashed4. */
    T* Find(int p1, int p2, int p3, int p4 = -1);
 
-   /** @brief Item const accessor with key (or parents) the pair 'p1', 'p2'.
-       Return nullptr if no value correspond to the requested key.
+   /** @brief Item const accessor with key (or parents) the pair p1, p2.
+       Return NULL if no value corresponds to the requested key.
 
        @param[in] p1 First part of the key.
        @param[in] p2 Second part of the key.
@@ -179,9 +182,9 @@ public:
       @warning This method should only be called if T inherits from Hashed2. */
    const T* Find(int p1, int p2) const;
 
-   /** @brief Item const accessor with key (or parents) the quadruplet 'p1',
-       'p2', 'p3', 'p4'. The key 'p4' is optional. Return nullptr if no value
-       correspond to the requested key.
+   /** @brief Item const accessor with key (or parents) the quadruplet p1, p2,
+       p3, p4. The key p4 is optional. Return NULL if no value corresponds to the
+       requested key.
 
        @param[in] p1 First part of the key.
        @param[in] p2 Second part of the key.
@@ -192,10 +195,12 @@ public:
        @warning This method should only be called if T inherits from Hashed4. */
    const T* Find(int p1, int p2, int p3, int p4 = -1) const;
 
-   /// Find id of item whose parents are p1, p2... Return -1 if it doesn't exist.
-   /** @brief Find the "id" of an item, this "id" corresponding to the index of
-       the item in the underlying BlockArray<T> object. Default construct an
-       item and id if no value correspond to the requested key.
+   /** @brief Find the "id" of an item whose parents are p1, p2. Return -1 if it
+       does not exist.
+
+       This "id" corresponds to the index of the item in the underlying
+       BlockArray<T> object. Default construct an item and "id" if no value
+       corresponds to the requested key.
 
        @param[in] p1 First part of the key.
        @param[in] p2 Second part of the key.
@@ -205,8 +210,9 @@ public:
    int FindId(int p1, int p2) const;
 
    /** @brief Find the "id" of an item, this "id" corresponding to the index of
-       the item in the underlying BlockArray<T> object. Default construct an
-       item and id if no value correspond to the requested key.
+       the item in the underlying BlockArray<T> object. Return -1 if it does not
+       exist. Default construct an item and "id" if no value corresponds to the
+       requested key.
 
        @param[in] p1 First part of the key.
        @param[in] p2 Second part of the key.
@@ -217,16 +223,16 @@ public:
        @warning This method should only be called if T inherits from Hashed4. */
    int FindId(int p1, int p2, int p3, int p4 = -1) const;
 
-   /// @brief Return the number of elements currently stored in the HashTable.
+   /// Return the number of elements currently stored in the HashTable.
    int Size() const { return Base::Size() - unused.Size(); }
 
-   /// @brief Return the total number of ids (used and unused) in the HashTable.
+   /// Return the total number of ids (used and unused) in the HashTable.
    int NumIds() const { return Base::Size(); }
 
-   /// @brief Return the number of free/unused ids in the HashTable.
+   /// Return the number of free/unused ids in the HashTable.
    int NumFreeIds() const { return unused.Size(); }
 
-   /** @brief Return true if item 'id' exists in (is used by) the container.
+   /** @brief Return true if item @a id exists in (is used by) the container.
 
        @param[in] id Index of the item in the underlying BlockArray<T>.
 
@@ -237,13 +243,13 @@ public:
 
        @param[in] id Index of the item in the underlying BlockArray<T>.
 
-       @warning Its id will be reused by newly added items. */
+       @warning Its @a id will be reused by newly added items. */
    void Delete(int id);
 
-   /// @brief Remove all items.
+   /// Remove all items.
    void DeleteAll();
 
-   /** @brief Allocate an item at 'id'. Enlarge the underlying BlockArray if
+   /** @brief Allocate an item at @a id. Enlarge the underlying BlockArray if
        necessary.
 
        @param[in] id Index of the item in the underlying BlockArray<T>.
@@ -251,7 +257,7 @@ public:
        @param[in] p2 Second part of the key.
 
        @warning This is a special purpose method used when loading data from a
-       file. Does nothing if the slot 'id' has already been allocated. */
+       file. Does nothing if the slot @a id has already been allocated. */
    void Alloc(int id, int p1, int p2);
 
    /** @brief Reinitialize the internal list of unallocated items.
@@ -283,13 +289,13 @@ public:
        @warning This method should only be called if T inherits from Hashed4. */
    void Reparent(int id, int new_p1, int new_p2, int new_p3, int new_p4 = -1);
 
-   /// @brief Return total size of allocated memory (tables plus items), in bytes.
+   /// Return total size of allocated memory (tables plus items), in bytes.
    std::size_t MemoryUsage() const;
 
-   /// @brief Write details of the memory usage to the mfem output stream.
+   /// Write details of the memory usage to the mfem output stream.
    void PrintMemoryDetail() const;
 
-   /// @brief Print a histogram of bin sizes for debugging purposes.
+   /// Print a histogram of bin sizes for debugging purposes.
    void PrintStats() const;
 
    class iterator : public Base::iterator
@@ -334,13 +340,15 @@ public:
 
    iterator begin() { return iterator(Base::begin()); }
    iterator end() { return iterator(); }
+   const_iterator begin() const { return const_iterator(Base::cbegin()); }
+   const_iterator end() const { return const_iterator(); }
 
    const_iterator cbegin() const { return const_iterator(Base::cbegin()); }
    const_iterator cend() const { return const_iterator(); }
 
 protected:
    /** The hash table: each bin is a linked list of items. For each non-empty
-       bin, this arrays stores the 'id' of the first item in the list, or -1
+       bin, this arrays stores the "id" of the first item in the list, or -1
        if the bin is empty. */
    int* table;
 
@@ -378,11 +386,11 @@ protected:
    { return (984120265ul*p1 + 125965121ul*p2 + 495698413ul*p3) & mask; }
 
    // Delete() and Reparent() use one of these:
-   /// @brief Hash function for items of type T that inherit from Hashed2.
+   /// Hash function for items of type T that inherit from Hashed2.
    inline int Hash(const Hashed2& item) const
    { return Hash(item.p1, item.p2); }
 
-   /// @brief Hash function for items of type T that inherit from Hashed4.
+   /// Hash function for items of type T that inherit from Hashed4.
    inline int Hash(const Hashed4& item) const
    { return Hash(item.p1, item.p2, item.p3); }
 
@@ -409,15 +417,15 @@ protected:
        @warning This method should only be called if T inherits from Hashed4. */
    int SearchList(int id, int p1, int p2, int p3) const;
 
-   /** @brief Insert the item 'id' into bin 'idx'.
+   /** @brief Insert the item @a id into bin @a idx.
 
        @param[in] idx The bin/bucket index.
        @param[in] id The index of the item in the BlockArray<T>.
        @param[in] item The item to insert at the beginning of the linked list.
 
-       @warning The method only works with bin 'idx' and does not check the
-                overall fill factor of the hash table. If appropriate,
-                use CheckRehash() for that. */
+       @warning The method only works with bin @a idx and does not check the
+                overall fill factor of the hash table. If appropriate, use
+                CheckRehash() for that. */
    inline void Insert(int idx, int id, T &item);
 
    /** @brief Unlink an item @a id from the linked list of bin @a idx.
@@ -438,17 +446,85 @@ protected:
        and reinsert all items into the new bins.
 
        NOTE: Rehashing is computationally expensive (O(N) in the number of items),
-       but since it is only done rarely (when the number of items doubles),
-       the amortized complexity of inserting an item is still O(1). */
+       but since it is only done rarely (when the number of items doubles), the
+       amortized complexity of inserting an item is still O(1). */
    void DoRehash();
 
-   /** @brief Return the size of the bin "idx".
+   /** @brief Return the size of the bin @a idx.
 
        @param[in] idx The index of the bin.
        @return The size of the bin. */
    int BinSize(int idx) const;
 };
 
+///
+/// @brief streaming implementation for murmurhash3 128 (x64).
+/// Constructs the hash in 3 stages: init, append, finalize.
+///
+struct Hasher
+{
+   /// where the final hash result is stored after finalize. Use data[1] when
+   /// only 64 bits are required.
+   uint64_t data[2] = {0, 0};
+
+private:
+   uint64_t nbytes = 0;
+
+   uint64_t buf_[2] = {0, 0};
+
+public:
+
+   /// resets this hasher back to an initial seed
+   void init(uint64_t seed = 0);
+   void append(const uint8_t *vs, uint64_t bytes);
+
+   void finalize();
+
+private:
+   // add 16 bytes
+   void add_block(uint64_t k1, uint64_t k2);
+
+   // add [1-8] more bytes, then finalize
+   void finalize(uint64_t k1, int num);
+
+   // add [1-15] more bytes, then finalize
+   // 0 < num < 16
+   void finalize(uint64_t k1, uint64_t k2, int num);
+};
+
+/// Helper class for hashing std::pair. Usable in place of std::hash<std::pair<T,U>>
+struct PairHasher
+{
+   template <class T, class V>
+   size_t operator()(const std::pair<T, V> &v) const noexcept
+   {
+      Hasher hash;
+      // chosen randomly with a 2^64-sided dice
+      hash.init(0xfebd1fe69813c14full);
+      hash.append(reinterpret_cast<const uint8_t *>(&v.first), sizeof(T));
+      hash.append(reinterpret_cast<const uint8_t *>(&v.second), sizeof(V));
+      hash.finalize();
+      return hash.data[1];
+   }
+};
+
+/// Helper class for hashing std::array. Usable in place of std::hash<std::array<T,N>>
+struct ArrayHasher
+{
+   template <class T, size_t N>
+   size_t operator()(const std::array<T, N> &v) const noexcept
+   {
+      Hasher hash;
+      // chosen randomly with a 2^64-sided dice
+      hash.init(0xfebd1fe69813c14full);
+      for (size_t i = 0; i < N; ++i)
+      {
+         hash.append(reinterpret_cast<const uint8_t *>(&v[i]), sizeof(T));
+      }
+      hash.finalize();
+      return hash.data[1];
+   }
+};
 
 /// Hash function for data sequences.
 /** Depends on GnuTLS for SHA-256 hashing. */
@@ -505,14 +581,14 @@ public:
    /// Add a sequence of doubles for hashing, given as a c-array.
    /** Before hashing the sequence is encoded so that the result is independent
        of endianness. */
-   HashFunction &AppendDoubles(const double *doubles, size_t num_doubles)
+   HashFunction &AppendDoubles(const real_t *doubles, size_t num_doubles)
    { return EncodeAndHashDoubles(doubles, doubles + num_doubles); }
 
    /// Add a sequence of doubles for hashing, given as a fixed-size c-array.
    /** Before hashing the sequence is encoded so that the result is independent
        of endianness. */
    template <size_t num_doubles>
-   HashFunction &AppendDoubles(const double (&doubles)[num_doubles])
+   HashFunction &AppendDoubles(const real_t (&doubles)[num_doubles])
    { return EncodeAndHashDoubles(doubles, doubles + num_doubles); }
 
    /// Add a sequence of doubles for hashing, given as a container.
@@ -987,7 +1063,7 @@ HashFunction &HashFunction::EncodeAndHashDoubles(double_const_iter begin,
    // For hashing, a double is encoded in little endian byte-order.
 
    static_assert(
-      std::is_same<decltype(*begin), const double &>::value,
+      std::is_same<decltype(*begin), const real_t &>::value,
       "invalid iterator type");
 
    // Skip encoding if hashing is not available:
