@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -108,7 +108,7 @@ public:
    void print_on_error(const char *msg) const
    {
       if (good()) { return; }
-      mfem::out << "Error in " << msg << ": " << gnutls_strerror(res)
+      mfem::err << "Error in " << msg << ": " << gnutls_strerror(res)
                 << std::endl;
    }
 };
@@ -186,23 +186,23 @@ public:
 
    /** Attach a new socket descriptor to the socketbuf. Returns the old socket
        descriptor which is NOT closed. */
-   virtual int attach(int sd);
+   int attach(int sd) override;
 
-   virtual int open(const char hostname[], int port);
+   int open(const char hostname[], int port) override;
 
-   virtual int close();
+   int close() override;
 
 protected:
-   virtual int sync();
+   int sync() override;
 
-   virtual int_type underflow();
+   int_type underflow() override;
 
    // Same as in the base class:
    // virtual int_type overflow(int_type c = traits_type::eof());
 
-   virtual std::streamsize xsgetn(char_type *s__, std::streamsize n__);
+   std::streamsize xsgetn(char_type *s__, std::streamsize n__) override;
 
-   virtual std::streamsize xsputn(const char_type *s__, std::streamsize n__);
+   std::streamsize xsputn(const char_type *s__, std::streamsize n__) override;
 };
 
 #endif // MFEM_USE_GNUTLS
