@@ -12,7 +12,8 @@
 #ifndef MFEM_DTENSOR
 #define MFEM_DTENSOR
 
-#include "../general/backends.hpp"
+#include "../config/config.hpp"
+#include "../general/error.hpp"
 
 namespace mfem
 {
@@ -42,7 +43,8 @@ public:
    static inline int result(const int* sizes, T first, Args... args)
    {
 #if !(defined(MFEM_USE_CUDA) || defined(MFEM_USE_HIP))
-      MFEM_ASSERT(first<sizes[Dim-1],"Trying to access out of boundary.");
+      MFEM_ASSERT(first<static_cast<T>(sizes[Dim-1]),
+                  "Trying to access out of boundary.");
 #endif
       return static_cast<int>(first);
    }
@@ -87,7 +89,9 @@ protected:
 
 public:
    /// Default constructor
-   DeviceTensor() = delete;
+   // DeviceTensor() = delete;
+   MFEM_HOST_DEVICE
+   DeviceTensor() {}
 
    /// Constructor to initialize a tensor from the Scalar array data_
    template <typename... Args> MFEM_HOST_DEVICE
@@ -97,7 +101,7 @@ public:
       // Initialize sizes, and compute the number of values
       const long int nb = Init<1, Dim, Args...>::result(sizes, args...);
       capacity = nb;
-      data = (capacity > 0) ? data_ : NULL;
+      data = (capacity > 0) ? data_ : nullptr;
    }
 
    /// Copy constructor (default)
@@ -122,6 +126,9 @@ public:
    {
       return data[i];
    }
+
+   /// Returns the shape of the tensor.
+   MFEM_HOST_DEVICE inline auto &GetShape() const { return sizes; }
 };
 
 
@@ -134,17 +141,17 @@ inline DeviceTensor<sizeof...(Dims),T> Reshape(T *ptr, Dims... dims)
 }
 
 
-typedef DeviceTensor<1,int> DeviceArray;
-typedef DeviceTensor<1,const int> ConstDeviceArray;
+using DeviceArray = DeviceTensor<1,int>;
+using ConstDeviceArray = DeviceTensor<1,const int>;
 
-typedef DeviceTensor<1,real_t> DeviceVector;
-typedef DeviceTensor<1,const real_t> ConstDeviceVector;
+using DeviceVector = DeviceTensor<1,real_t>;
+using ConstDeviceVector = DeviceTensor<1,const real_t>;
 
-typedef DeviceTensor<2,real_t> DeviceMatrix;
-typedef DeviceTensor<2,const real_t> ConstDeviceMatrix;
+using DeviceMatrix = DeviceTensor<2,real_t>;
+using ConstDeviceMatrix = DeviceTensor<2,const real_t>;
 
-typedef DeviceTensor<3,real_t> DeviceCube;
-typedef DeviceTensor<3,const real_t> ConstDeviceCube;
+using DeviceCube = DeviceTensor<3,real_t>;
+using ConstDeviceCube = DeviceTensor<3,const real_t>;
 
 } // mfem namespace
 
