@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -17,8 +17,8 @@
 
 // Make sure that hypre and PETSc use the same size indices.
 #if defined(MFEM_USE_MPI) && defined(MFEM_USE_PETSC)
-#if (defined(HYPRE_BIGINT) && !defined(PETSC_USE_64BIT_INDICES)) || \
-    (!defined(HYPRE_BIGINT) && defined(PETSC_USE_64BIT_INDICES))
+#if ((defined(HYPRE_BIGINT) || defined(HYPRE_MIXEDINT)) && !defined(PETSC_USE_64BIT_INDICES)) || \
+    (!defined(HYPRE_BIGINT) && !defined(HYPRE_MIXEDINT) && defined(PETSC_USE_64BIT_INDICES))
 #error HYPRE and PETSC do not use the same size integers!
 #endif
 #endif
@@ -44,6 +44,7 @@ Operator::Type OperatorHandle::CheckType(Operator::Type tid)
 #endif
       case Operator::PETSC_MATAIJ:
       case Operator::PETSC_MATIS:
+      case Operator::PETSC_MATGENERIC:
 #ifdef MFEM_USE_PETSC
          break;
 #else
@@ -148,6 +149,7 @@ void OperatorHandle::MakePtAP(OperatorHandle &A, OperatorHandle &P)
 #ifdef MFEM_USE_PETSC
       case Operator::PETSC_MATAIJ:
       case Operator::PETSC_MATIS:
+      case Operator::PETSC_MATGENERIC:
       {
          pSet(mfem::RAP(A.As<PetscParMatrix>(), P.As<PetscParMatrix>()));
          break;
@@ -186,6 +188,7 @@ void OperatorHandle::MakeRAP(OperatorHandle &Rt, OperatorHandle &A,
 #ifdef MFEM_USE_PETSC
       case Operator::PETSC_MATAIJ:
       case Operator::PETSC_MATIS:
+      case Operator::PETSC_MATGENERIC:
       {
          pSet(mfem::RAP(Rt.As<PetscParMatrix>(), A.As<PetscParMatrix>(),
                         P.As<PetscParMatrix>()));
@@ -288,6 +291,7 @@ void OperatorHandle::EliminateRowsCols(OperatorHandle &A,
       }
       case Operator::PETSC_MATAIJ:
       case Operator::PETSC_MATIS:
+      case Operator::PETSC_MATGENERIC:
       {
 #ifdef MFEM_USE_PETSC
          pSet(A.As<PetscParMatrix>()->EliminateRowsCols(ess_dof_list));
@@ -368,6 +372,7 @@ void OperatorHandle::EliminateBC(const OperatorHandle &A_e,
       }
       case Operator::PETSC_MATAIJ:
       case Operator::PETSC_MATIS:
+      case Operator::PETSC_MATGENERIC:
       {
 #ifdef MFEM_USE_PETSC
          mfem::EliminateBC(*As<PetscParMatrix>(), *A_e.As<PetscParMatrix>(),
