@@ -13,10 +13,7 @@
 #define MFEM_LINALG_KERNELS_HPP
 
 #include "../config/config.hpp"
-#include "../general/backends.hpp"
-#include "../general/globals.hpp"
 
-#include "matrix.hpp"
 #include "tmatrix.hpp"
 #include "tlayout.hpp"
 #include "ttensor.hpp"
@@ -188,6 +185,40 @@ void Mult(const int height, const int width, const TA *data, const TX *x, TY *y)
    }
 }
 
+/** @brief Absolute-value matrix vector multiplication: y = |A| x, where the
+    matrix A is of size @a height x @a width with given @a data, while @a x and
+    @a y specify the data of the input and output vectors. */
+template<typename TA, typename TX, typename TY>
+MFEM_HOST_DEVICE inline
+void AbsMult(const int height, const int width, const TA *data,
+             const TX *x, TY *y)
+{
+   if (width == 0)
+   {
+      for (int row = 0; row < height; row++)
+      {
+         y[row] = 0.0;
+      }
+      return;
+   }
+   const TA *d_col = data;
+   TX x_col = x[0];
+   for (int row = 0; row < height; row++)
+   {
+      y[row] = x_col*std::fabs(d_col[row]);
+   }
+   d_col += height;
+   for (int col = 1; col < width; col++)
+   {
+      x_col = x[col];
+      for (int row = 0; row < height; row++)
+      {
+         y[row] += x_col*std::fabs(d_col[row]);
+      }
+      d_col += height;
+   }
+}
+
 /** @brief Matrix transpose vector multiplication: y = At x, where the matrix A
     is of size @a height x @a width with given @a data, while @a x and @a y
     specify the data of the input and output vectors. */
@@ -217,6 +248,35 @@ void MultTranspose(const int height, const int width, const TA *data,
    }
 }
 
+/** @brief Absolute-value matrix transpose vector multiplication: y = |At| x,
+    where the matrix A is of size @a height x @a width with given @a data, while
+    @a x and @a y specify the data of the input and output vectors. */
+template<typename TA, typename TX, typename TY>
+MFEM_HOST_DEVICE inline
+void AbsMultTranspose(const int height, const int width, const TA *data,
+                      const TX *x, TY *y)
+{
+   if (height == 0)
+   {
+      for (int row = 0; row < width; row++)
+      {
+         y[row] = 0.0;
+      }
+      return;
+   }
+   TY *y_off = y;
+   for (int i = 0; i < width; ++i)
+   {
+      TY val = 0.0;
+      for (int j = 0; j < height; ++j)
+      {
+         val += x[j] * std::fabs(data[i * height + j]);
+      }
+      *y_off = val;
+      y_off++;
+   }
+}
+
 /// Symmetrize a square matrix with given @a size and @a data: A -> (A+A^T)/2.
 template<typename T>
 MFEM_HOST_DEVICE inline
@@ -1702,7 +1762,7 @@ inline void LSolve(const real_t *data, const int m, const int *ipiv, real_t *x)
    // X <- P X
    for (int i = 0; i < m; i++)
    {
-      internal::Swap<real_t>(x[i], x[ipiv[i]]);
+      internal::Swap<real_t>(x[i], x[ipiv[i] - 1]);
    }
    // X <- L^{-1} X
    for (int j = 0; j < m; j++)
@@ -1841,7 +1901,7 @@ inline bool LUFactor(real_t *A, const int m, int *ipiv, const real_t tol=0.0)
                piv = j;
             }
          }
-         ipiv[i] = piv;
+         ipiv[i] = piv + 1;
          if (piv != i)
          {
             // swap rows i and piv in both L and U parts
