@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -204,9 +204,9 @@ void MUMPSSolver::SetOperator(const Operator &op)
          delete id;
       }
 #ifdef MFEM_USE_SINGLE
-      id = new SMUMPS_STRUC_C;
+      id = new SMUMPS_STRUC_C();
 #else
-      id = new DMUMPS_STRUC_C;
+      id = new DMUMPS_STRUC_C();
 #endif
       id->sym = mat_type;
 
@@ -341,7 +341,7 @@ void MUMPSSolver::InitRhsSol(int nrhs) const
 #else
       if (myid == 0)
       {
-         delete rhs_glob;
+         delete [] rhs_glob;
          rhs_glob = new real_t[nrhs * id->lrhs];
          id->rhs = rhs_glob;
       }
