@@ -106,6 +106,17 @@ public:
 
 
 /// Abstract class for solving systems of ODEs: dx/dt = f(x,t)
+/** For systems of split ODEs:
+    $$ M dx/dt = f_1(x,t) + f_2(x,t) $$
+    where $ M^{-1} f_1 $ and $ M^{-1} f_2 $ are treated differently (e.g.,
+    explicitly and implicitly), the solver class expects a
+    TimeDependentOperator with split functionality. Setting
+    TimeDependentOperator::EvalMode = TimeDependentOperator::ADDITIVE_TERM_1
+    and calling TimeDependentOperator::Mult() should return
+    $ k_1=M^{-1} f_1(x,t) $. Setting TimeDependentOperator::EvalMode =
+    TimeDependentOperator::ADDITIVE_TERM_2 and calling
+    TimeDependentOperator::ImplicitSolve() should solve
+    $ M k_2 = f_2(x+\gamma k_2,t) $. */
 class ODESolver
 {
 protected:
@@ -184,6 +195,7 @@ public:
    // Help info for ODESolver options
    static MFEM_EXPORT std::string ExplicitTypes;
    static MFEM_EXPORT std::string ImplicitTypes;
+   static MFEM_EXPORT std::string IMEXTypes;
    static MFEM_EXPORT std::string Types;
 
    /// Function for selecting the desired ODESolver (Explicit and Implicit)
@@ -203,6 +215,13 @@ public:
    static MFEM_EXPORT std::unique_ptr<ODESolver> SelectImplicit(
       const int ode_solver_type);
 
+
+   /// Function for selecting the desired IMEX ODESolver
+   /// Returns an ODESolver pointer based on an type
+   /// Caller gets ownership of the object and is responsible for its deletion
+   static MFEM_EXPORT std::unique_ptr<ODESolver> SelectIMEX(
+      const int ode_solver_type);
+
    virtual ~ODESolver() { }
 };
 
@@ -931,6 +950,70 @@ public:
 
 };
 
+/// Forward-backward Euler method
+class IMEXExpImplEuler : public ODESolver
+{
+private:
+   Vector k1; Vector k2;
+public:
+   void Init(TimeDependentOperator &f_) override;
+
+   void Step(Vector &x, real_t &t, real_t &dt) override;
+};
+
+/// Second order, two-stage implicit-explicit (IMEX) Runge-Kutta (RK) method
+/** L-stable IMEX RK2 method adopted from "On the Stability of IMEX Upwind gSBP
+    Schemes for 1D Linear Advection‑Difusion Equations" by Sigrun Ortleb. Same
+    as (2,2,2) from "Implicit-explicit Runge-Kutta methods for time-dependent
+    partial differential equations" by Ascher, Ruuth and Spiteri, Applied
+    Numerical Mathematics (1997). */
+class IMEXRK2 : public ODESolver
+{
+private:
+   Vector k1_exp; Vector k2_exp; Vector k_imp;
+   //helper vector
+   Vector y;
+public:
+   void Init(TimeDependentOperator &f_) override;
+
+   void Step(Vector &x, real_t &t, real_t &dt) override;
+};
+
+/// Second order, 2/3-stage implicit-explicit (IMEX) Runge-Kutta (RK) method
+/** L-stable method (2,3,2) from "Implicit-explicit Runge-Kutta methods for
+    time-dependent partial differential equations" by Ascher, Ruuth and
+    Spiteri, Applied Numerical Mathematics (1997). */
+class IMEXRK2_3StageExplicit : public ODESolver
+{
+private:
+   Vector k1_exp; Vector k2_exp; Vector k3_exp;
+   Vector k_imp;
+   //helper vectors
+   Vector y;
+public:
+   void Init(TimeDependentOperator &f_) override;
+
+   void Step(Vector &x, real_t &t, real_t &dt) override;
+};
+
+/// Third order, 3/4-stage implicit-explicit (IMEX) Runge-Kutta (RK) method
+/** L-stable method (3,4,3) from "Implicit-explicit Runge-Kutta methods for
+    time-dependent partial differential equations" by Ascher, Ruuth and
+    Spiteri, Applied Numerical Mathematics (1997). */
+class IMEX_DIRK_RK3 : public ODESolver
+{
+private:
+   Vector k1_exp; Vector k2_exp; Vector k3_exp; Vector k4_exp;
+   Vector k2_imp; Vector k3_imp;
+   //helper vectors
+   Vector y;
+public:
+   void Init(TimeDependentOperator &f_) override;
+
+   void Step(Vector &x, real_t &t, real_t &dt) override;
+};
+
+
 }
 
 #endif
