@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -98,9 +98,17 @@ protected:
 #ifdef MFEM_USE_CUDA_OR_HIP
    // common for hipSPARSE and cuSPARSE
    static int SparseMatrixCount;
+   mutable bool initBuffers = false;
+
+#if defined(MFEM_USE_CUDA) && CUDA_VERSION >= 12300 && CUDA_VERSION < 12602
+   // Workaround for bug CUSPARSE-1897
+#define MFEM_CUDA_1897_WORKAROUND
+   mutable size_t bufferSize = 0;
+   mutable void *dBuffer = nullptr;
+#else
    static size_t bufferSize;
    static void *dBuffer;
-   mutable bool initBuffers = false;
+#endif
 
 #if defined(MFEM_USE_CUDA)
    cusparseStatus_t status;
@@ -216,7 +224,7 @@ public:
    void ClearCuSparse() { ClearGPUSparse(); }
 
    /// Check if the SparseMatrix is empty.
-   bool Empty() const { return (A == NULL) && (Rows == NULL); }
+   bool Empty() const { return A.Empty() && (Rows == NULL); }
 
    /// Return the array #I.
    inline int *GetI() { return I; }
@@ -319,10 +327,10 @@ public:
    void MoveDiagonalFirst();
 
    /// Returns reference to a_{ij}.
-   virtual real_t &Elem(int i, int j);
+   real_t &Elem(int i, int j) override;
 
    /// Returns constant reference to a_{ij}.
-   virtual const real_t &Elem(int i, int j) const;
+   const real_t &Elem(int i, int j) const override;
 
    /// Returns reference to A[i][j].
    real_t &operator()(int i, int j);
@@ -339,31 +347,31 @@ public:
    /// Produces a DenseMatrix from a SparseMatrix
    void ToDenseMatrix(DenseMatrix & B) const;
 
-   virtual MemoryClass GetMemoryClass() const
+   MemoryClass GetMemoryClass() const override
    {
       return Finalized() ?
              Device::GetDeviceMemoryClass() : Device::GetHostMemoryClass();
    }
 
    /// Matrix vector multiplication.
-   virtual void Mult(const Vector &x, Vector &y) const;
+   void Mult(const Vector &x, Vector &y) const override;
 
    /// y += A * x (default)  or  y += a * A * x
-   virtual void AddMult(const Vector &x, Vector &y,
-                        const real_t a = 1.0) const;
+   void AddMult(const Vector &x, Vector &y,
+                const real_t a = 1.0) const override;
 
    /// Multiply a vector with the transposed matrix. y = At * x
    /** If the matrix is modified, call ResetTranspose() and optionally
        EnsureMultTranspose() to make sure this method uses the correct updated
        transpose. */
-   virtual void MultTranspose(const Vector &x, Vector &y) const;
+   void MultTranspose(const Vector &x, Vector &y) const override;
 
    /// y += At * x (default)  or  y += a * At * x
    /** If the matrix is modified, call ResetTranspose() and optionally
        EnsureMultTranspose() to make sure this method uses the correct updated
        transpose. */
-   virtual void AddMultTranspose(const Vector &x, Vector &y,
-                                 const real_t a = 1.0) const;
+   void AddMultTranspose(const Vector &x, Vector &y,
+                         const real_t a = 1.0) const override;
 
    /** @brief Build and store internally the transpose of this matrix which will
        be used in the methods AddMultTranspose(), MultTranspose(), and
@@ -422,13 +430,13 @@ public:
    void BooleanMultTranspose(const Array<int> &x, Array<int> &y) const;
 
    /// y = |A| * x, using entry-wise absolute values of matrix A
-   void AbsMult(const Vector &x, Vector &y) const;
+   void AbsMult(const Vector &x, Vector &y) const override;
 
    /// y = |At| * x, using entry-wise absolute values of the transpose of matrix A
    /** If the matrix is modified, call ResetTranspose() and optionally
        EnsureMultTranspose() to make sure this method uses the correct updated
        transpose. */
-   void AbsMultTranspose(const Vector &x, Vector &y) const;
+   void AbsMultTranspose(const Vector &x, Vector &y) const override;
 
    /// Compute y^t A x
    real_t InnerProduct(const Vector &x, const Vector &y) const;
@@ -439,7 +447,7 @@ public:
    real_t GetRowNorml1(int irow) const;
 
    /// This virtual method is not supported: it always returns NULL.
-   virtual MatrixInverse *Inverse() const;
+   MatrixInverse *Inverse() const override;
 
    /// Eliminates a column from the transpose matrix.
    void EliminateRow(int row, const real_t sol, Vector &rhs);
@@ -512,7 +520,7 @@ public:
    /// If a row contains only one diag entry of zero, set it to 1.
    void SetDiagIdentity();
    /// If a row contains only zeros, set its diagonal to 1.
-   virtual void EliminateZeroRows(const real_t threshold = 1e-12);
+   void EliminateZeroRows(const real_t threshold = 1e-12) override;
 
    /// Gauss-Seidel forward and backward iterations over a vector x.
    void Gauss_Seidel_forw(const Vector &x, Vector &y) const;
@@ -543,7 +551,7 @@ public:
    /** This method should be called once, after the matrix has been initialized.
        Internally, this method converts the matrix from row-wise linked list
        (LIL) format into CSR (compressed sparse row) format. */
-   virtual void Finalize(int skip_zeros = 1) { Finalize(skip_zeros, false); }
+   void Finalize(int skip_zeros = 1) override { Finalize(skip_zeros, false); }
 
    /// A slightly more general version of the Finalize(int) method.
    void Finalize(int skip_zeros, bool fix_empty_rows);
@@ -626,7 +634,7 @@ public:
          when the matrix is finalized.
        @warning This method breaks the const-ness when the matrix is finalized
        because it gives write access to the #J and #A arrays. */
-   virtual int GetRow(const int row, Array<int> &cols, Vector &srow) const;
+   int GetRow(const int row, Array<int> &cols, Vector &srow) const override;
 
    void SetRow(const int row, const Array<int> &cols, const Vector &srow);
    void AddRow(const int row, const Array<int> &cols, const Vector &srow);
@@ -651,11 +659,24 @@ public:
 
    /// Prints matrix to stream out.
    /** @note The host in synchronized when the finalized matrix is on the device. */
-   void Print(std::ostream &out = mfem::out, int width_ = 4) const;
+   void Print(std::ostream &out = mfem::out, int width_ = 4) const override;
 
    /// Prints matrix in matlab format.
    /** @note The host in synchronized when the finalized matrix is on the device. */
-   virtual void PrintMatlab(std::ostream &out = mfem::out) const;
+   void PrintMatlab(std::ostream &out = mfem::out) const override;
+
+   /// Prints matrix as a SparseArray for importing into Mathematica.
+   /** The resulting file can be read into Mathematica using an expression such
+       as: myMat = Get["output_file_name"]
+       The Mathematica variable "myMat" will then be assigned to a new
+       SparseArray object containing the data from this MFEM SparseMatrix.
+
+       @note Mathematica uses 1-based indexing so the MFEM row and column
+       indices will be sifted up by one in the Mathematica output.
+
+       @note The host in synchronized when the finalized matrix is on the
+       device. */
+   virtual void PrintMathematica(std::ostream &out = mfem::out) const;
 
    /// Prints matrix in Matrix Market sparse format.
    /** @note The host in synchronized when the finalized matrix is on the device. */
@@ -679,7 +700,7 @@ public:
    void Symmetrize();
 
    /// Returns the number of the nonzero elements in the matrix
-   virtual int NumNonZeroElems() const;
+   int NumNonZeroElems() const override;
 
    real_t MaxNorm() const;
 
