@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -54,6 +54,10 @@
 
 #include <functional>
 
+#define MFEM_SUNDIALS_VERSION \
+   (SUNDIALS_VERSION_MAJOR*10000 + SUNDIALS_VERSION_MINOR*100 + \
+    SUNDIALS_VERSION_PATCH)
+
 #if (SUNDIALS_VERSION_MAJOR < 6)
 
 /// (DEPRECATED) Map SUNDIALS version >= 6 datatypes and constants to
@@ -68,7 +72,30 @@ constexpr ARKODE_ERKTableID ARKODE_FEHLBERG_13_7_8 = FEHLBERG_13_7_8;
 /// arbitrary type for more compact backwards compatibility
 using SUNContext = void*;
 
-#endif // SUNDIALS_VERSION_MAJOR < 6
+/// 'sunrealtype' was first introduced in v6.0.0
+typedef realtype sunrealtype;
+/// 'sunbooleantype' was first introduced in v6.0.0
+typedef booleantype sunbooleantype;
+
+/// New constant names introduced in v6.0.0
+enum { SUN_PREC_NONE, SUN_PREC_LEFT, SUN_PREC_RIGHT, SUN_PREC_BOTH };
+
+// KIN_ORTH_MGS was introduced in SUNDIALS v6; here, we define it just so that
+// it can be used as the default option in the second parameter of
+// KINSolver::EnableAndersonAcc -- the actual value of the parameter will be
+// ignored when using SUNDIALS < v6.
+#define KIN_ORTH_MGS 0
+
+#endif // #if SUNDIALS_VERSION_MAJOR < 6
+
+#if (SUNDIALS_VERSION_MAJOR < 7)
+
+/** @brief The enum constant SUN_SUCCESS was added in v7 as a replacement of
+    various *_SUCCESS macros that were removed in v7. */
+enum { SUN_SUCCESS = 0 };
+
+#endif // #if SUNDIALS_VERSION_MAJOR < 7
+
 
 namespace mfem
 {
@@ -155,10 +182,19 @@ public:
    /// Disable copy assignment
    void operator=(const Sundials &other) = delete;
 
-   /// Initializes SUNContext and SundialsMemHelper objects. Should be called at
+   /// @brief Initializes SUNContext and SundialsMemHelper objects. Should be called at
    /// the beginning of the calling program (after Mpi::Init if applicable)
+   ///
+   /// Multiple calls to Sundials::Init() have no effect.
    static void Init();
 
+   /// @brief Finalize sundials (called automatically at program exit if
+   /// Sundials::Init() has been called).
+   ///
+   /// Multiple calls to Sundials::Finalize() have no effect. This function can be
+   /// called manually to more precisely control when sundials is finalized.
+   static void Finalize();
+
    /// Provides access to the SUNContext object
    static SUNContext &GetContext();
 
@@ -175,6 +211,7 @@ private:
    /// Destructor called at end of calling program (does nothing for version < 6)
    ~Sundials();
 
+   bool isInitialized;
    SUNContext context;
    SundialsMemHelper memHelper;
 };
@@ -238,7 +275,14 @@ public:
 
 #ifdef MFEM_USE_MPI
    /// Returns the MPI communicator for the internal N_Vector x.
-   inline MPI_Comm GetComm() const { return *static_cast<MPI_Comm*>(N_VGetCommunicator(x)); }
+   inline MPI_Comm GetComm() const
+   {
+#if SUNDIALS_VERSION_MAJOR < 7
+      return *static_cast<MPI_Comm*>(N_VGetCommunicator(x));
+#else
+      return N_VGetCommunicator(x);
+#endif
+   }
 
    /// Returns the MPI global length for the internal N_Vector x.
    inline long GlobalSize() const { return N_VGetLength(x); }
@@ -279,7 +323,7 @@ public:
    /// Typecasting to SUNDIALS' N_Vector type
    operator N_Vector() const { return x; }
 
-   /// Changes the ownership of the the vector
+   /// Changes the ownership of the vector
    N_Vector StealNVector() { own_NVector = 0; return x; }
 
    /// Sets ownership of the internal N_Vector
@@ -390,24 +434,26 @@ protected:
    int root_components; /// Number of components in gout
 
    /// Wrapper to compute the ODE rhs function.
-   static int RHS(realtype t, const N_Vector y, N_Vector ydot, void *user_data);
+   static int RHS(sunrealtype t, const N_Vector y, N_Vector ydot,
+                  void *user_data);
 
    /// Setup the linear system $ A x = b $.
-   static int LinSysSetup(realtype t, N_Vector y, N_Vector fy, SUNMatrix A,
-                          booleantype jok, booleantype *jcur,
-                          realtype gamma, void *user_data, N_Vector tmp1,
+   static int LinSysSetup(sunrealtype t, N_Vector y, N_Vector fy, SUNMatrix A,
+                          sunbooleantype jok, sunbooleantype *jcur,
+                          sunrealtype gamma, void *user_data, N_Vector tmp1,
                           N_Vector tmp2, N_Vector tmp3);
 
    /// Solve the linear system $ A x = b $.
    static int LinSysSolve(SUNLinearSolver LS, SUNMatrix A, N_Vector x,
-                          N_Vector b, realtype tol);
+                          N_Vector b, sunrealtype tol);
 
    /// Prototype to define root finding for CVODE
-   static int root(realtype t, N_Vector y, realtype *gout, void *user_data);
+   static int root(sunrealtype t, N_Vector y, sunrealtype *gout,
+                   void *user_data);
 
    /// Typedef for root finding functions
-   typedef std::function<int(realtype t, Vector y, Vector gout, CVODESolver *)>
-   RootFunction;
+   typedef std::function<int(sunrealtype t, Vector y, Vector gout,
+                             CVODESolver *)> RootFunction;
 
    /// A class member to facilitate pointing to a user-specified root function
    RootFunction root_func;
@@ -415,7 +461,8 @@ protected:
    /// Typedef declaration for error weight functions
    typedef std::function<int(Vector y, Vector w, CVODESolver*)> EWTFunction;
 
-   /// A class member to facilitate pointing to a user-specified error weight function
+   /** @brief A class member to facilitate pointing to a user-specified error
+       weight function */
    EWTFunction ewt_func;
 
 public:
@@ -449,7 +496,7 @@ public:
        @note If this method is called a second time with a different problem
        size, then any non-default user-set options will be lost and will need
        to be set again. */
-   void Init(TimeDependentOperator &f_);
+   void Init(TimeDependentOperator &f_) override;
 
    /// Integrate the ODE with CVODE using the specified step mode.
    /** @param[in,out] x  On output, the solution vector at the requested output
@@ -460,7 +507,7 @@ public:
        @note On input, the values of @a t and @a dt are used to compute desired
        output time for the integration, tout = @a t + @a dt.
    */
-   virtual void Step(Vector &x, double &t, double &dt);
+   void Step(Vector &x, double &t, double &dt) override;
 
    /** @brief Attach the linear system setup and solve methods from the
        TimeDependentOperator i.e., SUNImplicitSetup() and SUNImplicitSolve() to
@@ -525,14 +572,15 @@ protected:
    int indexB; ///< backward problem index
 
    /// Wrapper to compute the ODE RHS Quadrature function.
-   static int RHSQ(realtype t, const N_Vector y, N_Vector qdot, void *user_data);
+   static int RHSQ(sunrealtype t, const N_Vector y, N_Vector qdot,
+                   void *user_data);
 
    /// Wrapper to compute the ODE RHS backward function.
-   static int RHSB(realtype t, N_Vector y,
+   static int RHSB(sunrealtype t, N_Vector y,
                    N_Vector yB, N_Vector yBdot, void *user_dataB);
 
    /// Wrapper to compute the ODE RHS Backwards Quadrature function.
-   static int RHSQB(realtype t, N_Vector y, N_Vector yB,
+   static int RHSQB(sunrealtype t, N_Vector y, N_Vector yB,
                     N_Vector qBdot, void *user_dataB);
 
    /// Error control function
@@ -586,7 +634,7 @@ public:
 
        @note On input, the values of t and dt are used to compute desired
        output time for the integration, tout = t + dt. */
-   virtual void Step(Vector &x, double &t, double &dt);
+   void Step(Vector &x, double &t, double &dt) override;
 
    /// Solve one adjoint time step
    virtual void StepB(Vector &w, double &t, double &dt);
@@ -648,15 +696,15 @@ public:
    void SetSVtolerancesB(double reltol, Vector abstol);
 
    /// Setup the linear system A x = b
-   static int LinSysSetupB(realtype t, N_Vector y, N_Vector yB, N_Vector fyB,
+   static int LinSysSetupB(sunrealtype t, N_Vector y, N_Vector yB, N_Vector fyB,
                            SUNMatrix A,
-                           booleantype jok, booleantype *jcur,
-                           realtype gamma, void *user_data, N_Vector tmp1,
+                           sunbooleantype jok, sunbooleantype *jcur,
+                           sunrealtype gamma, void *user_data, N_Vector tmp1,
                            N_Vector tmp2, N_Vector tmp3);
 
    /// Solve the linear system A x = b
    static int LinSysSolveB(SUNLinearSolver LS, SUNMatrix A, N_Vector x,
-                           N_Vector b, realtype tol);
+                           N_Vector b, sunrealtype tol);
 
 
    /// Destroy the associated CVODES memory and SUNDIALS objects.
@@ -689,33 +737,35 @@ protected:
        RHS1 is explicit RHS and RHS2 the implicit RHS for IMEX integration. When
        purely implicit or explicit only RHS1 is used. */
    ///@{
-   static int RHS1(realtype t, const N_Vector y, N_Vector ydot, void *user_data);
-   static int RHS2(realtype t, const N_Vector y, N_Vector ydot, void *user_data);
+   static int RHS1(sunrealtype t, const N_Vector y, N_Vector ydot,
+                   void *user_data);
+   static int RHS2(sunrealtype t, const N_Vector y, N_Vector ydot,
+                   void *user_data);
    ///@}
 
    /// Setup the linear system $ A x = b $.
-   static int LinSysSetup(realtype t, N_Vector y, N_Vector fy, SUNMatrix A,
-                          SUNMatrix M, booleantype jok, booleantype *jcur,
-                          realtype gamma, void *user_data, N_Vector tmp1,
+   static int LinSysSetup(sunrealtype t, N_Vector y, N_Vector fy, SUNMatrix A,
+                          SUNMatrix M, sunbooleantype jok, sunbooleantype *jcur,
+                          sunrealtype gamma, void *user_data, N_Vector tmp1,
                           N_Vector tmp2, N_Vector tmp3);
 
    /// Solve the linear system $ A x = b $.
    static int LinSysSolve(SUNLinearSolver LS, SUNMatrix A, N_Vector x,
-                          N_Vector b, realtype tol);
+                          N_Vector b, sunrealtype tol);
 
    /// Setup the linear system $ M x = b $.
-   static int MassSysSetup(realtype t, SUNMatrix M, void *user_data,
+   static int MassSysSetup(sunrealtype t, SUNMatrix M, void *user_data,
                            N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
 
    /// Solve the linear system $ M x = b $.
    static int MassSysSolve(SUNLinearSolver LS, SUNMatrix M, N_Vector x,
-                           N_Vector b, realtype tol);
+                           N_Vector b, sunrealtype tol);
 
    /// Compute the matrix-vector product $ v = M x $.
    static int MassMult1(SUNMatrix M, N_Vector x, N_Vector v);
 
    /// Compute the matrix-vector product $v = M_t x $ at time t.
-   static int MassMult2(N_Vector x, N_Vector v, realtype t,
+   static int MassMult2(N_Vector x, N_Vector v, sunrealtype t,
                         void* mtimes_data);
 
 public:
@@ -751,7 +801,7 @@ public:
        @note If this method is called a second time with a different problem
        size, then any non-default user-set options will be lost and will need
        to be set again. */
-   void Init(TimeDependentOperator &f_);
+   void Init(TimeDependentOperator &f_) override;
 
    /// Integrate the ODE with ARKode using the specified step mode.
    /**
@@ -763,7 +813,7 @@ public:
        @note On input, the values of @a t and @a dt are used to compute desired
        output time for the integration, tout = @a t + @a dt.
    */
-   virtual void Step(Vector &x, double &t, double &dt);
+   void Step(Vector &x, real_t &t, real_t &dt) override;
 
    /** @brief Attach the linear system setup and solve methods from the
        TimeDependentOperator i.e., SUNImplicitSetup() and SUNImplicitSolve() to
@@ -850,18 +900,22 @@ protected:
    bool use_oper_grad;                         ///< use the Jv prod function
    mutable SundialsNVector *y_scale, *f_scale; ///< scaling vectors
    const Operator *jacobian;                   ///< stores oper->GetGradient()
-   int maa;           ///< number of acceleration vectors
-   bool jfnk = false; ///< enable JFNK
-   Vector wrk;        ///< Work vector needed for the JFNK PC
-   int maxli = 5;     ///< Maximum linear iterations
-   int maxlrs = 0;    ///< Maximum linear solver restarts
+   int aa_n = 0;            ///< number of acceleration vectors
+   int aa_delay;            ///< Anderson Acceleration delay
+   double aa_damping;       ///< Anderson Acceleration damping
+   int aa_orth;             ///< Anderson Acceleration orthogonalization routine
+   double fp_damping = 1.0; ///< Fixed Point or Picard damping parameter
+   bool jfnk = false;       ///< enable JFNK
+   Vector wrk;              ///< Work vector needed for the JFNK PC
+   int maxli = 5;           ///< Maximum linear iterations
+   int maxlrs = 0;          ///< Maximum linear solver restarts
 
    /// Wrapper to compute the nonlinear residual $ F(u) = 0 $.
    static int Mult(const N_Vector u, N_Vector fu, void *user_data);
 
    /// Wrapper to compute the Jacobian-vector product $ J(u) v = Jv $.
    static int GradientMult(N_Vector v, N_Vector Jv, N_Vector u,
-                           booleantype *new_u, void *user_data);
+                           sunbooleantype *new_u, void *user_data);
 
    /// Setup the linear system $ J u = b $.
    static int LinSysSetup(N_Vector u, N_Vector fu, SUNMatrix J,
@@ -869,7 +923,7 @@ protected:
 
    /// Solve the linear system $ J u = b $.
    static int LinSysSolve(SUNLinearSolver LS, SUNMatrix J, N_Vector u,
-                          N_Vector b, realtype tol);
+                          N_Vector b, sunrealtype tol);
 
    /// Setup the preconditioner.
    static int PrecSetup(N_Vector uu,
@@ -916,7 +970,7 @@ public:
    /** @note If this method is called a second time with a different problem
        size, then non-default KINSOL-specific options will be lost and will need
        to be set again. */
-   virtual void SetOperator(const Operator &op);
+   void SetOperator(const Operator &op) override;
 
    /// Set the linear solver for inverting the Jacobian.
    /** @note This function assumes that Operator::GetGradient(const Vector &)
@@ -924,10 +978,10 @@ public:
              SetOperator(const Operator &).
 
              This method must be called after SetOperator(). */
-   virtual void SetSolver(Solver &solver);
+   void SetSolver(Solver &solver) override;
 
    /// Equivalent to SetSolver(solver).
-   virtual void SetPreconditioner(Solver &solver) { SetSolver(solver); }
+   void SetPreconditioner(Solver &solver) override { SetSolver(solver); }
 
    /// Set KINSOL's scaled step tolerance.
    /** The default tolerance is $ U^\frac{2}{3} $ , where
@@ -940,13 +994,22 @@ public:
        @note This method must be called after SetOperator(). */
    void SetMaxSetupCalls(int max_calls);
 
-   /// Set the number of acceleration vectors to use with KIN_FP or KIN_PICARD.
-   /** The default is 0.
-       @ note This method must be called before SetOperator() to set the
-       maximum size of the acceleration space. The value of @a maa can be
-       altered after SetOperator() is called but it can't be higher than initial
-       maximum. */
-   void SetMAA(int maa);
+   /// Enable Anderson Acceleration for KIN_FP or KIN_PICARD.
+   /** @note Has to be called once before SetOperator() in order to set up the
+       maximum subspace size. Subsequent calls need @a n less or equal to the
+       initial subspace size.
+       @param[in] n Anderson Acceleration subspace size
+       @param[in] orth Anderson Acceleration orthogonalization routine
+       @param[in] delay Anderson Acceleration delay
+       @param[in] damping Anderson Acceleration damping parameter valid from 0 <
+       d <= 1.0. Default is 1.0 (no damping) */
+   void EnableAndersonAcc(int n, int orth = KIN_ORTH_MGS, int delay = 0,
+                          double damping = 1.0);
+
+   /// Specifies the value of the damping parameter in the fixed point or Picard
+   /// iteration.
+   /** param[in] damping fixed point iteration or Picard damping parameter */
+   void SetDamping(double damping);
 
    /// Set the Jacobian Free Newton Krylov flag. The default is false.
    /** This flag indicates to use JFNK as the linear solver for KINSOL. This
@@ -967,10 +1030,10 @@ public:
    void SetLSMaxRestarts(int m) { maxlrs = m; }
 
    /// Set the print level for the KINSetPrintLevel function.
-   virtual void SetPrintLevel(int print_lvl) { print_level = print_lvl; }
+   void SetPrintLevel(int print_lvl) override { print_level = print_lvl; }
 
    /// This method is not supported and will throw an error.
-   virtual void SetPrintLevel(PrintLevel);
+   void SetPrintLevel(PrintLevel) override;
 
    /// Solve the nonlinear system $ F(x) = 0 $.
    /** This method computes the x_scale and fx_scale vectors and calls the
@@ -981,7 +1044,7 @@ public:
        @param[in,out] x  On input, initial guess, if @a #iterative_mode = true,
                          otherwise the initial guess is zero; on output, the
                          solution */
-   virtual void Mult(const Vector &b, Vector &x) const;
+   void Mult(const Vector &b, Vector &x) const override;
 
    /// Solve the nonlinear system $ F(x) = 0 $.
    /** Calls KINSol() to solve the nonlinear system. Before calling KINSol(),
