@@ -11,19 +11,19 @@
 
 // Implementation of data type vector
 
-#include "kernels.hpp"
-#include "vector.hpp"
 #include "../general/forall.hpp"
+#include "../general/reducers.hpp"
+#include "../general/hash.hpp"
+#include "../general/scan.hpp"
+#include "vector.hpp"
 
 #ifdef MFEM_USE_OPENMP
 #include <omp.h>
 #endif
 
 #include <iostream>
-#include <iomanip>
 #include <cmath>
 #include <ctime>
-#include <limits>
 
 namespace mfem
 {
@@ -119,8 +119,9 @@ Vector::Vector(const Vector &v)
 }
 
 Vector::Vector(Vector &&v)
+   : data(std::move(v.data)), size(v.size)
 {
-   *this = std::move(v);
+   v.size = 0;
 }
 
 void Vector::Load(std::istream **in, int np, int *dim)
@@ -207,21 +208,24 @@ Vector &Vector::operator=(const Vector &v)
    UseDevice(v.UseDevice());
 #else
    SetSize(v.Size());
-   bool vuse = v.UseDevice();
+   const bool vuse = v.UseDevice();
    const bool use_dev = UseDevice() || vuse;
-   v.UseDevice(use_dev);
+   if (use_dev != vuse) { v.UseDevice(use_dev); }
    // keep 'data' where it is, unless 'use_dev' is true
    if (use_dev) { Write(); }
    data.CopyFrom(v.data, v.Size());
-   v.UseDevice(vuse);
+   if (use_dev != vuse) { v.UseDevice(vuse); }
 #endif
    return *this;
 }
 
 Vector &Vector::operator=(Vector &&v)
 {
-   v.Swap(*this);
-   if (this != &v) { v.Destroy(); }
+   if (this != &v)
+   {
+      v.Swap(*this);
+      v.Destroy();
+   }
    return *this;
 }
 
@@ -249,8 +253,8 @@ Vector &Vector::operator*=(const Vector &v)
 
    const bool use_dev = UseDevice() || v.UseDevice();
    const int N = size;
+   const auto x = v.Read(use_dev);
    auto y = ReadWrite(use_dev);
-   auto x = v.Read(use_dev);
    mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i) { y[i] *= x[i]; });
    return *this;
 }
@@ -271,8 +275,8 @@ Vector &Vector::operator/=(const Vector &v)
 
    const bool use_dev = UseDevice() || v.UseDevice();
    const int N = size;
+   const auto x = v.Read(use_dev);
    auto y = ReadWrite(use_dev);
-   auto x = v.Read(use_dev);
    mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i) { y[i] /= x[i]; });
    return *this;
 }
@@ -292,8 +296,8 @@ Vector &Vector::operator-=(const Vector &v)
 
    const bool use_dev = UseDevice() || v.UseDevice();
    const int N = size;
+   const auto x = v.Read(use_dev);
    auto y = ReadWrite(use_dev);
-   auto x = v.Read(use_dev);
    mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i) { y[i] -= x[i]; });
    return *this;
 }
@@ -313,8 +317,8 @@ Vector &Vector::operator+=(const Vector &v)
 
    const bool use_dev = UseDevice() || v.UseDevice();
    const int N = size;
+   const auto x = v.Read(use_dev);
    auto y = ReadWrite(use_dev);
-   auto x = v.Read(use_dev);
    mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i) { y[i] += x[i]; });
    return *this;
 }
@@ -327,8 +331,8 @@ Vector &Vector::Add(const real_t a, const Vector &Va)
    {
       const int N = size;
       const bool use_dev = UseDevice() || Va.UseDevice();
+      const auto x = Va.Read(use_dev);
       auto y = ReadWrite(use_dev);
-      auto x = Va.Read(use_dev);
       mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i) { y[i] += a * x[i]; });
    }
    return *this;
@@ -340,7 +344,7 @@ Vector &Vector::Set(const real_t a, const Vector &Va)
 
    const bool use_dev = UseDevice() || Va.UseDevice();
    const int N = size;
-   auto x = Va.Read(use_dev);
+   const auto x = Va.Read(use_dev);
    auto y = Write(use_dev);
    mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i) { y[i] = a * x[i]; });
    return *this;
@@ -350,26 +354,23 @@ void Vector::SetVector(const Vector &v, int offset)
 {
    MFEM_ASSERT(v.Size() + offset <= size, "invalid sub-vector");
 
+   const bool use_dev = UseDevice() || v.UseDevice();
    const int vs = v.Size();
-   const real_t *vp = v.data;
-   real_t *p = data + offset;
-   for (int i = 0; i < vs; i++)
-   {
-      p[i] = vp[i];
-   }
+   const auto vp = v.Read(use_dev);
+   // Use read+write access for *this - we only modify some of its entries
+   auto p = ReadWrite(use_dev) + offset;
+   mfem::forall_switch(use_dev, vs, [=] MFEM_HOST_DEVICE (int i) { p[i] = vp[i]; });
 }
 
 void Vector::AddSubVector(const Vector &v, int offset)
 {
    MFEM_ASSERT(v.Size() + offset <= size, "invalid sub-vector");
 
+   const bool use_dev = UseDevice() || v.UseDevice();
    const int vs = v.Size();
-   const real_t *vp = v.data;
-   real_t *p = data + offset;
-   for (int i = 0; i < vs; i++)
-   {
-      p[i] += vp[i];
-   }
+   const auto vp = v.Read(use_dev);
+   auto p = ReadWrite(use_dev) + offset;
+   mfem::forall_switch(use_dev, vs, [=] MFEM_HOST_DEVICE (int i) { p[i] += vp[i]; });
 }
 
 void Vector::Neg()
@@ -388,6 +389,28 @@ void Vector::Reciprocal()
    mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i) { y[i] = 1.0/y[i]; });
 }
 
+void Vector::Abs()
+{
+   const bool use_dev = UseDevice();
+   const int N = size;
+   auto y = ReadWrite(use_dev);
+   mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i)
+   {
+      y[i] = std::abs(y[i]);
+   });
+}
+
+void Vector::Pow(const real_t p)
+{
+   const bool use_dev = UseDevice();
+   const int N = size;
+   auto y = ReadWrite(use_dev);
+   mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i)
+   {
+      y[i] = std::pow(y[i], p);
+   });
+}
+
 void add(const Vector &v1, const Vector &v2, Vector &v)
 {
    MFEM_ASSERT(v.size == v1.size && v.size == v2.size,
@@ -397,8 +420,8 @@ void add(const Vector &v1, const Vector &v2, Vector &v)
    const bool use_dev = v1.UseDevice() || v2.UseDevice() || v.UseDevice();
    const int N = v.size;
    // Note: get read access first, in case v is the same as v1/v2.
-   auto x1 = v1.Read(use_dev);
-   auto x2 = v2.Read(use_dev);
+   const auto x1 = v1.Read(use_dev);
+   const auto x2 = v2.Read(use_dev);
    auto y = v.Write(use_dev);
    mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i) { y[i] = x1[i] + x2[i]; });
 #else
@@ -429,8 +452,8 @@ void add(const Vector &v1, real_t alpha, const Vector &v2, Vector &v)
       const bool use_dev = v1.UseDevice() || v2.UseDevice() || v.UseDevice();
       const int N = v.size;
       // Note: get read access first, in case v is the same as v1/v2.
-      auto d_x = v1.Read(use_dev);
-      auto d_y = v2.Read(use_dev);
+      const auto d_x = v1.Read(use_dev);
+      const auto d_y = v2.Read(use_dev);
       auto d_z = v.Write(use_dev);
       mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i)
       {
@@ -468,8 +491,8 @@ void add(const real_t a, const Vector &x, const Vector &y, Vector &z)
       const bool use_dev = x.UseDevice() || y.UseDevice() || z.UseDevice();
       const int N = x.size;
       // Note: get read access first, in case z is the same as x/y.
-      auto xd = x.Read(use_dev);
-      auto yd = y.Read(use_dev);
+      const auto xd = x.Read(use_dev);
+      const auto yd = y.Read(use_dev);
       auto zd = z.Write(use_dev);
       mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i)
       {
@@ -523,8 +546,8 @@ void add(const real_t a, const Vector &x,
       const bool use_dev = x.UseDevice() || y.UseDevice() || z.UseDevice();
       const int N = x.size;
       // Note: get read access first, in case z is the same as x/y.
-      auto xd = x.Read(use_dev);
-      auto yd = y.Read(use_dev);
+      const auto xd = x.Read(use_dev);
+      const auto yd = y.Read(use_dev);
       auto zd = z.Write(use_dev);
       mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i)
       {
@@ -553,8 +576,8 @@ void subtract(const Vector &x, const Vector &y, Vector &z)
    const bool use_dev = x.UseDevice() || y.UseDevice() || z.UseDevice();
    const int N = x.size;
    // Note: get read access first, in case z is the same as x/y.
-   auto xd = x.Read(use_dev);
-   auto yd = y.Read(use_dev);
+   const auto xd = x.Read(use_dev);
+   const auto yd = y.Read(use_dev);
    auto zd = z.Write(use_dev);
    mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i)
    {
@@ -592,8 +615,8 @@ void subtract(const real_t a, const Vector &x, const Vector &y, Vector &z)
       const bool use_dev = x.UseDevice() || y.UseDevice() || z.UseDevice();
       const int N = x.size;
       // Note: get read access first, in case z is the same as x/y.
-      auto xd = x.Read(use_dev);
-      auto yd = y.Read(use_dev);
+      const auto xd = x.Read(use_dev);
+      const auto yd = y.Read(use_dev);
       auto zd = z.Write(use_dev);
       mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i)
       {
@@ -634,8 +657,8 @@ void Vector::median(const Vector &lo, const Vector &hi)
    const bool use_dev = UseDevice() || lo.UseDevice() || hi.UseDevice();
    const int N = size;
    // Note: get read access first, in case *this is the same as lo/hi.
-   auto l = lo.Read(use_dev);
-   auto h = hi.Read(use_dev);
+   const auto l = lo.Read(use_dev);
+   const auto h = hi.Read(use_dev);
    auto m = Write(use_dev);
    mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i)
    {
@@ -655,9 +678,9 @@ void Vector::GetSubVector(const Array<int> &dofs, Vector &elemvect) const
    const int n = dofs.Size();
    elemvect.SetSize(n);
    const bool use_dev = dofs.UseDevice() || elemvect.UseDevice();
+   const auto d_X = Read(use_dev);
+   const auto d_dofs = dofs.Read(use_dev);
    auto d_y = elemvect.Write(use_dev);
-   auto d_X = Read(use_dev);
-   auto d_dofs = dofs.Read(use_dev);
    mfem::forall_switch(use_dev, n, [=] MFEM_HOST_DEVICE (int i)
    {
       const int dof_i = d_dofs[i];
@@ -667,7 +690,7 @@ void Vector::GetSubVector(const Array<int> &dofs, Vector &elemvect) const
 
 void Vector::GetSubVector(const Array<int> &dofs, real_t *elem_data) const
 {
-   data.Read(MemoryClass::HOST, size);
+   HostRead();
    const int n = dofs.Size();
    for (int i = 0; i < n; i++)
    {
@@ -678,11 +701,11 @@ void Vector::GetSubVector(const Array<int> &dofs, real_t *elem_data) const
 
 void Vector::SetSubVector(const Array<int> &dofs, const real_t value)
 {
-   const bool use_dev = dofs.UseDevice();
+   const bool use_dev = UseDevice() || dofs.UseDevice();
    const int n = dofs.Size();
    // Use read+write access for *this - we only modify some of its entries
    auto d_X = ReadWrite(use_dev);
-   auto d_dofs = dofs.Read(use_dev);
+   const auto d_dofs = dofs.Read(use_dev);
    mfem::forall_switch(use_dev, n, [=] MFEM_HOST_DEVICE (int i)
    {
       const int j = d_dofs[i];
@@ -697,6 +720,23 @@ void Vector::SetSubVector(const Array<int> &dofs, const real_t value)
    });
 }
 
+void Vector::SetSubVectorHost(const Array<int> &dofs, const real_t value)
+{
+   HostReadWrite();
+   for (int i = 0; i < dofs.Size(); ++i)
+   {
+      const int j = dofs[i];
+      if (j >= 0)
+      {
+         (*this)[j] = value;
+      }
+      else
+      {
+         (*this)[-1-j] = -value;
+      }
+   }
+}
+
 void Vector::SetSubVector(const Array<int> &dofs, const Vector &elemvect)
 {
    MFEM_ASSERT(dofs.Size() <= elemvect.Size(),
@@ -707,8 +747,8 @@ void Vector::SetSubVector(const Array<int> &dofs, const Vector &elemvect)
    const int n = dofs.Size();
    // Use read+write access for X - we only modify some of its entries
    auto d_X = ReadWrite(use_dev);
-   auto d_y = elemvect.Read(use_dev);
-   auto d_dofs = dofs.Read(use_dev);
+   const auto d_y = elemvect.Read(use_dev);
+   const auto d_dofs = dofs.Read(use_dev);
    mfem::forall_switch(use_dev, n, [=] MFEM_HOST_DEVICE (int i)
    {
       const int dof_i = d_dofs[i];
@@ -726,7 +766,7 @@ void Vector::SetSubVector(const Array<int> &dofs, const Vector &elemvect)
 void Vector::SetSubVector(const Array<int> &dofs, real_t *elem_data)
 {
    // Use read+write access because we overwrite only part of the data.
-   data.ReadWrite(MemoryClass::HOST, size);
+   HostReadWrite();
    const int n = dofs.Size();
    for (int i = 0; i < n; i++)
    {
@@ -750,9 +790,9 @@ void Vector::AddElementVector(const Array<int> &dofs, const Vector &elemvect)
 
    const bool use_dev = dofs.UseDevice() || elemvect.UseDevice();
    const int n = dofs.Size();
-   auto d_y = elemvect.Read(use_dev);
+   const auto d_y = elemvect.Read(use_dev);
+   const auto d_dofs = dofs.Read(use_dev);
    auto d_X = ReadWrite(use_dev);
-   auto d_dofs = dofs.Read(use_dev);
    mfem::forall_switch(use_dev, n, [=] MFEM_HOST_DEVICE (int i)
    {
       const int j = d_dofs[i];
@@ -769,7 +809,7 @@ void Vector::AddElementVector(const Array<int> &dofs, const Vector &elemvect)
 
 void Vector::AddElementVector(const Array<int> &dofs, real_t *elem_data)
 {
-   data.ReadWrite(MemoryClass::HOST, size);
+   HostReadWrite();
    const int n = dofs.Size();
    for (int i = 0; i < n; i++)
    {
@@ -794,9 +834,9 @@ void Vector::AddElementVector(const Array<int> &dofs, const real_t a,
 
    const bool use_dev = dofs.UseDevice() || elemvect.UseDevice();
    const int n = dofs.Size();
+   const auto d_x = elemvect.Read(use_dev);
+   const auto d_dofs = dofs.Read(use_dev);
    auto d_y = ReadWrite(use_dev);
-   auto d_x = elemvect.Read(use_dev);
-   auto d_dofs = dofs.Read(use_dev);
    mfem::forall_switch(use_dev, n, [=] MFEM_HOST_DEVICE (int i)
    {
       const int j = d_dofs[i];
@@ -821,7 +861,7 @@ void Vector::SetSubVectorComplement(const Array<int> &dofs, const real_t val)
                     Device::GetHostMemoryType());
    auto d_data = ReadWrite(use_dev);
    auto d_dofs_vals = dofs_vals.Write(use_dev);
-   auto d_dofs = dofs.Read(use_dev);
+   const auto d_dofs = dofs.Read(use_dev);
    mfem::forall_switch(use_dev, n, [=] MFEM_HOST_DEVICE (int i) { d_dofs_vals[i] = d_data[d_dofs[i]]; });
    mfem::forall_switch(use_dev, N, [=] MFEM_HOST_DEVICE (int i) { d_data[i] = val; });
    mfem::forall_switch(use_dev, n, [=] MFEM_HOST_DEVICE (int i) { d_data[d_dofs[i]] = d_dofs_vals[i]; });
@@ -830,7 +870,7 @@ void Vector::SetSubVectorComplement(const Array<int> &dofs, const real_t val)
 void Vector::Print(std::ostream &os, int width) const
 {
    if (!size) { return; }
-   data.Read(MemoryClass::HOST, size);
+   HostRead();
    for (int i = 0; 1; )
    {
       os << ZeroSubnormal(data[i]);
@@ -856,7 +896,7 @@ void Vector::Print(adios2stream &os,
                    const std::string& variable_name) const
 {
    if (!size) { return; }
-   data.Read(MemoryClass::HOST, size);
+   HostRead();
    os.engine.Put(variable_name, &data[0] );
 }
 #endif
@@ -914,10 +954,7 @@ void Vector::PrintHash(std::ostream &os) const
 
 void Vector::Randomize(int seed)
 {
-   if (seed == 0)
-   {
-      seed = (int)time(0);
-   }
+   if (seed == 0) { seed = (int)time(0); }
 
    srand((unsigned)seed);
 
@@ -933,20 +970,15 @@ real_t Vector::Norml2() const
    // Scale entries of Vector on the fly, using algorithms from
    // std::hypot() and LAPACK's drm2. This scaling ensures that the
    // argument of each call to std::pow is <= 1 to avoid overflow.
-   if (size == 0)
-   {
-      return 0.0;
-   }
+   if (size == 0) { return 0.0; }
 
-   auto m_data = Read(UseDevice());
+   const auto m_data = Read(UseDevice());
    using value_type = DevicePair<real_t, real_t>;
    value_type res;
    res.first = 0;
    res.second = 0;
    // first compute sum (|m_data|/scale)^2
-   reduce(
-      size, res,
-      [=] MFEM_HOST_DEVICE(int i, value_type &r)
+   reduce(size, res, [=] MFEM_HOST_DEVICE(int i, value_type &r)
    {
       real_t n = fabs(m_data[i]);
       if (n > 0)
@@ -973,11 +1005,12 @@ real_t Vector::Normlinf() const
 {
    if (size == 0) { return 0; }
 
-   auto m_data = Read(UseDevice());
    real_t res = 0;
-   reduce(
-      size, res,
-   [=] MFEM_HOST_DEVICE(int i, real_t &r) { r = fmax(r, fabs(m_data[i])); },
+   const auto m_data = Read(UseDevice());
+   reduce(size, res, [=] MFEM_HOST_DEVICE(int i, real_t &r)
+   {
+      r = fmax(r, fabs(m_data[i]));
+   },
    MaxReducer<real_t> {}, UseDevice(), vector_workspace());
    return res;
 }
@@ -986,11 +1019,12 @@ real_t Vector::Norml1() const
 {
    if (size == 0) { return 0.0; }
 
-   auto m_data = Read(UseDevice());
    real_t res = 0;
-   reduce(
-      size, res,
-   [=] MFEM_HOST_DEVICE(int i, real_t &r) { r += fabs(m_data[i]); },
+   const auto m_data = Read(UseDevice());
+   reduce(size, res, [=] MFEM_HOST_DEVICE(int i, real_t &r)
+   {
+      r += fabs(m_data[i]);
+   },
    SumReducer<real_t> {}, UseDevice(), vector_workspace());
    return res;
 }
@@ -999,33 +1033,24 @@ real_t Vector::Normlp(real_t p) const
 {
    MFEM_ASSERT(p > 0.0, "Vector::Normlp");
 
-   if (p == 1.0)
-   {
-      return Norml1();
-   }
-   if (p == 2.0)
-   {
-      return Norml2();
-   }
+   if (p == 1.0) { return Norml1(); }
+
+   if (p == 2.0) { return Norml2(); }
+
    if (p < infinity())
    {
       // Scale entries of Vector on the fly, using algorithms from
       // std::hypot() and LAPACK's drm2. This scaling ensures that the
       // argument of each call to std::pow is <= 1 to avoid overflow.
-      if (size == 0)
-      {
-         return 0.0;
-      }
+      if (size == 0) { return 0.0; }
 
-      auto m_data = Read(UseDevice());
       using value_type = DevicePair<real_t, real_t>;
       value_type res;
       res.first = 0;
       res.second = 0;
+      const auto m_data = Read(UseDevice());
       // first compute sum (|m_data|/scale)^p
-      reduce(
-         size, res,
-         [=] MFEM_HOST_DEVICE(int i, value_type &r)
+      reduce(size, res, [=] MFEM_HOST_DEVICE(int i, value_type &r)
       {
          real_t n = fabs(m_data[i]);
          if (n > 0)
@@ -1054,163 +1079,211 @@ real_t Vector::Normlp(real_t p) const
 real_t Vector::operator*(const Vector &v) const
 {
    MFEM_ASSERT(size == v.size, "incompatible Vectors!");
+
    if (size == 0) { return 0.0; }
 
    const bool use_dev = UseDevice() || v.UseDevice();
+   const auto m_data = Read(use_dev), v_data = v.Read(use_dev);
 
-   auto m_data = Read(use_dev);
-   auto v_data = v.Read(use_dev);
-
-   if (use_dev)
-   {
-      // special path for OCCA and OpenMP
+   // If OCCA is enabled, it handles all selected backends
 #ifdef MFEM_USE_OCCA
-      if (DeviceCanUseOcca())
-      {
-         return occa::linalg::dot<real_t, real_t, real_t>(
-                   OccaMemoryRead(data, size), OccaMemoryRead(v.data, size));
-      }
+   if (use_dev && DeviceCanUseOcca())
+   {
+      return occa::linalg::dot<real_t, real_t, real_t>(
+                OccaMemoryRead(data, size), OccaMemoryRead(v.data, size));
+   }
 #endif
 
-#ifdef MFEM_USE_OPENMP
-      if (Device::Allows(Backend::OMP_MASK))
+   const auto compute_dot = [&]()
+   {
+      real_t res = 0;
+      reduce(size, res, [=] MFEM_HOST_DEVICE (int i, real_t &r)
       {
+         r += m_data[i] * v_data[i];
+      },
+      SumReducer<real_t> {}, use_dev, vector_workspace());
+      return res;
+   };
+
+   // Device backends have top priority
+   if (Device::Allows(Backend::DEVICE_MASK)) { return compute_dot(); }
+
+   // Special path for OpenMP
+#ifdef MFEM_USE_OPENMP
+   if (use_dev && Device::Allows(Backend::OMP_MASK))
+   {
+      // By default, use a deterministic way of computing the dot product
 #define MFEM_USE_OPENMP_DETERMINISTIC_DOT
 #ifdef MFEM_USE_OPENMP_DETERMINISTIC_DOT
-         // By default, use a deterministic way of computing the dot product
-         static Vector th_dot;
-         #pragma omp parallel
+      static Vector th_dot;
+      #pragma omp parallel
+      {
+         const int nt = omp_get_num_threads();
+         #pragma omp master
+         th_dot.SetSize(nt);
+         const int tid = omp_get_thread_num();
+         const int stride = (size + nt - 1) / nt;
+         const int start = tid * stride;
+         const int stop = std::min(start + stride, size);
+         real_t my_dot = 0.0;
+         for (int i = start; i < stop; i++)
          {
-            const int nt = omp_get_num_threads();
-            #pragma omp master
-            th_dot.SetSize(nt);
-            const int tid = omp_get_thread_num();
-            const int stride = (size + nt - 1) / nt;
-            const int start = tid * stride;
-            const int stop = std::min(start + stride, size);
-            real_t my_dot = 0.0;
-            for (int i = start; i < stop; i++)
-            {
-               my_dot += m_data[i] * v_data[i];
-            }
-            #pragma omp barrier
-            th_dot(tid) = my_dot;
+            my_dot += m_data[i] * v_data[i];
          }
-         return th_dot.Sum();
-#else
-         // The standard way of computing the dot product is non-deterministic
-         real_t prod = 0.0;
-         #pragma omp parallel for reduction(+ : prod)
-         for (int i = 0; i < size; i++)
-         {
-            prod += m_data[i] * v_data[i];
-         }
-         return prod;
-#endif // MFEM_USE_OPENMP_DETERMINISTIC_DOT
+         #pragma omp barrier
+         th_dot(tid) = my_dot;
       }
-#endif // MFEM_USE_OPENMP
+      return th_dot.Sum();
+#else
+      // The standard way of computing the dot product is non-deterministic
+      real_t prod = 0.0;
+      #pragma omp parallel for reduction(+ : prod)
+      for (int i = 0; i < size; i++)
+      {
+         prod += m_data[i] * v_data[i];
+      }
+      return prod;
+#endif // MFEM_USE_OPENMP_DETERMINISTIC_DOT
    }
+#endif // MFEM_USE_OPENMP
 
-   // normal path for everything else (cuda, hip, debug, cpu)
-   real_t res = 0;
-   reduce(
-      size, res,
-   [=] MFEM_HOST_DEVICE(int i, real_t &r) { r += m_data[i] * v_data[i]; },
-   SumReducer<real_t> {}, use_dev, vector_workspace());
-   return res;
+   // All other CPU backends
+   return compute_dot();
 }
 
 real_t Vector::Min() const
 {
    if (size == 0) { return infinity(); }
 
-   const bool use_dev = UseDevice();
-   auto m_data = Read(use_dev);
-
-   if (use_dev)
-   {
-      // special case for OCCA and OpenMP
+   const auto use_dev = UseDevice();
+   const auto m_data = Read(use_dev);
 
 #ifdef MFEM_USE_OCCA
-      if (DeviceCanUseOcca())
-      {
-         return occa::linalg::min<real_t,real_t>(OccaMemoryRead(data, size));
-      }
-#endif
-
-#ifdef MFEM_USE_OPENMP
-      if (Device::Allows(Backend::OMP_MASK))
-      {
-         real_t minimum = m_data[0];
-         #pragma omp parallel for reduction(min:minimum)
-         for (int i = 0; i < size; i++)
-         {
-            minimum = std::min(minimum, m_data[i]);
-         }
-         return minimum;
-      }
-#endif
+   if (use_dev && DeviceCanUseOcca())
+   {
+      return occa::linalg::min<real_t,real_t>(OccaMemoryRead(data, size));
    }
+#endif
 
-   // normal path for everything else (cuda, hip, debug, cpu)
-   real_t res = infinity();
-   reduce(
-      size, res,
-   [=] MFEM_HOST_DEVICE(int i, real_t &r) { r = fmin(r, m_data[i]); },
-   MinReducer<real_t> {}, use_dev, vector_workspace());
-   return res;
+   const auto compute_min = [&]()
+   {
+      real_t res = infinity();
+      reduce(size, res, [=] MFEM_HOST_DEVICE(int i, real_t &r)
+      {
+         r = fmin(r, m_data[i]);
+      },
+      MinReducer<real_t> {}, use_dev, vector_workspace());
+      return res;
+   };
+
+   // Device backends have top priority
+   if (Device::Allows(Backend::DEVICE_MASK)) { return compute_min(); }
+
+   // Special path for OpenMP
+#ifdef MFEM_USE_OPENMP
+   if (use_dev && Device::Allows(Backend::OMP_MASK))
+   {
+      real_t minimum = m_data[0];
+      #pragma omp parallel for reduction(min:minimum)
+      for (int i = 0; i < size; i++)
+      {
+         minimum = std::min(minimum, m_data[i]);
+      }
+      return minimum;
+   }
+#endif
+
+   // All other CPU backends
+   return compute_min();
 }
 
 real_t Vector::Max() const
 {
    if (size == 0) { return -infinity(); }
 
-   const bool use_dev = UseDevice();
-   auto m_data = Read(use_dev);
+   const auto use_dev = UseDevice();
+   const auto m_data = Read(use_dev);
 
-   if (use_dev)
-   {
-      // special cases where OCCA or OenMP are used
 #ifdef MFEM_USE_OCCA
-      if (DeviceCanUseOcca())
-      {
-         return occa::linalg::max<real_t, real_t>(OccaMemoryRead(data, size));
-      }
-#endif
-
-#ifdef MFEM_USE_OPENMP
-      if (Device::Allows(Backend::OMP_MASK))
-      {
-         real_t maximum = m_data[0];
-         #pragma omp parallel for reduction(max : maximum)
-         for (int i = 0; i < size; i++)
-         {
-            maximum = fmax(maximum, m_data[i]);
-         }
-         return maximum;
-      }
-#endif
+   if (use_dev && DeviceCanUseOcca())
+   {
+      return occa::linalg::max<real_t, real_t>(OccaMemoryRead(data, size));
    }
+#endif
 
-   // normal path for everything else (cuda, hip, debug, cpu)
-   real_t res = -infinity();
-   reduce(
-      size, res,
-   [=] MFEM_HOST_DEVICE(int i, real_t &r) { r = fmax(r, m_data[i]); },
-   MaxReducer<real_t> {}, use_dev, vector_workspace());
-   return res;
+   const auto compute_max = [&]()
+   {
+      real_t res = -infinity();
+      reduce(size, res, [=] MFEM_HOST_DEVICE(int i, real_t &r)
+      {
+         r = fmax(r, m_data[i]);
+      },
+      MaxReducer<real_t> {}, use_dev, vector_workspace());
+      return res;
+   };
+
+   // Device backends have top priority
+   if (Device::Allows(Backend::DEVICE_MASK)) { return compute_max(); }
+
+   // Special path for OpenMP
+#ifdef MFEM_USE_OPENMP
+   if (use_dev && Device::Allows(Backend::OMP_MASK))
+   {
+      real_t maximum = m_data[0];
+      #pragma omp parallel for reduction(max : maximum)
+      for (int i = 0; i < size; i++)
+      {
+         maximum = fmax(maximum, m_data[i]);
+      }
+      return maximum;
+   }
+#endif
+
+   // All other CPU backends
+   return compute_max();
 }
 
 real_t Vector::Sum() const
 {
    if (size == 0) { return 0.0; }
 
-   auto m_data = Read(UseDevice());
    real_t res = 0;
-   reduce(
-   size, res, [=] MFEM_HOST_DEVICE(int i, real_t &r) { r += m_data[i]; },
+   const auto m_data = Read(UseDevice());
+   reduce(size, res, [=] MFEM_HOST_DEVICE(int i, real_t &r)
+   {
+      r += m_data[i];
+   },
    SumReducer<real_t> {}, UseDevice(), vector_workspace());
    return res;
 }
 
+void Vector::DeleteAt(const Array<int> &indices)
+{
+   if (indices.Size())
+   {
+      const bool use_dev = UseDevice();
+
+      // extra entry for number of selected out
+      Array<int> workspace(size + 1);
+      const auto d_flag = workspace.Write(use_dev);
+      mfem::forall_switch(use_dev, size,
+      [=] MFEM_HOST_DEVICE(int i) { d_flag[i] = true; });
+      const auto d_indices = indices.Read(use_dev);
+      mfem::forall_switch(use_dev, indices.Size(), [=] MFEM_HOST_DEVICE(int i)
+      {
+         // fine as long as indices are unique; to support non-unique indices
+         // assignment to d_flag must be atomic
+         d_flag[d_indices[i]] = false;
+      });
+
+      Vector copy(*this);
+      auto d_in = copy.Read(use_dev);
+      auto d_out = Write(use_dev);
+      CopyFlagged(use_dev, d_in, d_flag, d_out, d_flag + size, size);
+
+      // assumes indices are unique
+      size -= indices.Size();
+   }
 }
+
+} // namespace mfem
