@@ -129,7 +129,8 @@ public:
    /// Create a vector using a braced initializer list
    template <typename CT, typename std::enable_if<
                 std::is_convertible<CT,real_t>::value,bool>::type = true>
-   explicit Vector(std::initializer_list<CT> values) : Vector(values.size())
+   explicit Vector(std::initializer_list<CT> values) :
+      Vector(static_cast<int> (values.size()))
    { std::copy(values.begin(), values.end(), begin()); }
 
    /// Enable execution of Vector operations using the mfem::Device.
@@ -171,6 +172,13 @@ public:
    /// Resize the vector to size @a s using the MemoryType of @a v.
    void SetSize(int s, const Vector &v) { SetSize(s, v.GetMemory().GetMemoryType()); }
 
+   /// Update \ref Capacity() to @a res (if less than current), keeping existing entries.
+   void Reserve(int res);
+
+   /// Delete entries at @a indices and resize vector accordingly.
+   /// @warning Indices must be unique!
+   void DeleteAt(const Array<int> &indices);
+
    /// Set the Vector data.
    /// @warning This method should be called only when OwnsData() is false.
    void SetData(real_t *d) { data.Wrap(d, data.Capacity(), false); }
@@ -348,8 +356,10 @@ public:
    /// (*this) = a * x
    Vector &Set(const real_t a, const Vector &x);
 
+   /// (*this)[i + offset] = v[i]
    void SetVector(const Vector &v, int offset);
 
+   /// (*this)[i + offset] += v[i]
    void AddSubVector(const Vector &v, int offset);
 
    /// (*this) = -(*this)
@@ -358,7 +368,14 @@ public:
    /// (*this)(i) = 1.0 / (*this)(i)
    void Reciprocal();
 
+   /// (*this)(i) = abs((*this)(i))
+   void Abs();
+
+   /// (*this)(i) = pow((*this)(i), p)
+   void Pow(const real_t p);
+
    /// Swap the contents of two Vectors
+   /** Implemented without using move assignment, avoiding Destroy() calls. */
    inline void Swap(Vector &other);
 
    /// Set v = v1 + v2.
@@ -403,6 +420,15 @@ public:
        the -value. */
    void SetSubVector(const Array<int> &dofs, const real_t value);
 
+   /// Set the entries listed in @a dofs to the given @a value (always on host).
+   /** Negative dof values cause the -dof-1 position in this Vector to receive
+       the -value.
+
+       As opposed to SetSubVector(const Array<int>&, const real_t), this
+       function will execute only on host, even if the vector or the @a dofs
+       array have the device flag set. */
+   void SetSubVectorHost(const Array<int> &dofs, const real_t value);
+
    /** @brief Set the entries listed in @a dofs to the values given in the @a
        elemvect Vector. Negative dof values cause the -dof-1 position in this
        Vector to receive the -val from @a elemvect. */
@@ -604,6 +630,18 @@ inline void Vector::SetSize(int s, MemoryType mt)
    data.UseDevice(use_dev);
 }
 
+inline void Vector::Reserve(int res)
+{
+   if (res > Capacity())
+   {
+      Memory<real_t> p(res, data.GetMemoryType());
+      p.CopyFrom(data, size);
+      p.UseDevice(data.UseDevice());
+      data.Delete();
+      data = p;
+   }
+}
+
 inline void Vector::NewMemoryAndSize(const Memory<real_t> &mem, int s,
                                      bool own_mem)
 {
@@ -635,9 +673,8 @@ inline void Vector::MakeRef(Vector &base, int offset)
 inline void Vector::Destroy()
 {
    const bool use_dev = data.UseDevice();
-   data.Delete();
+   data.Delete();  // calls data.Reset(h_mt) as well
    size = 0;
-   data.Reset();
    data.UseDevice(use_dev);
 }
 
@@ -663,8 +700,9 @@ inline void Vector::Swap(Vector &other)
    mfem::Swap(size, other.size);
 }
 
-/// Specialization of the template function Swap<> for class Vector
-template<> inline void Swap<Vector>(Vector &a, Vector &b)
+/** @brief Swap of Vector objects for use with standard library algorithms.
+    Also, used by mfem::Swap(). */
+inline void swap(Vector &a, Vector &b)
 {
    a.Swap(b);
 }
