@@ -22,14 +22,12 @@
 #include "../general/communication.hpp"
 #endif
 #include <iostream>
-#include <set>
 
 namespace mfem
 {
 
 class GridFunction;
 
-
 /** @brief A vector of knots in one dimension, with B-spline basis functions of
     a prescribed order.
 
@@ -55,7 +53,7 @@ protected:
 
 public:
    /// Create an empty KnotVector.
-   KnotVector() { }
+   KnotVector() = default;
 
    /** @brief Create a KnotVector by reading data from stream @a input. Two
        integers are read, for order and number of control points. */
@@ -74,7 +72,7 @@ public:
        polynomial degree). Periodicity is not supported.
    */
    KnotVector(int order, const Vector& intervals,
-              const Array<int>& continuity );
+              const Array<int>& continuity);
 
    /// Copy constructor.
    KnotVector(const KnotVector &kv) { (*this) = kv; }
@@ -144,8 +142,13 @@ public:
    /** @brief Global curve interpolation through the points @a x (overwritten).
        @a x is an array with the length of the spatial dimension containing
        vectors with spatial coordinates. The control points of the interpolated
-       curve are returned in @a x in the same form. */
-   void FindInterpolant(Array<Vector*> &x);
+       curve are returned in @a x in the same form.
+
+       The inverse of the collocation matrix, used in the interpolation, is
+       stored for repeated calls and used if @a reuse_inverse is true. Reuse is
+       valid only if this KnotVector has not changed since the initial call with
+       @a reuse_inverse false. */
+   void FindInterpolant(Array<Vector*> &x, bool reuse_inverse = false);
 
    /** Set @a diff, comprised of knots in @a kv not contained in this KnotVector.
        @a kv must be of the same order as this KnotVector. The current
@@ -154,10 +157,10 @@ public:
    void Difference(const KnotVector &kv, Vector &diff) const;
 
    /// Uniformly refine by factor @a rf, by inserting knots in each span.
-   void UniformRefinement(Vector &newknots, int rf) const;
+   void UniformRefinement(Vector &new_knots, int rf) const;
 
    /// Refine with refinement factor @a rf.
-   void Refinement(Vector &newknots, int rf) const;
+   void Refinement(Vector &new_knots, int rf) const;
 
    /** Returns the coarsening factor needed for non-nested nonuniform spacing
        functions, to result in a single element from which refinement can be
@@ -197,12 +200,23 @@ public:
    /// Const access function to knot @a i.
    const real_t &operator[](int i) const { return knot(i); }
 
+   /// Coarsen to a single element.
+   KnotVector* FullyCoarsen();
+
    /// Function to define the distribution of knots for any number of knot spans.
    std::shared_ptr<SpacingFunction> spacing;
 
-   /** Flag to indicate whether the KnotVector has been coarsened, which means
-       it is ready for non-nested refinement. */
+   /** @brief Flag to indicate whether the KnotVector has been coarsened, which
+       means it is ready for non-nested refinement. */
    bool coarse;
+
+#ifdef MFEM_USE_LAPACK
+   // Data for reusing banded matrix factorization in FindInterpolant().
+   DenseMatrix fact_AB; /// Banded matrix factorization
+   Array<int> fact_ipiv; /// Row pivot indices
+#else
+   DenseMatrix A_coll_inv; /// Collocation matrix inverse
+#endif
 };
 
 
@@ -243,7 +257,6 @@ protected:
        Returns the number of variables per knot in flattened structure. */
    int SetLoopDirection(int dir);
 
-
    /** @brief Access function for the effectively 1D flattened net, where @a i
        is a knot index, and @a j is an index of a variable per knot. */
    inline       real_t &slice(int i, int j);
@@ -286,7 +299,7 @@ public:
        includes the weight. The array of control point coordinates stores each
        point's coordinates contiguously, and points are ordered in a standard
        ijk grid ordering. */
-   NURBSPatch(Array<const KnotVector *> &kv_,  int dim_,
+   NURBSPatch(Array<const KnotVector *> &kv_, int dim_,
               const real_t* control_points);
 
    /// Constructor for a patch of dimension equal to the size of @a kv.
@@ -307,6 +320,9 @@ public:
    /// Increase the order in direction @a dir by @a t >= 0.
    void DegreeElevate(int dir, int t);
 
+   /// Increase the order in all directions by @a t >= 0.
+   void DegreeElevate(int t);
+
    /**  @brief Insert any new knots from @a knot in direction @a dir. If the
         order of @a knot is higher than the current order in direction
         @a dir, then the order is elevated in that direction to match. */
@@ -336,17 +352,20 @@ public:
    /// Remove all knots in @a knot once, for each direction.
    void KnotRemove(Array<Vector *> &knot, real_t tol = 1.0e-12);
 
-   void DegreeElevate(int t);
-
    /** @brief Refine with optional refinement factor @a rf. Uniform means
        refinement is done everywhere by the same factor, although nonuniform
        spacing functions may be used.
 
        @param[in] rf Optional refinement factor. If scalar, the factor is used
                      for all dimensions. If an array, factors can be specified
-                     for each dimension. */
-   void UniformRefinement(int rf = 2);
-   void UniformRefinement(Array<int> const& rf);
+                     for each dimension.
+       @param[in] multiplicity Optional multiplicity for new knots inserted. */
+   void UniformRefinement(int rf = 2, int multiplicity = 1);
+   void UniformRefinement(const Array<int> &rf, int multiplicity = 1);
+
+   /// Flag @a coarsened indicates whether the patch is a single element.
+   void UniformRefinement(const std::vector<Array<int>> &rf,
+                          bool coarsened = false, int multiplicity = 1);
 
    /** @brief Coarsen with optional coarsening factor @a cf which divides the
        number of elements in each dimension. Nonuniform spacing functions may be
@@ -358,14 +377,20 @@ public:
        @param[in] tol NURBS geometry deviation tolerance, cf. Algorithm A5.8 of
        "The NURBS Book", 2nd ed, Piegl and Tiller. */
    void Coarsen(int cf = 2, real_t tol = 1.0e-12);
-   void Coarsen(Array<int> const& cf, real_t tol = 1.0e-12);
+   void Coarsen(const Array<int> &cf, real_t tol = 1.0e-12);
 
    /// Calls KnotVector::GetCoarseningFactor for each direction.
-   void GetCoarseningFactors(Array<int> & f) const;
+   void GetCoarseningFactors(Array<int> &f) const;
 
    /// Marks the KnotVector in each dimension as coarse.
    void SetKnotVectorsCoarse(bool c);
 
+   /// Coarsen to a single element.
+   void FullyCoarsen(const Array2D<double> &cp, int ncp1D);
+
+   /// Update piecewise spacing function partitions to match refined @a pkv.
+   void UpdateSpacingPartitions(const Array<KnotVector*> &pkv);
+
    /// Return the number of components stored in the NURBSPatch
    int GetNC() const { return Dim; }
 
@@ -412,7 +437,7 @@ public:
    /// Rotate the NURBSPatch in 2D or 3D..
    /** A rotation of a 2D NURBS-patch requires an angle only. Rotating
        a 3D NURBS-patch requires a normal as well.*/
-   void Rotate(real_t angle, real_t normal[]= NULL);
+   void Rotate(real_t angle, real_t normal[] = NULL);
 
    /// Rotate the NURBSPatch, 2D case.
    void Rotate2D(real_t angle);
@@ -469,7 +494,7 @@ protected:
    /// Orders of all KnotVectors
    Array<int> mOrders;
 
-   /// Number of KnotVectors
+   /// Number of unique (not comprehensive) KnotVectors
    int NumOfKnotVectors;
 
    /// Global entity counts
@@ -490,8 +515,8 @@ protected:
    /// Whether this object owns patchTopo
    bool own_topo;
 
-   /// Map from edge indices to KnotVector indices
-   Array<int> edge_to_knot;
+   /// Map from patchTopo edge indices to unique KnotVector indices
+   Array<int> edge_to_ukv;
 
    /// Set of unique KnotVectors
    Array<KnotVector *> knotVectors;
@@ -543,6 +568,22 @@ protected:
 
    /// Return the unsigned index of the KnotVector for edge @a edge.
    inline int KnotInd(int edge) const;
+   /// Return the sign (orientation) of the KnotVector for edge @a edge.
+   inline int KnotSign(int edge) const;
+
+   bool nonconformingPT = false; /// Whether patchTopo is a nonconforming mesh.
+
+   int num_structured_patches = 0; /// Number of structured patches
+
+   Array3D<double> patchCP; /// Control points for coarse structured patches
+
+   std::vector<Array<int>> kvf, kvf_coarse; /// Knotvector refinement factors
+
+   Array<int> ref_factors; /// Refinement factors in each dimension.
+
+   static constexpr int unsetFactor = 0; /// Unset refinement factor value
+
+   Array<int> dof2patch; /// DOF to owning patch map in @a SetSolutionVector()
 
    /// Access function for the KnotVector associated with edge @a edge.
    /// @note The returned object should NOT be deleted by the caller.
@@ -550,12 +591,12 @@ protected:
    /// Const access function for the KnotVector associated with edge @a edge.
    /// @note The returned object should NOT be deleted by the caller.
    inline const KnotVector *KnotVec(int edge) const;
-   /* brief Const access function for the KnotVector associated with edge
-      @a edge. The output orientation @a okv is set to @a oedge with sign flipped
-      if the KnotVector index associated with edge @a edge is negative. */
+   /** @brief Const access function for the KnotVector associated with edge
+       @a edge. The output orientation @a okv is set to @a oedge with sign flipped
+       if the KnotVector index associated with edge @a edge is negative. */
    inline const KnotVector *KnotVec(int edge, int oedge, int *okv) const;
 
-   /// Throw an error if any patch has an inconsistent edge-to-knot mapping.
+   /// Throw an error if any patch has an inconsistent edge_to_ukv mapping.
    void CheckPatches();
 
    /// Throw an error if any boundary patch has invalid KnotVector orientation.
@@ -603,7 +644,7 @@ protected:
 
    /** @brief Set the mesh and space offsets, and also count the global
    @a NumOfVertices and the global @a NumOfDofs. */
-   void GenerateOffsets();
+   virtual void GenerateOffsets();
 
    /// Count the global @a NumOfElements.
    void CountElements();
@@ -673,13 +714,50 @@ protected:
    /// Set @a patch_to_bel.
    void SetPatchToBdrElements();
 
+   /// Load data from file (used by constructor).
+   void Load(std::istream &input, bool spacing);
+
+   /// Return true if @a edge is a master NC-patch edge.
+   virtual bool IsMasterEdge(int edge) const { return false; }
+
+   /// Return true if @a face is a master NC-patch face.
+   virtual bool IsMasterFace(int face) const { return false; }
+
+   /// Given a pair of vertices, return the corresponding edge.
+   virtual int VertexPairToEdge(const std::pair<int, int> &vertices) const;
+
+   /** @brief Get the DOFs (dof = true) or vertices (dof = false) for
+        master edge @a me. */
+   virtual void GetMasterEdgeDofs(bool dof, int me, Array<int> &dofs) const;
+
+   /** @brief Get the DOFs (dof = true) or vertices (dof = false) for
+       master face @a mf. */
+   virtual void GetMasterFaceDofs(bool dof, int mf, Array2D<int> &dofs) const;
+
+   /// Helper function for @a GenerateOffsets().
+   void GetPatchOffsets(int &meshCounter, int &spaceCounter);
+
+   /// Return NURBSPatch object; returned object should NOT be deleted.
+   const NURBSPatch* GetPatch(int patch) const { return patches[patch]; }
+
    /// To be used by ParNURBSExtension constructor(s)
    NURBSExtension() : el_dof(nullptr), bel_dof(nullptr) { }
 
+private:
+   /// Get the degrees of freedom for the vertex @a vertex in @a dofs.
+   void GetVertexDofs(int vertex, Array<int> &dofs) const;
+
+   /// Get the degrees of freedom for the edge @a edge in @a dofs.
+   void GetEdgeDofs(int edge, Array<int> &dofs) const;
+
+   // TODO: does this still need to be virtual?
+   /// Helper function for @a GenerateOffsets().
+   virtual void SetDofToPatch() { };
+
 public:
    /// Copy constructor: deep copy
    NURBSExtension(const NURBSExtension &orig);
-   /// Read-in a NURBSExtension from a stream @a input..
+   /// Read-in a NURBSExtension from a stream @a input.
    NURBSExtension(std::istream &input, bool spacing=false);
    /** @brief Create a NURBSExtension with elevated order by repeating the
        endpoints of the KnotVectors and using uniform weights of 1. */
@@ -698,7 +776,8 @@ public:
 
    NURBSExtension(Mesh *mesh_array[], int num_pieces);
 
-   NURBSExtension(const Mesh *patch_topology, const Array<const NURBSPatch*> p);
+   NURBSExtension(const Mesh *patch_topology,
+                  const Array<const NURBSPatch*> &patches_);
 
    /// Copy assignment not supported.
    NURBSExtension& operator=(const NURBSExtension&) = delete;
@@ -834,6 +913,9 @@ public:
    int GetPatchBdrAttribute(int i) const
    { return patchTopo->GetBdrAttribute(i); }
 
+   /// Return the number of knotvector elements for edge @a edge.
+   inline int KnotVecNE(int edge) const;
+
    // Load functions
 
    /// Load element @a i into @a FE.
@@ -869,16 +951,23 @@ public:
    void DegreeElevate(int rel_degree, int degree = 16);
 
    /** @brief Refine with optional refinement factor @a rf. Uniform means
-   refinement is done everywhere by the same factor, although nonuniform
-   spacing functions may be used.
-   */
+       refinement is done everywhere by the same factor, although nonuniform
+       spacing functions may be used. */
    void UniformRefinement(int rf = 2);
-   void UniformRefinement(Array<int> const& rf);
+   virtual void UniformRefinement(const Array<int> &rf);
+
+   /// Refine with refinement factors loaded for some knotvectors specified in
+   /// the given file, with default refinement factor @a rf elsewhere. The flag
+   /// @a coarsened indicates whether each patch is a single element.
+   virtual void RefineWithKVFactors(int rf, const std::string &kvf_filename,
+                                    bool coarsened);
+
+   /// Coarsen with optional coarsening factor @a cf.
    void Coarsen(int cf = 2, real_t tol = 1.0e-12);
    void Coarsen(Array<int> const& cf, real_t tol = 1.0e-12);
 
    /** @brief Insert knots from @a kv into all KnotVectors in all patches. The
-        size of @a kv should be the same as @a knotVectors. */
+       size of @a kv should be the same as @a knotVectors. */
    void KnotInsert(Array<KnotVector *> &kv);
    void KnotInsert(Array<Vector *> &kv);
 
@@ -897,8 +986,7 @@ public:
    /** Calls GetCoarseningFactors for each patch and finds the minimum factor
        for each direction that ensures refinement will work in the case of
        non-nested spacing functions. */
-   void GetCoarseningFactors(Array<int> & f) const;
-
+   void GetCoarseningFactors(Array<int> &f) const;
 
    /// Returns the index of the patch containing element @a elem.
    int GetElementPatch(int elem) const { return el_to_patch[elem]; }
@@ -911,10 +999,34 @@ public:
        Cartesian order. */
    void GetPatchDofs(const int patch, Array<int> &dofs);
 
+   /// Returns a deep copy of the patch topology mesh
+   Mesh GetPatchTopology() const { return Mesh(*patchTopo); }
+
+   /** Returns a deep copy of all instantiated patches. To ensure that patches
+       are instantiated, use Mesh::GetNURBSPatches() instead. Caller gets
+       ownership of the returned object, and is responsible for deletion.*/
+   void GetPatches(Array<NURBSPatch*> &patches);
+
    /// Return the array of indices of all elements in patch @a patch.
    const Array<int>& GetPatchElements(int patch);
    /// Return the array of indices of all boundary elements in patch @a patch.
    const Array<int>& GetPatchBdrElements(int patch);
+
+   /// Return true if the patch topology mesh is nonconforming.
+   bool NonconformingPatches() const { return nonconformingPT; }
+
+   /// Return a pointer to the NCMesh of a nonconforming patch topology mesh.
+   NCMesh *GetNCMesh() const { return patchTopo->ncmesh; }
+
+   /// Read the control points for coarse patches.
+   virtual void ReadCoarsePatchCP(std::istream &input);
+
+   /** @brief Fully coarsen all structured patches, for non-nested refinement of
+       a mesh with a nonconforming patch topology. */
+   void FullyCoarsen();
+
+   /// Print control points for coarse patches.
+   virtual void PrintCoarsePatches(std::ostream &os);
 };
 
 
@@ -965,23 +1077,31 @@ public:
 #endif
 
 
-/** @brief Mapping for mesh vertices and NURBS space DOFs. */
+/** @brief Mapping for mesh vertices and NURBS space DOFs on a patch.
+
+    This class has two modes, for vertices or DOFs, depending on whether
+    @a SetPatchVertexMap or @a SetPatchDofMap is called.
+ */
 class NURBSPatchMap
 {
 private:
    /// This object must be associated with exactly one NURBSExtension.
    const NURBSExtension *Ext;
 
-   /// Number of elements in each direction, minus 1.
+   /// Vertex mode: Number of elements in each direction, minus 1.
+   /// DOF mode: Number of control points in each direction, minus 2.
    int I, J, K;
 
-   /// Vertex of DOF offset for this patch, among all patches.
+   /// Vertex/DOF offset for this patch, among all patches.
    int pOffset;
    /// Orientation for this boundary patch (0 in the patch case).
    int opatch;
 
    /// Patch topology entities for this patch or boundary patch.
    Array<int> verts, edges, faces, oedge, oface;
+   Array<bool> edgeMaster, faceMaster;
+   Array<int> edgeMasterOffset, faceMasterOffset;
+   Array<int> masterDofs;
 
    inline static int F(const int n, const int N)
    { return (n < 0) ? 0 : ((n >= N) ? 2 : 1); }
@@ -992,6 +1112,27 @@ private:
    inline static int Or2D(const int n1, const int n2,
                           const int N1, const int N2, const int Or);
 
+   inline int EC(const int e, const int n, const int N, const int s=1) const
+   {
+      return !edgeMaster[e] ? edges[e] + Or1D(n, N, s*oedge[e]) :
+             GetMasterEdgeDof(e, Or1D(n, N, s*oedge[e]));
+   }
+
+   inline int FC(const int f, const int m, const int n,
+                 const int M, const int N) const
+   {
+      return !faceMaster[f] ? faces[f] + Or2D(m, n, M, N, oface[f]) :
+             GetMasterFaceDof(f, Or2D(m, n, M, N, oface[f]));
+   }
+
+   inline int FCP(const int f, const int m, const int n,
+                  const int M, const int N) const
+   {
+      return (faceMaster.Size() == 0 || !faceMaster[f]) ?
+             pOffset + Or2D(m, n, M, N, opatch) :
+             GetMasterFaceDof(f, Or2D(m, n, M, N, opatch));
+   }
+
    // The following 2 functions also set verts, edges, faces, orientations etc.
 
    /// Get the KnotVectors for patch @a p in @a kv.
@@ -1000,15 +1141,25 @@ private:
        orientations output in @a okv. */
    void GetBdrPatchKnotVectors(int bp, const KnotVector *kv[], int *okv);
 
+   void SetMasterEdges(bool dof, const KnotVector *kv[] = nullptr);
+   void SetMasterFaces(bool dof);
+   int GetMasterEdgeDof(const int e, const int i) const;
+   int GetMasterFaceDof(const int f, const int i) const;
+
 public:
    /// Constructor for an object associated with NURBSExtension @a ext.
    NURBSPatchMap(const NURBSExtension *ext) { Ext = ext; }
 
-   /// Return the number of elements in the first direction.
+   /// Vertex mode: Return the number of elements in the first direction.
+   /// DOF mode: Return the number of control points - 1 in the first direction.
    inline int nx() const { return I + 1; }
-   /// Return the number of elements in the second direction (2D or 3D).
+
+   /// Vertex mode: Return the number of elements in the second direction (2D or 3D).
+   /// DOF mode: Return the number of control points - 1 in the second direction (2D or 3D).
    inline int ny() const { return J + 1; }
-   /// Return the number of elements in the third direction (3D).
+
+   /// Vertex mode: Return the number of elements in the third direction (3D).
+   /// DOF mode: Return the number of control points - 1 in the third direction (3D).
    inline int nz() const { return K + 1; }
 
    /// Set mesh vertex map for patch @a p with KnotVectors @a kv.
@@ -1138,8 +1289,13 @@ inline const real_t &NURBSPatch::operator()(int i, int j, int k, int l) const
 
 inline int NURBSExtension::KnotInd(int edge) const
 {
-   int kv = edge_to_knot[edge];
-   return (kv >= 0) ? kv : (-1-kv);
+   const int kv = edge_to_ukv[edge];
+   return kv >= 0 ? kv : -1 - kv;
+}
+
+inline int NURBSExtension::KnotSign(int edge) const
+{
+   return edge_to_ukv[edge] >= 0 ? 1 : -1;
 }
 
 inline KnotVector *NURBSExtension::KnotVec(int edge)
@@ -1155,7 +1311,7 @@ inline const KnotVector *NURBSExtension::KnotVec(int edge) const
 inline const KnotVector *NURBSExtension::KnotVec(int edge, int oedge, int *okv)
 const
 {
-   int kv = edge_to_knot[edge];
+   int kv = edge_to_ukv[edge];
    if (kv >= 0)
    {
       *okv = oedge;
@@ -1168,12 +1324,15 @@ const
    }
 }
 
+inline int NURBSExtension::KnotVecNE(int edge) const
+{
+   return knotVectors[KnotInd(edge)]->GetNE();
+}
 
 // static method
 inline int NURBSPatchMap::Or2D(const int n1, const int n2,
                                const int N1, const int N2, const int Or)
 {
-   // Needs testing
    switch (Or)
    {
       case 0: return n1 + n2*N1;
@@ -1197,7 +1356,9 @@ inline int NURBSPatchMap::operator()(const int i) const
    switch (F(i1, I))
    {
       case 0: return verts[0];
-      case 1: return pOffset + Or1D(i1, I, opatch);
+      case 1: return edgeMaster.Size() > 0 && edgeMaster[0] ?
+                        GetMasterEdgeDof(0, Or1D(i1, I, opatch)) :
+                        pOffset + Or1D(i1, I, opatch);
       case 2: return verts[1];
    }
 #ifdef MFEM_DEBUG
@@ -1212,13 +1373,13 @@ inline int NURBSPatchMap::operator()(const int i, const int j) const
    switch (3*F(j1, J) + F(i1, I))
    {
       case 0: return verts[0];
-      case 1: return edges[0] + Or1D(i1, I, oedge[0]);
+      case 1: return EC(0, i1, I);
       case 2: return verts[1];
-      case 3: return edges[3] + Or1D(j1, J, -oedge[3]);
-      case 4: return pOffset + Or2D(i1, j1, I, J, opatch);
-      case 5: return edges[1] + Or1D(j1, J, oedge[1]);
+      case 3: return EC(3, j1, J, -1);
+      case 4: return FCP(0, i1, j1, I, J);
+      case 5: return EC(1, j1, J);
       case 6: return verts[3];
-      case 7: return edges[2] + Or1D(i1, I, -oedge[2]);
+      case 7: return EC(2, i1, I, -1);
       case 8: return verts[2];
    }
 #ifdef MFEM_DEBUG
@@ -1230,36 +1391,35 @@ inline int NURBSPatchMap::operator()(const int i, const int j) const
 inline int NURBSPatchMap::operator()(const int i, const int j, const int k)
 const
 {
-   // Needs testing
    const int i1 = i - 1, j1 = j - 1, k1 = k - 1;
    switch (3*(3*F(k1, K) + F(j1, J)) + F(i1, I))
    {
       case  0: return verts[0];
-      case  1: return edges[0] + Or1D(i1, I, oedge[0]);
+      case  1: return EC(0, i1, I);
       case  2: return verts[1];
-      case  3: return edges[3] + Or1D(j1, J, oedge[3]);
-      case  4: return faces[0] + Or2D(i1, J - 1 - j1, I, J, oface[0]);
-      case  5: return edges[1] + Or1D(j1, J, oedge[1]);
+      case  3: return EC(3, j1, J);
+      case  4: return FC(0, i1, J - 1 - j1, I, J);
+      case  5: return EC(1, j1, J);
       case  6: return verts[3];
-      case  7: return edges[2] + Or1D(i1, I, oedge[2]);
+      case  7: return EC(2, i1, I);
       case  8: return verts[2];
-      case  9: return edges[8] + Or1D(k1, K, oedge[8]);
-      case 10: return faces[1] + Or2D(i1, k1, I, K, oface[1]);
-      case 11: return edges[9] + Or1D(k1, K, oedge[9]);
-      case 12: return faces[4] + Or2D(J - 1 - j1, k1, J, K, oface[4]);
+      case  9: return EC(8, k1, K);
+      case 10: return FC(1, i1, k1, I, K);
+      case 11: return EC(9, k1, K);
+      case 12: return FC(4, J - 1 - j1, k1, J, K);
       case 13: return pOffset + I*(J*k1 + j1) + i1;
-      case 14: return faces[2] + Or2D(j1, k1, J, K, oface[2]);
-      case 15: return edges[11] + Or1D(k1, K, oedge[11]);
-      case 16: return faces[3] + Or2D(I - 1 - i1, k1, I, K, oface[3]);
-      case 17: return edges[10] + Or1D(k1, K, oedge[10]);
+      case 14: return FC(2, j1, k1, J, K);
+      case 15: return EC(11, k1, K);
+      case 16: return FC(3, I - 1 - i1, k1, I, K);
+      case 17: return EC(10, k1, K);
       case 18: return verts[4];
-      case 19: return edges[4] + Or1D(i1, I, oedge[4]);
+      case 19: return EC(4, i1, I);
       case 20: return verts[5];
-      case 21: return edges[7] + Or1D(j1, J, oedge[7]);
-      case 22: return faces[5] + Or2D(i1, j1, I, J, oface[5]);
-      case 23: return edges[5] + Or1D(j1, J, oedge[5]);
+      case 21: return EC(7, j1, J);
+      case 22: return FC(5, i1, j1, I, J);
+      case 23: return EC(5, j1, J);
       case 24: return verts[7];
-      case 25: return edges[6] + Or1D(i1, I, oedge[6]);
+      case 25: return EC(6, i1, I);
       case 26: return verts[6];
    }
 #ifdef MFEM_DEBUG
