@@ -79,7 +79,7 @@ enum prob_type
 };
 
 prob_type prob;
-Vector beta;
+Vector beta_;
 real_t epsilon;
 
 real_t exact_u(const Vector & X);
@@ -120,7 +120,7 @@ int main(int argc, char *argv[])
                   "Theta parameter for AMR");
    args.AddOption(&iprob, "-prob", "--problem", "Problem case"
                   " 0: manufactured, 1: Erickson-Johnson ");
-   args.AddOption(&beta, "-beta", "--beta",
+   args.AddOption(&beta_, "-beta", "--beta",
                   "Vector Coefficient beta");
    args.AddOption(&static_cond, "-sc", "--static-condensation", "-no-sc",
                   "--no-static-condensation", "Enable static condensation.");
@@ -147,11 +147,11 @@ int main(int argc, char *argv[])
    int dim = mesh.Dimension();
    MFEM_VERIFY(dim > 1, "Dimension = 1 is not supported in this example");
 
-   if (beta.Size() == 0)
+   if (beta_.Size() == 0)
    {
-      beta.SetSize(dim);
-      beta = 0.0;
-      beta[0] = 1.;
+      beta_.SetSize(dim);
+      beta_ = 0.0;
+      beta_[0] = 1.;
    }
 
    args.PrintOptions(std::cout);
@@ -199,10 +199,10 @@ int main(int argc, char *argv[])
    ConstantCoefficient eps2(1/(epsilon*epsilon));
 
    ConstantCoefficient negeps(-epsilon);
-   VectorConstantCoefficient betacoeff(beta);
-   Vector negbeta = beta; negbeta.Neg();
-   DenseMatrix bbt(beta.Size());
-   MultVVt(beta, bbt);
+   VectorConstantCoefficient betacoeff(beta_);
+   Vector negbeta = beta_; negbeta.Neg();
+   DenseMatrix bbt(beta_.Size());
+   MultVVt(beta_, bbt);
    MatrixConstantCoefficient bbtcoeff(bbt);
    VectorConstantCoefficient negbetacoeff(negbeta);
 
@@ -598,7 +598,7 @@ void exact_hatf(const Vector & X, Vector & hatf)
    hatf.SetSize(X.Size());
    for (int i = 0; i<hatf.Size(); i++)
    {
-      hatf[i] = beta[i] * u - sigma[i];
+      hatf[i] = beta_[i] * u - sigma[i];
    }
 }
 
@@ -612,7 +612,7 @@ real_t f_exact(const Vector & X)
    real_t s = 0;
    for (int i = 0; i<du.Size(); i++)
    {
-      s += beta[i] * du[i];
+      s += beta_[i] * du[i];
    }
    return -epsilon * d2u + s;
 }
@@ -628,7 +628,7 @@ void setup_test_norm_coeffs(GridFunction & c1_gf, GridFunction & c2_gf)
       real_t c1 = std::min(epsilon/volume, (real_t) 1.);
       real_t c2 = std::min(1./epsilon, 1./volume);
       fes->GetElementDofs(i,vdofs);
-      c1_gf.SetSubVector(vdofs,c1);
-      c2_gf.SetSubVector(vdofs,c2);
+      c1_gf.SetSubVectorHost(vdofs,c1);
+      c2_gf.SetSubVectorHost(vdofs,c2);
    }
 }
