@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -589,12 +589,11 @@ VoltaSolver::InitializeGLVis()
 }
 
 void
-VoltaSolver::DisplayToGLVis()
+VoltaSolver::DisplayToGLVis(int visport)
 {
    if (myid_ == 0) { cout << "Sending data to GLVis ..." << flush; }
 
    char vishost[] = "localhost";
-   int  visport   = 19916;
 
    int Wx = 0, Wy = 0; // window position
    int Ww = 350, Wh = 350; // window size
