@@ -10,10 +10,9 @@
 // CONTRIBUTING.md for details.
 
 #include "diagonal_preconditioner.hpp"
-#include "general/forall.hpp"
 #include "linalg/tensor.hpp"
 
-using mfem::internal::tensor;
+using mfem::future::tensor;
 
 namespace mfem
 {
