@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -32,6 +32,44 @@ real_t size_indicator(const Vector &x)
    return val;
 }
 
+real_t size_indicator_periodic(const Vector &x)
+{
+   // top right
+   real_t xc = x(0) - 0.75, yc = x(1) - 0.75,
+          zc = (x.Size() == 3) ? x(2) - 0.0 : 0.0;
+   real_t r = sqrt(xc*xc + yc*yc + zc*zc);
+   real_t r1 = 0.45; real_t r2 = 0.55; real_t sf=30.0;
+   real_t val = 0.5*(1+std::tanh(sf*(r-r1))) - 0.5*(1+std::tanh(sf*(r-r2)));
+   val = std::max((real_t) 0.,val);
+   val = std::min((real_t) 1.,val);
+
+   // bottom right
+   xc = x(0) - 0.75; yc = x(1) + 1.25;
+   zc = (x.Size() == 3) ? x(2) - 0.0 : 0.0;
+   r = sqrt(xc*xc + yc*yc + zc*zc);
+   r1 = 0.45; r2 = 0.55; sf=30.0;
+   real_t val1 = 0.5*(1+std::tanh(sf*(r-r1))) - 0.5*(1+std::tanh(sf*(r-r2)));
+   val = std::max(val, val1);
+
+   // top left
+   xc = x(0) + 1.25; yc = x(1) - 0.75;
+   zc = (x.Size() == 3) ? x(2) - 0.0 : 0.0;
+   r = sqrt(xc*xc + yc*yc + zc*zc);
+   r1 = 0.45; r2 = 0.55; sf=30.0;
+   real_t val2 = 0.5*(1+std::tanh(sf*(r-r1))) - 0.5*(1+std::tanh(sf*(r-r2)));
+   val = std::max(val, val2);
+
+   // bottom left
+   xc = x(0) + 1.25; yc = x(1) + 1.25;
+   zc = (x.Size() == 3) ? x(2) - 0.0 : 0.0;
+   r = sqrt(xc*xc + yc*yc + zc*zc);
+   r1 = 0.45; r2 = 0.55; sf=30.0;
+   real_t val3 = 0.5*(1+std::tanh(sf*(r-r1))) - 0.5*(1+std::tanh(sf*(r-r2)));
+   val = std::max(val, val3);
+
+   return val;
+}
+
 void calc_mass_volume(const GridFunction &g, real_t &mass, real_t &vol)
 {
    Mesh &mesh = *g.FESpace()->GetMesh();
@@ -68,9 +106,19 @@ void calc_mass_volume(const GridFunction &g, real_t &mass, real_t &vol)
 
 void ConstructSizeGF(GridFunction &size)
 {
+   const bool per = size.FESpace()->GetMesh()->GetNodalFESpace()->IsDGSpace();
+
    // Indicator for small (value -> 1) or big (value -> 0) elements.
-   FunctionCoefficient size_ind_coeff(size_indicator);
-   size.ProjectCoefficient(size_ind_coeff);
+   if (per)
+   {
+      FunctionCoefficient size_ind_coeff(size_indicator_periodic);
+      size.ProjectCoefficient(size_ind_coeff);
+   }
+   else
+   {
+      FunctionCoefficient size_ind_coeff(size_indicator);
+      size.ProjectCoefficient(size_ind_coeff);
+   }
 
    // Determine small/big target sizes based on the total number of
    // elements and the volume occupied by small elements.
@@ -140,8 +188,8 @@ public:
    HessianCoefficient(int dim, int metric_id)
       : TMOPMatrixCoefficient(dim), metric(metric_id) { }
 
-   virtual void Eval(DenseMatrix &K, ElementTransformation &T,
-                     const IntegrationPoint &ip)
+   void Eval(DenseMatrix &K, ElementTransformation &T,
+             const IntegrationPoint &ip) override
    {
       Vector pos(3);
       T.Transform(ip, pos);
@@ -207,8 +255,8 @@ public:
       }
    }
 
-   virtual void EvalGrad(DenseMatrix &K, ElementTransformation &T,
-                         const IntegrationPoint &ip, int comp)
+   void EvalGrad(DenseMatrix &K, ElementTransformation &T,
+                 const IntegrationPoint &ip, int comp) override
    {
       Vector pos(3);
       T.Transform(ip, pos);
@@ -251,8 +299,8 @@ public:
       : TMOPMatrixCoefficient(dim_), dim(dim_),
         hr_target_type(hr_target_type_) { }
 
-   virtual void Eval(DenseMatrix &K, ElementTransformation &T,
-                     const IntegrationPoint &ip)
+   void Eval(DenseMatrix &K, ElementTransformation &T,
+             const IntegrationPoint &ip) override
    {
       Vector pos(3);
       T.Transform(ip, pos);
@@ -364,8 +412,8 @@ public:
       else { MFEM_ABORT("Unsupported option / wrong input."); }
    }
 
-   virtual void EvalGrad(DenseMatrix &K, ElementTransformation &T,
-                         const IntegrationPoint &ip, int comp)
+   void EvalGrad(DenseMatrix &K, ElementTransformation &T,
+                 const IntegrationPoint &ip, int comp) override
    {
       K = 0.;
    }
