@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -67,8 +67,8 @@ public:
       meshOrig(mesh), r2o(r2o_), perm(refPerm)
    { }
 
-   virtual void Eval(Vector &V, ElementTransformation &T,
-                     const IntegrationPoint &ip);
+   void Eval(Vector &V, ElementTransformation &T,
+             const IntegrationPoint &ip) override;
 
    using VectorCoefficient::Eval;
 };
@@ -1019,6 +1019,7 @@ int main(int argc, char *argv[])
 
    normal = 0.0;
    normal[2] = 1.0;
+   int visport = 19916;
    origin = 0.0;
 
    OptionsParser args(argc, argv);
@@ -1031,6 +1032,7 @@ int main(int argc, char *argv[])
    args.AddOption(&visualization, "-vis", "--visualization", "-no-vis",
                   "--no-visualization",
                   "Enable or disable GLVis visualization.");
+   args.AddOption(&visport, "-p", "--send-port", "Socket for GLVis.");
    args.Parse();
    if (!args.Good())
    {
@@ -1054,7 +1056,6 @@ int main(int argc, char *argv[])
    {
       // GLVis server to visualize to
       char vishost[] = "localhost";
-      int  visport   = 19916;
       socketstream sol_sock(vishost, visport);
       sol_sock.precision(8);
       sol_sock << "mesh\n" << *reflected << flush;
