@@ -25,7 +25,8 @@ namespace mfem
 ///        L-Vectors. For example, a block may be a BilinearForm.
 class BlockFESpaceOperator : public Operator
 {
-private:
+   using FESVector = std::vector<const FiniteElementSpace*>;
+
    /// Offsets for the square "A" operator.
    Array<int> offsets;
    /// Column offsets for the prolongation operator.
@@ -39,33 +40,27 @@ private:
    /// Maps true dofs of each block to local dofs.
    BlockOperator restriction;
    /// Computes height for parent operator.
-   static int GetHeight(const std::vector<const FiniteElementSpace*>
-                        &fespaces);
+   static int GetHeight(const FESVector &fespaces);
    /// Computes offsets for A BlockOperator.
-   static Array<int> GetBlockOffsets(const std::vector<const FiniteElementSpace*>
-                                     &fespaces);
+   static Array<int> GetBlockOffsets(const FESVector &fespaces);
    /// Computes col_offsets for prolongation operator.
-   static Array<int> GetProColBlockOffsets(const
-                                           std::vector<const FiniteElementSpace*> &fespaces);
+   static Array<int> GetProColBlockOffsets(const FESVector &fespaces);
    /// Computes row_offsets for restriction operator.
-   static Array<int> GetResRowBlockOffsets(const
-                                           std::vector<const FiniteElementSpace*> &fespaces);
+   static Array<int> GetResRowBlockOffsets(const FESVector &fespaces);
+
 public:
    /// @brief Constructor for BlockFESpaceOperator.
    /// @param[in] fespaces Finite element spaces for diagonal blocks. Spaces are not owned.
-   BlockFESpaceOperator(const std::vector<const FiniteElementSpace*> &fespaces);
+   BlockFESpaceOperator(const FESVector &fespaces);
    const Operator* GetProlongation () const override;
    const Operator* GetRestriction () const override;
    void Mult(const Vector &x, Vector &y) const override {A.Mult(x,y);};
    /// @brief Wraps BlockOperator::SetBlock. Eventually would like this class to inherit
    /// from BlockOperator instead, but can't easily due to ownership of offset data
    /// in BlockOperator being by reference.
-   void SetBlock( int   iRow,
-                  int   iCol,
-                  Operator *  op,
-                  real_t   c = 1.0) {A.SetBlock(iRow, iCol, op, c);};
+   void SetBlock(int iRow, int iCol, Operator *op, real_t c = 1.0) { A.SetBlock(iRow, iCol, op, c); };
 };
 
 } // namespace mfem
 
-#endif
+#endif // MFEM_BLOCK_FESPACE_OPERATOR
