@@ -13,13 +13,10 @@
 #define MFEM_DARCY_SOLVER_HPP
 
 #include "mfem.hpp"
-#include <memory>
-#include <vector>
 
-namespace mfem
-{
-namespace blocksolvers
+namespace mfem::blocksolvers
 {
+
 struct IterSolveParameters
 {
    int print_level = 0;
@@ -32,8 +29,6 @@ struct IterSolveParameters
    real_t rel_tol = 1e-5;
 #else
 #error "Only single and double precision are supported!"
-   real_t abs_tol = 1e-12;
-   real_t rel_tol = 1e-9;
 #endif
 };
 
@@ -68,7 +63,7 @@ public:
    void SetEssZeroDofs(const Array<int>& dofs) { dofs.Copy(ess_zero_dofs_); }
    int GetNumIterations() const override { return solver_.GetNumIterations(); }
 };
-} // namespace blocksolvers
-} // namespace mfem
+
+} // namespace mfem::blocksolvers
 
 #endif // MFEM_DARCY_SOLVER_HPP
