@@ -13,16 +13,16 @@
 
 using namespace std;
 
-namespace mfem
+namespace mfem::blocksolvers
 {
-namespace blocksolvers
+
+static HypreParMatrix* TwoStepsRAP(const HypreParMatrix *Rt,
+                                   const HypreParMatrix *A,
+                                   const HypreParMatrix *P)
 {
-HypreParMatrix* TwoStepsRAP(const HypreParMatrix& Rt, const HypreParMatrix& A,
-                            const HypreParMatrix& P)
-{
-   OperatorPtr R(Rt.Transpose());
-   OperatorPtr RA(ParMult(R.As<HypreParMatrix>(), &A));
-   return ParMult(RA.As<HypreParMatrix>(), &P, true);
+   OperatorPtr R(Rt->Transpose());
+   OperatorPtr RA(ParMult(R.As<HypreParMatrix>(), A));
+   return ParMult(RA.As<HypreParMatrix>(), P, true);
 }
 
 void GetRowColumnsRef(const SparseMatrix& A, int row, Array<int>& cols)
@@ -59,34 +59,36 @@ DFSSpaces::DFSSpaces(int order, int num_refine, ParMesh *mesh,
 
    if (mesh->Dimension() == 3)
    {
-      hcurl_fec_.reset(new ND_FECollection(order+1, mesh->Dimension()));
+      hcurl_fec_ = std::make_unique<ND_FECollection>(order+1, mesh->Dimension());
    }
    else
    {
-      hcurl_fec_.reset(new H1_FECollection(order+1, mesh->Dimension()));
+      hcurl_fec_ = std::make_unique<H1_FECollection>(order+1, mesh->Dimension());
    }
 
    all_bdr_attr_.SetSize(ess_attr.Size(), 1);
-   hdiv_fes_.reset(new ParFiniteElementSpace(mesh, &hdiv_fec_));
-   l2_fes_.reset(new ParFiniteElementSpace(mesh, &l2_fec_));
-   coarse_hdiv_fes_.reset(new ParFiniteElementSpace(*hdiv_fes_));
-   coarse_l2_fes_.reset(new ParFiniteElementSpace(*l2_fes_));
-   l2_0_fes_.reset(new ParFiniteElementSpace(mesh, &l2_0_fec_));
+   hdiv_fes_ = std::make_unique<ParFiniteElementSpace>(mesh, &hdiv_fec_);
+   l2_fes_ = std::make_unique<ParFiniteElementSpace>(mesh, &l2_fec_);
+   coarse_hdiv_fes_ = std::make_unique<ParFiniteElementSpace>(*hdiv_fes_);
+   coarse_l2_fes_ = std::make_unique<ParFiniteElementSpace>(*l2_fes_);
+   l2_0_fes_ = std::make_unique<ParFiniteElementSpace>(mesh, &l2_0_fec_);
    l2_0_fes_->SetUpdateOperatorType(Operator::MFEM_SPARSEMAT);
    el_l2dof_.reserve(num_refine+1);
    el_l2dof_.push_back(ElemToDof(*coarse_l2_fes_));
 
    data_.agg_hdivdof.resize(num_refine);
    data_.agg_l2dof.resize(num_refine);
-   data_.P_hdiv.resize(num_refine, OperatorPtr(Operator::Hypre_ParCSR));
-   data_.P_l2.resize(num_refine, OperatorPtr(Operator::Hypre_ParCSR));
+   data_.P_hdiv.resize(num_refine);
+   data_.P_l2.resize(num_refine);
+
    data_.Q_l2.resize(num_refine);
    hdiv_fes_->GetEssentialTrueDofs(ess_attr, data_.coarsest_ess_hdivdofs);
    data_.C.resize(num_refine+1);
+   data_.Ae.resize(num_refine+1);
 
-   hcurl_fes_.reset(new ParFiniteElementSpace(mesh, hcurl_fec_.get()));
-   coarse_hcurl_fes_.reset(new ParFiniteElementSpace(*hcurl_fes_));
-   data_.P_hcurl.resize(num_refine, OperatorPtr(Operator::Hypre_ParCSR));
+   hcurl_fes_ = std::make_unique<ParFiniteElementSpace>(mesh, hcurl_fec_.get());
+   coarse_hcurl_fes_ = std::make_unique<ParFiniteElementSpace>(*hcurl_fes_);
+   data_.P_hcurl.resize(num_refine);
 }
 
 SparseMatrix* AggToInteriorDof(const Array<int>& bdr_truedofs,
@@ -104,8 +106,8 @@ SparseMatrix* AggToInteriorDof(const Array<int>& bdr_truedofs,
    agg_tdof_T.As<HypreParMatrix>()->GetDiag(tdof_agg);
    agg_tdof_T.As<HypreParMatrix>()->GetOffd(is_shared, trash);
 
-   int * I = new int [tdof_agg.NumRows()+1]();
-   int * J = new int[tdof_agg.NumNonZeroElems()];
+   int *I = new int[tdof_agg.NumRows()+1]();
+   int *J = new int[tdof_agg.NumNonZeroElems()];
 
    Array<int> is_bdr;
    FiniteElementSpace::ListToMarker(bdr_truedofs, tdof_agg.NumRows(), is_bdr);
@@ -119,7 +121,7 @@ SparseMatrix* AggToInteriorDof(const Array<int>& bdr_truedofs,
       J[counter++] = tdof_agg.GetRowColumns(i)[0];
    }
 
-   real_t * D = new real_t[I[tdof_agg.NumRows()]];
+   auto *D = new real_t[I[tdof_agg.NumRows()]];
    std::fill_n(D, I[tdof_agg.NumRows()], 1.0);
 
    SparseMatrix intdof_agg(I, J, D, tdof_agg.NumRows(), tdof_agg.NumCols());
@@ -146,20 +148,21 @@ void DFSSpaces::MakeDofRelationTables(int level)
 
 void DFSSpaces::CollectDFSData()
 {
-   auto GetP = [this](OperatorPtr& P, unique_ptr<ParFiniteElementSpace>& cfes,
-                      ParFiniteElementSpace& fes, bool remove_zero)
+   auto GetP = [&](std::unique_ptr<OperatorPtr> &P,
+                   std::unique_ptr<ParFiniteElementSpace> &cfes,
+                   ParFiniteElementSpace& fes, const bool remove_zero)
    {
       fes.Update();
-      fes.GetTrueTransferOperator(*cfes, P);
-      if (remove_zero)
-      {
-         P.As<HypreParMatrix>()->DropSmallEntries(1e-16);
-      }
+      auto T = new OperatorHandle(Operator::Hypre_ParCSR);
+      fes.GetTrueTransferOperator(*cfes, *T);
+      P.reset(T);
+      if (remove_zero) { P->As<HypreParMatrix>()->DropSmallEntries(1e-16); }
       (level_ < (int)data_.P_l2.size()-1) ? cfes->Update() : cfes.reset();
    };
 
    GetP(data_.P_hdiv[level_], coarse_hdiv_fes_, *hdiv_fes_, true);
    GetP(data_.P_l2[level_], coarse_l2_fes_, *l2_fes_, false);
+
    MakeDofRelationTables(level_);
 
    GetP(data_.P_hcurl[level_], coarse_hcurl_fes_, *hcurl_fes_, true);
@@ -171,7 +174,9 @@ void DFSSpaces::CollectDFSData()
    data_.C[level_+1].Reset(curl.ParallelAssemble());
    mfem::Array<int> ess_hcurl_tdof;
    hcurl_fes_->GetEssentialTrueDofs(ess_bdr_attr_, ess_hcurl_tdof);
-   data_.C[level_+1].As<HypreParMatrix>()->EliminateCols(ess_hcurl_tdof);
+   data_.Ae[level_+1].reset(
+      data_.C[level_+1].As<HypreParMatrix>()
+      ->EliminateCols(ess_hcurl_tdof));
 
    ++level_;
 
@@ -189,7 +194,7 @@ void DFSSpaces::DataFinalize()
    SparseMatrix P_l2;
    for (int l = (int)data_.P_l2.size()-1; l >= 0; --l)
    {
-      data_.P_l2[l].As<HypreParMatrix>()->GetDiag(P_l2);
+      data_.P_l2[l]->As<HypreParMatrix>()->GetDiag(P_l2);
       OperatorPtr PT_l2(Transpose(P_l2));
       auto PTW = Mult(*PT_l2.As<SparseMatrix>(), *W.As<SparseMatrix>());
       auto cW = Mult(*PTW, P_l2);
@@ -245,7 +250,7 @@ SaddleSchwarzSmoother::SaddleSchwarzSmoother(const HypreParMatrix& M,
                                              const SparseMatrix& agg_hdivdof,
                                              const SparseMatrix& agg_l2dof,
                                              const HypreParMatrix& P_l2,
-                                             const HypreParMatrix& Q_l2)
+                                             const ProductOperator& Q_l2)
    : Solver(M.NumRows() + B.NumRows()), agg_hdivdof_(agg_hdivdof),
      agg_l2dof_(agg_l2dof), solvers_loc_(agg_l2dof.NumRows())
 {
@@ -312,23 +317,27 @@ void SaddleSchwarzSmoother::Mult(const Vector & x, Vector & y) const
    blk_y.GetBlock(1) -= coarse_l2_projection;
 }
 
-DivFreeSolver::DivFreeSolver(const HypreParMatrix &M, const HypreParMatrix& B,
+DivFreeSolver::DivFreeSolver(const HypreParMatrix &M,
+                             const HypreParMatrix &B,
                              const DFSData& data)
    : DarcySolver(M.NumRows(), B.NumRows()), data_(data), param_(data.param),
-     BT_(B.Transpose()), BBT_solver_(B, param_.BBT_solve_param),
-     ops_offsets_(data.P_l2.size()+1), ops_(ops_offsets_.size()),
-     blk_Ps_(ops_.Size()-1), smoothers_(ops_.Size())
+     BT_(B.Transpose()),
+     BBT_solver_(B, param_.BBT_solve_param),
+     ops_offsets_(data.P_l2.size()+1),
+     ops_(ops_offsets_.size()),
+     blk_Ps_(ops_.size()-1),
+     smoothers_(ops_.size())
 {
    ops_offsets_.back().MakeRef(DarcySolver::offsets_);
-   ops_.Last() = new BlockOperator(ops_offsets_.back());
-   ops_.Last()->SetBlock(0, 0, const_cast<HypreParMatrix*>(&M));
-   ops_.Last()->SetBlock(1, 0, const_cast<HypreParMatrix*>(&B));
-   ops_.Last()->SetBlock(0, 1, BT_.Ptr());
+   ops_.back() = std::make_unique<BlockOperator>(ops_offsets_.back());
+   ops_.back()->SetBlock(0, 0, const_cast<HypreParMatrix*>(&M));
+   ops_.back()->SetBlock(1, 0, const_cast<HypreParMatrix*>(&B));
+   ops_.back()->SetBlock(0, 1, BT_.Ptr());
 
    for (int l = data.P_l2.size(); l >= 0; --l)
    {
-      auto& M_f = static_cast<const HypreParMatrix&>(ops_[l]->GetBlock(0, 0));
-      auto& B_f = static_cast<const HypreParMatrix&>(ops_[l]->GetBlock(1, 0));
+      auto &M_f = static_cast<const HypreParMatrix&>(ops_[l]->GetBlock(0, 0));
+      auto &B_f = static_cast<const HypreParMatrix&>(ops_[l]->GetBlock(1, 0));
 
       if (l == 0)
       {
@@ -343,123 +352,112 @@ DivFreeSolver::DivFreeSolver(const HypreParMatrix &M, const HypreParMatrix& B,
 
          const IterSolveParameters& param = param_.coarse_solve_param;
          auto coarse_solver = new BDPMinresSolver(M_f, B_f, param);
-         if (ops_.Size() > 1)
+         if (ops_.size() > 1)
          {
             coarse_solver->SetEssZeroDofs(data.coarsest_ess_hdivdofs);
          }
-         smoothers_[l] = coarse_solver;
+         smoothers_[l].reset(coarse_solver);
          continue;
       }
 
-      HypreParMatrix& P_hdiv_l = *data.P_hdiv[l-1].As<HypreParMatrix>();
-      HypreParMatrix& P_l2_l = *data.P_l2[l-1].As<HypreParMatrix>();
+      auto P_hdiv_l = data.P_hdiv[l-1]->As<HypreParMatrix>();
+      auto P_l2_l = data.P_l2[l-1]->As<HypreParMatrix>();
       SparseMatrix& agg_hdivdof_l = *data.agg_hdivdof[l-1].As<SparseMatrix>();
       SparseMatrix& agg_l2dof_l = *data.agg_l2dof[l-1].As<SparseMatrix>();
-      HypreParMatrix& Q_l2_l = *data.Q_l2[l-1].As<HypreParMatrix>();
-      HypreParMatrix* C_l = data.C[l].As<HypreParMatrix>();
+      ProductOperator& Q_l2_l = *data.Q_l2[l-1].As<ProductOperator>();
+      auto* C_l = data.C[l].As<HypreParMatrix>();
 
       auto S0 = new SaddleSchwarzSmoother(M_f, B_f, agg_hdivdof_l,
-                                          agg_l2dof_l, P_l2_l, Q_l2_l);
+                                          agg_l2dof_l, *P_l2_l, Q_l2_l);
       if (param_.coupled_solve)
       {
          auto S1 = new BlockDiagonalPreconditioner(ops_offsets_[l]);
          S1->SetDiagonalBlock(0, new AuxSpaceSmoother(M_f, C_l));
-         S1->owns_blocks = true;
-         smoothers_[l] = new ProductSolver(ops_[l], S0, S1, false, true, true);
+         S1->owns_blocks = 1;
+         smoothers_[l] =
+            std::make_unique<ProductSolver>(ops_[l].get(), S0, S1, false, true, true);
       }
       else
       {
-         smoothers_[l] = S0;
+         smoothers_[l].reset(S0);
       }
 
-      HypreParMatrix* M_c = TwoStepsRAP(P_hdiv_l, M_f, P_hdiv_l);
-      HypreParMatrix* B_c = TwoStepsRAP(P_l2_l, B_f, P_hdiv_l);
+      HypreParMatrix* M_c = TwoStepsRAP(P_hdiv_l, &M_f, P_hdiv_l);
+      HypreParMatrix* B_c = TwoStepsRAP(P_l2_l, &B_f, P_hdiv_l);
 
       ops_offsets_[l-1].SetSize(3, 0);
       ops_offsets_[l-1][1] = M_c->NumRows();
       ops_offsets_[l-1][2] = M_c->NumRows() + B_c->NumRows();
 
-      blk_Ps_[l-1] = new BlockOperator(ops_offsets_[l], ops_offsets_[l-1]);
-      blk_Ps_[l-1]->SetBlock(0, 0, &P_hdiv_l);
-      blk_Ps_[l-1]->SetBlock(1, 1, &P_l2_l);
+      blk_Ps_[l-1] =
+         std::make_unique<BlockOperator>(ops_offsets_[l], ops_offsets_[l-1]);
+      blk_Ps_[l-1]->SetBlock(0, 0, P_hdiv_l);
+      blk_Ps_[l-1]->SetBlock(1, 1, P_l2_l);
 
-      ops_[l-1] = new BlockOperator(ops_offsets_[l-1]);
+      ops_[l-1] =
+         std::make_unique<BlockOperator>(ops_offsets_[l-1]);
       ops_[l-1]->SetBlock(0, 0, M_c);
       ops_[l-1]->SetBlock(1, 0, B_c);
       ops_[l-1]->SetBlock(0, 1, B_c->Transpose());
-      ops_[l-1]->owns_blocks = true;
+      ops_[l-1]->owns_blocks = 1;
    }
 
-   Array<bool> own_ops(ops_.Size());
-   Array<bool> own_smoothers(smoothers_.Size());
-   Array<bool> own_Ps(blk_Ps_.Size());
-   own_ops = true;
-   own_smoothers = true;
-   own_Ps = true;
-
    if (data_.P_l2.size() == 0) { return; }
 
+   Array<bool> own_ops(ops_.size());
+   Array<bool> own_smoothers(smoothers_.size());
+   Array<bool> own_blk_Ps(blk_Ps_.size());
+   own_ops = false, own_smoothers = false, own_blk_Ps = false;
+
+   Array<Solver*> smoothers(smoothers_.size());
+
    if (param_.coupled_solve)
    {
       solver_.Reset(new GMRESSolver(B.GetComm()));
-      solver_.As<GMRESSolver>()->SetOperator(*(ops_.Last()));
-      prec_.Reset(new Multigrid(ops_, smoothers_, blk_Ps_,
-                                own_ops, own_smoothers, own_Ps));
+      solver_.As<GMRESSolver>()->SetOperator(*(ops_.back()));
+      Array<BlockOperator*> ops(ops_.size()), blk_Ps(blk_Ps_.size());
+      for (size_t i = 0; i < ops_.size(); ++i) { ops[i] = ops_[i].get(); }
+      for (size_t i = 0; i < blk_Ps_.size(); ++i) { blk_Ps[i] = blk_Ps_[i].get(); }
+      for (size_t i = 0; i < smoothers_.size(); ++i) { smoothers[i] = smoothers_[i].get(); }
+      prec_.Reset(new Multigrid(ops, smoothers, blk_Ps,
+                                own_ops, own_smoothers, own_blk_Ps));
    }
    else
    {
       Array<HypreParMatrix*> ops(data_.P_hcurl.size()+1);
-      Array<Solver*> smoothers(ops.Size());
       Array<HypreParMatrix*> Ps(data_.P_hcurl.size());
-      own_Ps = false;
-
-      HypreParMatrix& C_finest = *data.C.back().As<HypreParMatrix>();
-      ops.Last() = TwoStepsRAP(C_finest, M, C_finest);
+      auto C_finest = data.C.back().As<HypreParMatrix>();
+      ops.Last() = TwoStepsRAP(C_finest, &M, C_finest);
       ops.Last()->EliminateZeroRows();
       ops.Last()->DropSmallEntries(1e-14);
-
       solver_.Reset(new CGSolver(B.GetComm()));
       solver_.As<CGSolver>()->SetOperator(*ops.Last());
       smoothers.Last() = new HypreSmoother(*ops.Last());
       static_cast<HypreSmoother*>(smoothers.Last())->SetOperatorSymmetry(true);
-
       for (int l = Ps.Size()-1; l >= 0; --l)
       {
-         Ps[l] = data_.P_hcurl[l].As<HypreParMatrix>();
-         ops[l] = TwoStepsRAP(*Ps[l], *ops[l+1], *Ps[l]);
+         Ps[l] = data_.P_hcurl[l]->As<HypreParMatrix>();
+         ops[l] = TwoStepsRAP(Ps[l], ops[l+1], Ps[l]);
          ops[l]->DropSmallEntries(1e-14);
          smoothers[l] = new HypreSmoother(*ops[l]);
          static_cast<HypreSmoother*>(smoothers[l])->SetOperatorSymmetry(true);
       }
-
-      prec_.Reset(new Multigrid(ops, smoothers, Ps, own_ops, own_smoothers, own_Ps));
+      own_ops = true, own_smoothers = true;
+      prec_.Reset(new Multigrid(ops, smoothers, Ps,
+                                own_ops, own_smoothers, own_blk_Ps));
    }
 
    solver_.As<IterativeSolver>()->SetPreconditioner(*prec_.As<Solver>());
    SetOptions(*solver_.As<IterativeSolver>(), param_);
 }
 
-DivFreeSolver::~DivFreeSolver()
-{
-   if (param_.coupled_solve) { return; }
-   for (int i = 0; i < ops_.Size(); ++i)
-   {
-      delete ops_[i];
-      delete smoothers_[i];
-      if (i == ops_.Size() - 1) { break; }
-      delete blk_Ps_[i];
-   }
-}
-
 void DivFreeSolver::SolveParticular(const Vector& rhs, Vector& sol) const
 {
-   std::vector<Vector> rhss(smoothers_.Size());
-   std::vector<Vector> sols(smoothers_.Size());
-
+   std::vector<Vector> rhss(smoothers_.size()), sols(smoothers_.size());
    rhss.back().SetDataAndSize(const_cast<real_t*>(rhs.HostRead()), rhs.Size());
    sols.back().SetDataAndSize(sol.HostWrite(), sol.Size());
 
-   for (int l = blk_Ps_.Size()-1; l >= 0; --l)
+   for (int l = blk_Ps_.size()-1; l >= 0; --l)
    {
       rhss[l].SetSize(blk_Ps_[l]->NumCols());
       sols[l].SetSize(blk_Ps_[l]->NumCols());
@@ -470,12 +468,12 @@ void DivFreeSolver::SolveParticular(const Vector& rhs, Vector& sol) const
       blk_Ps_[l]->MultTranspose(rhss[l+1], rhss[l]);
    }
 
-   for (int l = 0; l < smoothers_.Size(); ++l)
+   for (size_t l = 0; l < smoothers_.size(); ++l)
    {
       smoothers_[l]->Mult(rhss[l], sols[l]);
    }
 
-   for (int l = 0; l < blk_Ps_.Size(); ++l)
+   for (size_t l = 0; l < blk_Ps_.size(); ++l)
    {
       Vector P_sol(blk_Ps_[l]->NumRows());
       blk_Ps_[l]->Mult(sols[l], P_sol);
@@ -507,12 +505,12 @@ void DivFreeSolver::Mult(const Vector & x, Vector & y) const
    MFEM_VERIFY(x.Size() == offsets_[2], "MLDivFreeSolver: x size is invalid");
    MFEM_VERIFY(y.Size() == offsets_[2], "MLDivFreeSolver: y size is invalid");
 
-   if (ops_.Size() == 1) { smoothers_[0]->Mult(x, y); return; }
+   if (ops_.size() == 1) { smoothers_[0]->Mult(x, y); return; }
 
    BlockVector blk_y(y, offsets_);
 
    BlockVector resid(offsets_);
-   ops_.Last()->Mult(y, resid);
+   ops_.back()->Mult(y, resid);
    add(1.0, x, -1.0, resid, resid);
 
    BlockVector correction(offsets_);
@@ -539,7 +537,7 @@ void DivFreeSolver::Mult(const Vector & x, Vector & y) const
       ch.Clear();
       ch.Start();
 
-      ops_.Last()->Mult(y, resid);
+      ops_.back()->Mult(y, resid);
       add(1.0, x, -1.0, resid, resid);
 
       SolveDivFree(resid.GetBlock(0), correction.GetBlock(0));
@@ -553,7 +551,7 @@ void DivFreeSolver::Mult(const Vector & x, Vector & y) const
       ch.Clear();
       ch.Start();
 
-      auto& M = dynamic_cast<const HypreParMatrix&>(ops_.Last()->GetBlock(0, 0));
+      auto& M = dynamic_cast<const HypreParMatrix&>(ops_.back()->GetBlock(0, 0));
       M.Mult(-1.0, correction.GetBlock(0), 1.0, resid.GetBlock(0));
       SolvePotential(resid.GetBlock(0), correction.GetBlock(1));
       blk_y.GetBlock(1) += correction.GetBlock(1);
@@ -567,11 +565,12 @@ void DivFreeSolver::Mult(const Vector & x, Vector & y) const
 
 int DivFreeSolver::GetNumIterations() const
 {
-   if (ops_.Size() == 1)
+   if (ops_.size() == 1)
    {
-      return static_cast<BDPMinresSolver*>(smoothers_[0])->GetNumIterations();
+      return static_cast<BDPMinresSolver*>
+             (smoothers_.at(0).get())->GetNumIterations();
    }
    return solver_.As<IterativeSolver>()->GetNumIterations();
 }
-} // namespace blocksolvers
-} // namespace mfem
+
+} // namespace mfem::blocksolvers
